










#         ciforth : a generic I86 ISO FORTH by HCC FIG
# $Id: ci86.gnr,v 5.190 2022/04/06 14:41:12 albert Exp $
# Copyright (2012): Albert van der Horst by GNU Public License
#
#HCC FIG Holland : Hobby Computer Club, Forth Interest Group  Holland
        #  66,106
 #   ciforth $Revision: 5.190 $
#
# For the generic system (to generate ciforth in an other configuration than this one):
#     http://home.hccnet.nl/a.w.m.van.der.horst/ci86gnr.html
#
# The following configuration information should agree with the configuration
# on the frontpage of your docmentation (texinfo, ps, pdf.)
# WITHOUT FITTING DOCUMENTATION YOU MIGHT AS WELL GIVE UP! 
#
# Configuration of this particular version:
# 32-bits protected mode 
# running under Linux  #  with native forth I/O 

# Normally ciforth doesn't observe ISO >IN.
# Contains :
# (there may be no items here.)
#        Security words
#         Loadable words, i.e. all of ISO CORE, more than is needed
#           for a self contained kernel.
# A field in the header to point to source
# An extra field in the header for double links or free use.
#

         # Assembler directive

#
# Gnu as version of ciforth created by ``m4'' from generic listing.
# This source can be assembled using versions better than 2.13.
# This file can be assembled and linked in the following way:
#       as -32 lina.s
#       ld -melf_i386   -s -N a.out -o lina
# The -s (strip) is not necessary, and may even be detrimental.

.Intel_syntax prefix

 /* 
        A generic version of ISO FORTH for IBM type standard PC's by
                Albert van der Horst

                in cooperation with
                HCC Forth user group
                The Netherlands
                www.forth.hccnet.nl

                with
        direct contributions for OSX from Robert Spykerman.

              based on
              FIG-FORTH
   implemented by:  Charlie Krajewski
                    205 ( BIG ) Blue Rd.
                    Middletown, CT  06457

  The listing has been made possible by the
  prior work of:
               Thomas Newman, Hayward, Ca.

 : other_acknowledgements
         John_Cassidy
         Kim_Harris
         George_Flammer
         Robert_D._Villwock ;

 : for tools
         Richard M. Stallman
         Linus Torvalds

No one who programs with FORTH can afford to be without:
  "Starting Forth  by Leo Brodie" and "Thinking Forth by Leo Brodie".
   Both out of print.

This Forth is a descendant in the 300+ (RCS)- generations from fig-Forth.

For nostalgic reasons the following comment has never been removed:
   Although there is much to be said for typing in your own
   listing and getting it running, there is much to be said
   not typing in your own listing.  If you feel that 100+
   pages of plinking is nutty, contact me for availability
   of a disc with source & executable files.  Obtainable at
   a bargain basement price, prepare yourself for bargain
   basement support.

All publications of the FORTH Interest Group are public domain.
They may be further distributed by the inclusion of this
credit notice:
               This publication has been made available by:

               FORTH Interest Group
               P.O. Box 1105
               San Carlos, Ca.  94070
[I feel obliged to keep this last one in (%AH). Note that although it is
based on fig-Forth no stone is left unturned.]
 */ 

        #
# ########################################################################################
#                       PREPARATION (no code)
# ########################################################################################

FIGREL  =     5       # FIG RELEASE #
FIGREV  =     0       # FIG REVISION #
USRVER  =     0      # USER VERSION NUMBER, a digit now
#
#      VERY ELEMENTARY .
CW      =     4    # Size of a cell in Forth, not in the bootcode.
ERRORSCREEN =     48    # Screen where the error messages start.
#
#      MEMORY LAYOUT.
# Normally this is specified at the m4 configuration level.
# For a configured system these values can be changed at this single place. 
NBUF    =     16   # No. of buffers, or screens 
KBBUF   =     1024      # Data bytes per disk buffer
US      =     0x40*CW  # User variable space, eats into next data stack, if any.
EM      =     (0x4000000)-0x74     # Where the memory ends w.r.t. ORIG. 1) 
EMP     =     (EM-1)/0x1000+1 # Number of pages.
TAS     =     0x10000    #Size of unitialised area per task.
STACKSIZE   = TAS/4  #For both stacks. 
TIBSIZE     = TAS/4  #TIB size. 
#
# NOTE 1:

#This trick to not have a round memory allocated convinces loaders
# on Linux 1.24 and following to load the elf header in memory
# in front of the executable, and not in some other place.
# ciforth relies on modifying the elf header in behalf of SAVE-SYSTEM.
#This is more convenient than creating it from scratch. It also leaves
# alone the load address generated by ``ld'', and all other things we
# are not aware of.



#
#      ASCII CHARACTER EQUIVALENTS
#
ABL     =     ' '     # SPACE
ACR     =     0x0D     # CR
AMS     =     '-'          # MINUS SIGN 
ASO     =     '['          # SQUARE BRACKET OPEN 
ASC     =     ']'          # SQUARE BRACKET CLOSE 
ADOT    =     '.'          # PERIOD
ALF      =     0x0A     # LINE FEED, USED INTERNALLY AS
                        # LINE ENDER
AFF      =     0x0C     # FORM FEED
BELL    =     0x07     # ^G
BSIN    =     0x08     # INPUT DELETE CHARACTER
BSOUT   =     0x08     # OUTPUT BACKSPACE ( ^H )
#
#      HEADER RELATED EQUATES
B_DUMMY   =     0x01     # dea is dummy, from namespace link
B_INVIS   =     0x02     # dea is invisible, "smudged".
B_IMMED   =     0x04     # dea is a immediate.
B_DENOT   =     0x08     # dea is a denotation.
C_HOFFSET =     0       # Offsets of code field in cells, w.r.t. dea
D_HOFFSET =     1       # Same for data field
F_HOFFSET =     2       # Same for flag field
L_HOFFSET =     3       # Same for link field
N_HOFFSET =     4       # Same for name field
S_HOFFSET =     5       # Same for source field
X_HOFFSET =     S_HOFFSET+1# Same for extra field
PH_OFFSET =     5+1+1   # Past header field: Start of data area. 
BD_OFFSET =     5+1+1+1 # Start of BODY for CREATEd word.
HEADSIZE  =     (CW*(PH_OFFSET))  #In cells, only to clean up source.
#
 #  
#
#
BPS     =     512             #Bytes/sector, common to all of MSDOS
SPB     =     KBBUF/BPS
#
#
#
#
#
# 


# ------------------------------------------------------------      
#   syscalls are copied from                                            
#https://github.com/torvalds/linux/blob/master/arch/x86/entry/syscalls/syscall_32.tbl

# ------------------------------------------------------------      

# https://github.com/torvalds/linux/tree/master/arch/x86/entry/syscalls
# you may benefit from the program stealconstant 
# https://github.com/albertvanderhorst/stealconstant 

SEEK_SET        =     0x0
SEEK_CUR	=	0x1
TCGETS  =     0x5401
TCSETS  =     0x5402
ECHO    =     0x8
EAGAIN  =     0xb
EINTR   =     0x4
EPIPE   =     0x20
VMIN    =     0x6
VTIME   =     0x5
ICANON  =     0x2
O_RDWR  =     0x2
O_RDONLY        =     0x0
O_WRONLY        =     0x1
O_CREAT =     0x40
O_NONBLOCK      =     0x800
CLONE_VM        =     0x100
SIZE_TERMIO     =     0x3c

# Numbers of system calls. See "Linux kernel Internals" Appendix A. 
# By M.Beck, H. Boehme e.a. Addison Wesley.                         
# The system calls themselves are extensively documented in chapter 
# 2 of the man pages, e.g. "man 2 exit"
exit    =     0x1
open    =     0x5
close   =     0x6
creat   =     0x8
unlink  =     0xa
chdir   =     0xc
read    =     0x3
select  =     0x52
_newselect      =     0x8e
write   =     0x4
ioctl   =     0x36
ioperm  =     0x65
iopl    =     0x6e
lseek   =     0x13
execve  =     0xb
fork    =     0x2
waitpid =     0x7
pipe    =     0x2a
clone   =     0x78
kill    =     0x25
signal  =     0x30
time    =     0xd
__NR_times      =     0x2b
# ------------------------------------------------------------      
#   End of constants stolen from C.                                 
# ------------------------------------------------------------      
 

RAWIO           =     (ECHO | ICANON)

# 
# 
# 


            .section .forth,"awx",@progbits
        #
        .global  _start       # Entry point's guesses.
        .global  start        # Entry point.
        .global  _main        # Entry point.
_ORIG:
_start:      # Entry point.
# 
# 

# 
    
    

# 

# 
# 
NOBOOT:         # Skip till here if not booting.

# 
ENDBOOT:


#
# 

# 
# 

# 
#
#

#


# 
#


# 4.15.3.2
_main:
start:
#
COLD_ENTRY:
        CLD                     # DIR = INC

        MOV     %EAX, DWORD PTR[USINI+(CW*(1))]
        MOV     %EBX, %EAX
        DEC     %EBX
        OR      %EBX, OFFSET STACKSIZE-1    #Requires it is power of 2.
        INC     %EBX
        SUB     %EBX,%EAX
        ADD     DWORD PTR[USINI+(CW*(0))], %EBX
        ADD     DWORD PTR[USINI+(CW*(1))], %EBX
        ADD     DWORD PTR[USINI+(CW*(2))], %EBX
        ADD     DWORD PTR[USINI+(CW*(3))], %EBX
        ADD     DWORD PTR[USINI+(CW*(4))], %EBX

#
       MOV     DWORD PTR[USINI+(CW*(31))],%ESP #Remember ARGS.
        MOV     %DX,%DS
        MOV     %SS,%DX           #Atomic with next instruction.
#
        MOV     %ESP,DWORD PTR[USINI+(CW*(2))]    #Parameter stack
        MOV     %EBP,DWORD PTR[USINI+(CW*(3))]    #Return stack
        MOV     %ESI,OFFSET COLD+(CW*(PH_OFFSET)) #Interpreter pointer
        LODSD                  # NEXT
        JMP     DWORD PTR[%EAX]   
#
#
# ########################################################################################
#                       FORTH ITSELF
# ########################################################################################
#
 /* 
   FORTH REGISTERS
   The names under FORTH are used in the generic source.

   FORTH   Intel    FORTH PRESERVATION RULES
   -----   ----     ----- ------------ -----
   HIP   %ESI      High level Interpreter Pointer.  Must be preserved
                    across FORTH words.

   WOR   %EAX      Working register.  When entering a word
                    via its code field the dea is passed in WOR.

   SPO   %ESP      Parameter stack pointer.  Must be preserved
                    across FORTH words.

   RPO   %EBP      Return stack pointer.  Must be preserved across
                    FORTH words.

            %EAX      General register.  Used to pass data from
                    FORTH words, see label APUSH or macro _APUSH

            %EDX      General register.  Used to pass more data from
                    FORTH words, see label DPUSH or macro _DPUSH

            %EBX      General purpose register.

            %ECX      General purpose register.

            %CS      Segment register. Must be preserved
                    across FORTH words.

            %DS      ditto

            %SS      ibid

            %ES      Temporary segment register only used by
                    a few words. However it MUST remain equal to
                    %DS, such that string primitives can be used
                    with impunity.
            The remaining registers are free to use.
            Within assembler words %EAX can be used too,
            as it has served its purpose.

            Here are some  useful code sequences.
            For next:
                    LODSD                  # NEXT
        JMP     DWORD PTR[%EAX]   
            For push single and next:
                    PUSH    %EAX
        LODSD                  # NEXT
        JMP     DWORD PTR[%EAX]   
            For push double and next:
                    PUSH    %EDX
        PUSH    %EAX
        LODSD                  # NEXT
        JMP     DWORD PTR[%EAX]   


----------------------------------------------------------
 */ 
        #
 /* 
---------------------------------------------

   COMMENT CONVENTIONS
   ------- -----------

   =       IS EQUAL TO
   <-      ASSIGNMENT

  NAME        =  Address of name
  (NAME)      =  Contents of name
  DEA         =  DICTIONARY ENTRY ADDRESS
                 fields have fixed offsets from DEA
  CFA         =  CODE FIELD ADDRESS : a pointer to executable code
  DFA         =  DATA FIELD ADDRESS : a pointer to
                        data/high level code/ DOES> pointer
  FFA         =  FLAG FIELD ADDRESS: contains flags
  LFA         =  LINK FIELD ADDRESS: a pointer to the next DEA
  NFA         =  NAME FIELD ADDRESS: a pointer to a name buffer
  SFA         =  SOURCE FIELD ADDRESS: a pointer into source code
  PHA         =  POST HEADER ADDRESS: Afterall fields

  S1          =  Parameter stack - 1st cell
  S2          =  Parameter stack - 2nd cell
  R1          =  Return stack    - 1st cell
  R2          =  Return stack    - 2nd cell

  LSB         =  Least significant bit
  MSB         =  Most  significant bit
  LB          =  Low byte
  HB          =  High byte
  LC          =  Low  cell
  HC          =  High cell

------------------------------------------------------------
 */ 
# 
        #
# 
#
# 
# In 32 bit versions there may be no jumps to NEXT at all 
# The label NEXT1 is rarely relevant (for _OLDDEBUG_) 
DPUSH:  PUSH    %EDX      # Fall through.
APUSH:  PUSH    %EAX
NEXT:
#
        LODSD           #AX <- (IP)
NEXT1:  MOV     %EAX,%EAX   # (WOR) <- (IP)

        JMP     DWORD PTR[%EAX]    # TO `CFA'
#
#       Dictionary starts here.

DP0:
# End of ONLY namespace. Namespaces all end in a link to 0.

#  *********
#  *   '   *
#  *********
#
        .balign    4,0x00
N_TICK:
        .long      1
        .ASCII      "'"
        .balign    4,0x00
TICK:
        .long    DOCOL
        .long    TICK+HEADSIZE
        .long    B_IMMED + B_DENOT
        .long    0
        .long    N_TICK
        .long    0
        .long    0

        .long      ITICK
        .long      LITER
        .long      SEMIS
#

#  *********
#  *   &   *
#  *********
#
        .balign    4,0x00
N_DCHAR:
        .long      1
        .ASCII      "&"
        .balign    4,0x00
DCHAR:
        .long    DOCOL
        .long    DCHAR+HEADSIZE
        .long    B_IMMED + B_DENOT
        .long    TICK
        .long    N_DCHAR
        .long    0
        .long    0

        .long      PPFET
        .long      NIP
        .long      LDUP, QBL
        .long      LIT, 10, QERR
        .long      LITER
        .long      QDELIM
        .long      SEMIS

#

#  *********
#  *   ^   *
#  *********
#
        .balign    4,0x00
N_DCTL:
        .long      1
        .ASCII      "^"
        .balign    4,0x00
DCTL:
        .long    DOCOL
        .long    DCTL+HEADSIZE
        .long    B_IMMED + B_DENOT
        .long    DCHAR
        .long    N_DCTL
        .long    0
        .long    0

        .long      PPFET
        .long      NIP
        .long      LDUP, QBL
        .long      LIT, 10, QERR
        .long      LIT, '@', LSUB
        .long      LITER
        .long      QDELIM
        .long      SEMIS
#

#  *********
#  *   0   *
#  *********
#
        .balign    4,0x00
N_DEN0:
        .long      1
        .ASCII      "0"
        .balign    4,0x00
DEN0:
        .long    DOCOL
        .long    LNUMB
        .long    B_IMMED + B_DENOT
        .long    DCTL
        .long    N_DEN0
        .long    0
        .long    0

#  *********
#  *   1   *
#  *********
#
        .balign    4,0x00
N_DEN1:
        .long      1
        .ASCII      "1"
        .balign    4,0x00
DEN1:
        .long    DOCOL
        .long    LNUMB
        .long    B_IMMED + B_DENOT
        .long    DEN0
        .long    N_DEN1
        .long    0
        .long    0

#  *********
#  *   2   *
#  *********
#
        .balign    4,0x00
N_DEN2:
        .long      1
        .ASCII      "2"
        .balign    4,0x00
DEN2:
        .long    DOCOL
        .long    LNUMB
        .long    B_IMMED + B_DENOT
        .long    DEN1
        .long    N_DEN2
        .long    0
        .long    0

#  *********
#  *   3   *
#  *********
#
        .balign    4,0x00
N_DEN3:
        .long      1
        .ASCII      "3"
        .balign    4,0x00
DEN3:
        .long    DOCOL
        .long    LNUMB
        .long    B_IMMED + B_DENOT
        .long    DEN2
        .long    N_DEN3
        .long    0
        .long    0

#  *********
#  *   4   *
#  *********
#
        .balign    4,0x00
N_DEN4:
        .long      1
        .ASCII      "4"
        .balign    4,0x00
DEN4:
        .long    DOCOL
        .long    LNUMB
        .long    B_IMMED + B_DENOT
        .long    DEN3
        .long    N_DEN4
        .long    0
        .long    0

#  *********
#  *   5   *
#  *********
#
        .balign    4,0x00
N_DEN5:
        .long      1
        .ASCII      "5"
        .balign    4,0x00
DEN5:
        .long    DOCOL
        .long    LNUMB
        .long    B_IMMED + B_DENOT
        .long    DEN4
        .long    N_DEN5
        .long    0
        .long    0

#  *********
#  *   6   *
#  *********
#
        .balign    4,0x00
N_DEN6:
        .long      1
        .ASCII      "6"
        .balign    4,0x00
DEN6:
        .long    DOCOL
        .long    LNUMB
        .long    B_IMMED + B_DENOT
        .long    DEN5
        .long    N_DEN6
        .long    0
        .long    0


#  *********
#  *   7   *
#  *********
#
        .balign    4,0x00
N_DEN7:
        .long      1
        .ASCII      "7"
        .balign    4,0x00
DEN7:
        .long    DOCOL
        .long    LNUMB
        .long    B_IMMED + B_DENOT
        .long    DEN6
        .long    N_DEN7
        .long    0
        .long    0

#  *********
#  *   8   *
#  *********
#
        .balign    4,0x00
N_DEN8:
        .long      1
        .ASCII      "8"
        .balign    4,0x00
DEN8:
        .long    DOCOL
        .long    LNUMB
        .long    B_IMMED + B_DENOT
        .long    DEN7
        .long    N_DEN8
        .long    0
        .long    0

#  *********
#  *   9   *
#  *********
#
        .balign    4,0x00
N_DEN9:
        .long      1
        .ASCII      "9"
        .balign    4,0x00
DEN9:
        .long    DOCOL
        .long    LNUMB
        .long    B_IMMED + B_DENOT
        .long    DEN8
        .long    N_DEN9
        .long    0
        .long    0

#  *********
#  *   A   *
#  *********
#
        .balign    4,0x00
N_DENA:
        .long      1
        .ASCII      "A"
        .balign    4,0x00
DENA:
        .long    DOCOL
        .long    LNUMB
        .long    B_IMMED + B_DENOT
        .long    DEN9
        .long    N_DENA
        .long    0
        .long    0

#  *********
#  *   B   *
#  *********
#
        .balign    4,0x00
N_DENB:
        .long      1
        .ASCII      "B"
        .balign    4,0x00
DENB:
        .long    DOCOL
        .long    LNUMB
        .long    B_IMMED + B_DENOT
        .long    DENA
        .long    N_DENB
        .long    0
        .long    0


#  *********
#  *   C   *
#  *********
#
        .balign    4,0x00
N_DENC:
        .long      1
        .ASCII      "C"
        .balign    4,0x00
DENC:
        .long    DOCOL
        .long    LNUMB
        .long    B_IMMED + B_DENOT
        .long    DENB
        .long    N_DENC
        .long    0
        .long    0

#  *********
#  *   D   *
#  *********
#
        .balign    4,0x00
N_DEND:
        .long      1
        .ASCII      "D"
        .balign    4,0x00
DEND:
        .long    DOCOL
        .long    LNUMB
        .long    B_IMMED + B_DENOT
        .long    DENC
        .long    N_DEND
        .long    0
        .long    0

#  *********
#  *   E   *
#  *********
#
        .balign    4,0x00
N_DENE:
        .long      1
        .ASCII      "E"
        .balign    4,0x00
DENE:
        .long    DOCOL
        .long    LNUMB
        .long    B_IMMED + B_DENOT
        .long    DEND
        .long    N_DENE
        .long    0
        .long    0

#  *********
#  *   F   *
#  *********
#
        .balign    4,0x00
N_DENF:
        .long      1
        .ASCII      "F"
        .balign    4,0x00
DENF:
        .long    DOCOL
        .long    LNUMB
        .long    B_IMMED + B_DENOT
        .long    DENE
        .long    N_DENF
        .long    0
        .long    0

#

#  *********
#  *   -   *
#  *********
#
        .balign    4,0x00
N_DENM:
        .long      1
        .ASCII      "-"
        .balign    4,0x00
DENM:
        .long    DOCOL
        .long    DENM+HEADSIZE
        .long    B_IMMED + B_DENOT
        .long    DENF
        .long    N_DENM
        .long    0
        .long    0

        .long      PNUMB, DNEGA, SDLITE
        .long      SEMIS
#

#  *********
#  *   +   *
#  *********
#
        .balign    4,0x00
N_DENP:
        .long      1
        .ASCII      "+"
        .balign    4,0x00
DENP:
        .long    DOCOL
        .long    DENP+HEADSIZE
        .long    B_IMMED + B_DENOT
        .long    DENM
        .long    N_DENP
        .long    0
        .long    0

        .long      PNUMB, SDLITE
        .long      SEMIS
#

#  *********
#  *   "   *
#  *********
#
        .balign    4,0x00
N_DENQ:
        .long      1
        .ASCII      "\""
        .balign    4,0x00
DENQ:
        .long    DOCOL
        .long    DENQ+HEADSIZE
        .long    B_IMMED + B_DENOT
        .long    DENP
        .long    N_DENQ
        .long    0
        .long    0

        .long      LIT, SKIP, COMMA        #  'SKIP , HERE >R 0 ,
        .long      HERE, TOR, ZERO, COMMA
DENQ1:
        .long      LIT, '"', PARSE         #           BEGIN &" PARSE
        .long      PPFET, LDUP, LIT, '"', EQUAL #           PP@@ DUP &" =
        .long      ZBRAN
        .long      DENQ2-.-CW                 #           WHILE
        .long      TDROP, ONEP             #           2DROP 1+ R@ $+!
        .long      LDUP, ALLOT, RR, SADD
        .long      BRAN
        .long      DENQ1-.-CW                  #           REPEAT
DENQ2:
        .long      QBL, ZEQU
        .long      LIT, 10, QERR           #           ?BLANK 0= 10 ?ERROR
        .long      DROP                    #                DROP 
        .long      LDUP, ALLOT, RR, SADD     #         DUP ALLOT R@ $+!
    #          0 HERE ! 
    .long LALIGN
        .long      FROMR, SFET, DLITE      #           R> $@ POSTPONE DLITERAL ;
        .long      SEMIS
#

# The FORTH word is the only non-denotation in the ONLY namespace.
#  *************
#  *   FORTH   *
#  *************
#
        .balign    4,0x00
N_FORTH:
        .long      5
        .ASCII      "FORTH"
        .balign    4,0x00
FORTH:
        .long    DODOE
        .long    FORTH+HEADSIZE
        .long    0x0
        .long    DENQ
        .long    N_FORTH
        .long    0
        .long    0

        .long      DOVOC
        .long      0       # END OF NAMESPACE LIST

        .balign    4,0x00

        .long    0x0
        .long    0
        .long    B_DUMMY
        .long    TASK
        .long    (ZERO+(CW*(D_HOFFSET)))
        .long    0
        .long    0

#
#
#

#  ************
#  *   CORE   *
#  ************
#
        .balign    4,0x00
N_CORE:
        .long      4
        .ASCII      "CORE"
        .balign    4,0x00
CORE:
        .long    DOCOL
        .long    CORE+HEADSIZE
        .long    0x0
        .long    0
        .long    N_CORE
        .long    0
        .long    0

        .long      ZERO    # Not (fully) present.
        .long      SEMIS
#

#  ***********
#  *   CPU   *
#  ***********
#
        .balign    4,0x00
N_LCPU:
        .long      3
        .ASCII      "CPU"
        .balign    4,0x00
LCPU:
        .long    DOCOL
        .long    LCPU+HEADSIZE
        .long    0x0
        .long    CORE
        .long    N_LCPU
        .long    0
        .long    0


            # 'AMDX86'
       .long      LIT, 0x0CD1856, ZERO      # '80386'
           # '80386'
# 

        .long      SEMIS
#
#       If M4_VERSION exists and contains a . it is an official release
#

#  ***************
#  *   VERSION   *
#  ***************
#
        .balign    4,0x00
N_LVERSION:
        .long      7
        .ASCII      "VERSION"
        .balign    4,0x00
LVERSION:
        .long    DOCOL
        .long    LVERSION+HEADSIZE
        .long    0x0
        .long    LCPU
        .long    N_LVERSION
        .long    0
        .long    0

 /* 
#       If this is there it is an official release
        .long      SKIP
        .long      22
IBMPC:  .byte      'IBM-PC ciforth'
        .byte      FIGREL+0x40,ADOT,FIGREV+0x30,ADOT,USRVER+0x30
        .long      LIT, IBMPC, LIT, 22
 */ 
        .long      SKIP
         .long      5
SB0: .ASCII      "5.4.1"
       .balign    4,0x00
        .long      LIT, SB0
        .long      LIT, 5
        .long      SEMIS
#

#  ************
#  *   NAME   *
#  ************
#
        .balign    4,0x00
N_LENAME:
        .long      4
        .ASCII      "NAME"
        .balign    4,0x00
LENAME:
        .long    DOCOL
        .long    LENAME+HEADSIZE
        .long    0x0
        .long    LVERSION
        .long    N_LENAME
        .long    0
        .long    0

        .long      SKIP
         .long      7
SB1: .ASCII      "ciforth"
       .balign    4,0x00
        .long      LIT, SB1
        .long      LIT, 7
        .long      SEMIS
#

#  ****************
#  *   SUPPLIER   *
#  ****************
#
        .balign    4,0x00
N_SUPPLIER:
        .long      8
        .ASCII      "SUPPLIER"
        .balign    4,0x00
SUPPLIER:
        .long    DOCOL
        .long    SUPPLIER+HEADSIZE
        .long    0x0
        .long    LENAME
        .long    N_SUPPLIER
        .long    0
        .long    0

        .long      SKIP
         .long      20
SB2: .ASCII      "Albert van der Horst"
       .balign    4,0x00
        .long      LIT, SB2
        .long      LIT, 20
        .long      SEMIS
#
#

#  ************
#  *   ONLY   *
#  ************
#
        .balign    4,0x00
N_ONLY:
        .long      4
        .ASCII      "ONLY"
        .balign    4,0x00
ONLY:
        .long    DODOE
        .long    ONLY+HEADSIZE
        .long    0x0
        .long    0
        .long    N_ONLY
        .long    0
        .long    0

        .long      DOVOC
        .long      FORTH     # NEXT NAMESPACE 
ONLYBODY:

        .balign    4,0x00

        .long    0x0
        .long    0
        .long    B_DUMMY
        .long    FORTH
        .long    (ZERO+(CW*(D_HOFFSET)))
        .long    0
        .long    0

#

#  *******************
#  *   ENVIRONMENT   *
#  *******************
#
        .balign    4,0x00
N_ENV:
        .long      11
        .ASCII      "ENVIRONMENT"
        .balign    4,0x00
ENV:
        .long    DODOE
        .long    ENV+HEADSIZE
        .long    0x0
        .long    ONLY
        .long    N_ENV
        .long    0
        .long    0

        .long      DOVOC
        .long      ONLY       # NEXT NAMESPACE 

        .balign    4,0x00

        .long    0x0
        .long    0
        .long    B_DUMMY
        .long    SUPPLIER
        .long    (ZERO+(CW*(D_HOFFSET)))
        .long    0
        .long    0

#

#  ************
#  *   NOOP   *
#  ************
#
        .balign    4,0x00
N_NOOP:
        .long      4
        .ASCII      "NOOP"
        .balign    4,0x00
NOOP:
        .long    NOOP+HEADSIZE
        .long    NOOP+HEADSIZE
        .long    0x0
        .long    ENV
        .long    N_NOOP
        .long    0
        .long    0

       LODSD                  # NEXT
        JMP     DWORD PTR[%EAX]   
#
# 
#

#  ***********
#  *   LIT   *
#  ***********
#
        .balign    4,0x00
N_LIT:
        .long      3
        .ASCII      "LIT"
        .balign    4,0x00
LIT:
        .long    LIT+HEADSIZE
        .long    LIT+HEADSIZE
        .long    0x0
        .long    NOOP
        .long    N_LIT
        .long    0
        .long    0

        LODSD           # AX <- LITERAL
        PUSH    %EAX
        LODSD                  # NEXT
        JMP     DWORD PTR[%EAX]             # TO TOP OF STACK
#

#  ***************
#  *   EXECUTE   *
#  ***************
#
        .balign    4,0x00
N_EXEC:
        .long      7
        .ASCII      "EXECUTE"
        .balign    4,0x00
EXEC:
        .long    EXEC+HEADSIZE
        .long    EXEC+HEADSIZE
        .long    0x0
        .long    LIT
        .long    N_EXEC
        .long    0
        .long    0

        POP     %EAX      # GET XT
        JMP     DWORD PTR[%EAX]  #(IP) <- (PFA)
#


#  ***************
#  *   RECURSE   *
#  ***************
#
        .balign    4,0x00
N_RECURSE:
        .long      7
        .ASCII      "RECURSE"
        .balign    4,0x00
RECURSE:
        .long    DOCOL
        .long    RECURSE+HEADSIZE
        .long    B_IMMED
        .long    EXEC
        .long    N_RECURSE
        .long    0
        .long    0

        .long      LATEST, COMMA
        .long      SEMIS
#
#

#  **************
#  *   BRANCH   *
#  **************
#
        .balign    4,0x00
N_BRAN:
        .long      6
        .ASCII      "BRANCH"
        .balign    4,0x00
BRAN:
        .long    (SKIP+HEADSIZE)
        .long    BRAN+HEADSIZE
        .long    0x0
        .long    RECURSE
        .long    N_BRAN
        .long    0
        .long    0

#

#  ************
#  *   SKIP   *
#  ************
#
        .balign    4,0x00
N_SKIP:
        .long      4
        .ASCII      "SKIP"
        .balign    4,0x00
SKIP:
        .long    SKIP+HEADSIZE
        .long    SKIP+HEADSIZE
        .long    0x0
        .long    BRAN
        .long    N_SKIP
        .long    0
        .long    0

BRAN1:  LODSD

        DEC     %EAX
        OR      %AL, OFFSET 4 - 1
        INC     %EAX
#
        ADD     %ESI,%EAX
        LODSD                  # NEXT
        JMP     DWORD PTR[%EAX]   
#

#  ***************
#  *   0BRANCH   *
#  ***************
#
        .balign    4,0x00
N_ZBRAN:
        .long      7
        .ASCII      "0BRANCH"
        .balign    4,0x00
ZBRAN:
        .long    ZBRAN+HEADSIZE
        .long    ZBRAN+HEADSIZE
        .long    0x0
        .long    SKIP
        .long    N_ZBRAN
        .long    0
        .long    0

        POP     %EAX      # GET STACK VALUE
        OR      %EAX,%EAX   # ZERO?
        JZ      BRAN1   # YES, BRANCH
        LEA     %ESI,[%ESI+(CW*(1))]
        LODSD                  # NEXT
        JMP     DWORD PTR[%EAX]   
#

#  ***************
#  *   (+LOOP)   *
#  ***************
#
        .balign    4,0x00
N_XPLOOP:
        .long      7
        .ASCII      "(+LOOP)"
        .balign    4,0x00
XPLOOP:
        .long    XPLOOP+HEADSIZE
        .long    XPLOOP+HEADSIZE
        .long    0x0
        .long    ZBRAN
        .long    N_XPLOOP
        .long    0
        .long    0

        POP     %EBX      # GET INCREMENT
        ADD     [%EBP],%EBX # INDEX = INDEX + INCR
        MOV     %ECX,[%EBP] # GET NEW INDEX
        SUB     %ECX,[%EBP+(CW*(1))]        # COMPARE WITH LIMIT
        XOR     %ECX,%EBX   # compare sign
        SETNS    %AL      #different signs?
        MOVZX   %EAX,%AL
        NEG     %EAX
        PUSH    %EAX
        LODSD                  # NEXT
        JMP     DWORD PTR[%EAX]   

#

#  ************
#  *   (DO)   *
#  ************
#
        .balign    4,0x00
N_XDO:
        .long      4
        .ASCII      "(DO)"
        .balign    4,0x00
XDO:
        .long    XDO+HEADSIZE
        .long    XDO+HEADSIZE
        .long    0x0
        .long    XPLOOP
        .long    N_XDO
        .long    0
        .long    0

        LODSD
        ADD     %EAX,%ESI  #Make absolute
        POP     %EDX      # INITIAL INDEX VALUE
        POP     %EBX      # LIMIT VALUE
        XCHG    %EBP,%ESP   # GET RETURN STACK
        PUSH    %EAX      # Target location.
        PUSH    %EBX
        PUSH    %EDX
        XCHG    %EBP,%ESP   # GET PARAMETER STACK
        LODSD                  # NEXT
        JMP     DWORD PTR[%EAX]   
#

#  *************
#  *   (?DO)   *
#  *************
#
        .balign    4,0x00
N_XQDO:
        .long      5
        .ASCII      "(?DO)"
        .balign    4,0x00
XQDO:
        .long    XQDO+HEADSIZE
        .long    XQDO+HEADSIZE
        .long    0x0
        .long    XDO
        .long    N_XQDO
        .long    0
        .long    0

        LODSD
        ADD     %EAX,%ESI  #Make absolute
        POP     %EDX      # INITIAL INDEX VALUE
        POP     %EBX      # LIMIT VALUE
        CMP     %EDX,%EBX
        JGE     QXDO1
        XCHG    %EBP,%ESP   # GET RETURN STACK
        PUSH    %EAX      # Target location.
        PUSH    %EBX
        PUSH    %EDX
        XCHG    %EBP,%ESP   # GET PARAMETER STACK
        LODSD                  # NEXT
        JMP     DWORD PTR[%EAX]   
QXDO1:  MOV     %ESI,%EAX
        LODSD                  # NEXT
        JMP     DWORD PTR[%EAX]   
#

#  *********
#  *   I   *
#  *********
#
        .balign    4,0x00
N_IDO:
        .long      1
        .ASCII      "I"
        .balign    4,0x00
IDO:
        .long    IDO+HEADSIZE
        .long    IDO+HEADSIZE
        .long    0x0
        .long    XQDO
        .long    N_IDO
        .long    0
        .long    0

        MOV     %EAX,[%EBP] # GET INDEX VALUE
        PUSH    %EAX
        LODSD                  # NEXT
        JMP     DWORD PTR[%EAX]             # TO PARAMETER STACK
#

#  *********
#  *   J   *
#  *********
#
        .balign    4,0x00
N_JDO:
        .long      1
        .ASCII      "J"
        .balign    4,0x00
JDO:
        .long    JDO+HEADSIZE
        .long    JDO+HEADSIZE
        .long    0x0
        .long    IDO
        .long    N_JDO
        .long    0
        .long    0

        MOV     %EAX,[%EBP+(CW*(3))] # GET INDEX VALUE
        PUSH    %EAX
        LODSD                  # NEXT
        JMP     DWORD PTR[%EAX]             # TO PARAMETER STACK
#

#  **************
#  *   UNLOOP   *
#  **************
#
        .balign    4,0x00
N_UNLOOP:
        .long      6
        .ASCII      "UNLOOP"
        .balign    4,0x00
UNLOOP:
        .long    UNLOOP+HEADSIZE
        .long    UNLOOP+HEADSIZE
        .long    0x0
        .long    JDO
        .long    N_UNLOOP
        .long    0
        .long    0

        LEA     %EBP,[%EBP+(CW*(3))]
        LODSD                  # NEXT
        JMP     DWORD PTR[%EAX]   
#

#  ***************
#  *   +ORIGIN   *
#  ***************
#
        .balign    4,0x00
N_PORIG:
        .long      7
        .ASCII      "+ORIGIN"
        .balign    4,0x00
PORIG:
        .long    DOCOL
        .long    PORIG+HEADSIZE
        .long    0x0
        .long    UNLOOP
        .long    N_PORIG
        .long    0
        .long    0

        .long      LIT
        .long      USINI
        .long      PLUS
        .long      SEMIS
#
#      Initialisation block for user variables through DOC-LINK
#       <<<<< must be in same order as user variables >>>>>
# Still fig-compatible! Both entries take two cells.
#
        LEA     %EAX,[COLD]
        JMP     DWORD PTR[%EAX]          #Hope stacks are still okay.
        .balign    4,0x00
# Make it possible to revector WARM. You can even make it low level. 
        LEA     %EAX,[WARM]
        JMP     DWORD PTR[%EAX]          #Hope stacks are still okay.
        .balign    4,0x00
USINI:  .long      STRUSA   # user area, copied before use in this threading Forth.#
        .long      STRUSA  # INIT (U0) user area of the main task 1
        .long      INITS0  # INIT (S0)         2
# %RSP@ must not be divisible by TASK-SIZE, to serve as user pointer.
        .long      INITR0   - (CW*(1))
                        # INIT (R0)         3
        .long      STRTIB  # INIT (TIB)        4
        .long      BSIN    # RUBOUT: get rid of latest char 5
        .long      0       # AVAILABLE         6
        .long      1       # INIT (WARNING)     7
        .long      INITDP  #      INIT (FENCE)  8
DPA:    .long      INITDP  #      INIT (DP)     9
        .long      ENV #       INIT (VOC-LINK) 10
        .long      39+16  # INIT (MAX-USER) 11
#
#
#
#
        .long      0, 0            # WHERE             12 13 
        .long      0, 0            #REMAINDER   14 15 
# 
        .SPACE   US-(. - USINI)        # All user can be initialised.
#
#      <<<<< end of data used by cold start >>>>>

#  *************
#  *   DIGIT   *
#  *************
#
        .balign    4,0x00
N_DIGIT:
        .long      5
        .ASCII      "DIGIT"
        .balign    4,0x00
DIGIT:
        .long    DIGIT+HEADSIZE
        .long    DIGIT+HEADSIZE
        .long    0x0
        .long    PORIG
        .long    N_DIGIT
        .long    0
        .long    0

        POP     %EDX      #NUMBER BASE
        POP     %EAX      #ASCII DIGIT
        SUB     %AL,OFFSET '0'
        JB      DIGI2   #NUMBER ERROR
        CMP     %AL,OFFSET 9
        JBE     DIGI1   #NUMBER = 0 THRU 9
        SUB     %AL,OFFSET 7
        CMP     %AL,OFFSET 10   #NUMBER 'A' THRU 'Z'?
        JB      DIGI2   #NO
DIGI1:  CMP     %AL,%DL   # COMPARE NUMBER TO BASE
        JAE     DIGI2   #NUMBER ERROR
        SUB     %EDX,%EDX   #ZERO
        MOV     %DL,%AL   #NEW BINARY NUMBER
        MOV     %AL,OFFSET 1    #TRUE FLAG
        NEG     %EAX
        JMP     DIGI3
#   NUMBER ERROR
DIGI2:  SUB     %EAX,%EAX   #FALSE FLAG
DIGI3:
        PUSH    %EDX
        PUSH    %EAX
        LODSD                  # NEXT
        JMP     DWORD PTR[%EAX]   
#

#  **************
#  *   ~MATCH   *
#  **************
#
        .balign    4,0x00
N_NMATCH:
        .long      6
        .ASCII      "~MATCH"
        .balign    4,0x00
NMATCH:
        .long    DOCOL
        .long    NMATCH+HEADSIZE
        .long    0x0
        .long    DIGIT
        .long    N_NMATCH
        .long    0
        .long    0

#
        .long      TOR                           # Remember dea
        .long      OVER
        .long      RR, TNFA, FETCH, SFET
        .long      CORA
        .long      FROMR, SWAP                   # Leave flag on top of dea.
        .long      SEMIS                            #
#

#  **************
#  *   ?BLANK   *
#  **************
#
        .balign    4,0x00
N_QBL:
        .long      6
        .ASCII      "?BLANK"
        .balign    4,0x00
QBL:
        .long    DOCOL
        .long    QBL+HEADSIZE
        .long    0x0
        .long    NMATCH
        .long    N_QBL
        .long    0
        .long    0

        .long      LBL, ONEP, LESS
        .long      SEMIS
#

#  ************
#  *   PP@@   *
#  ************
#
        .balign    4,0x00
N_PPFET:
        .long      4
        .ASCII      "PP@@"
        .balign    4,0x00
PPFET:
        .long    DOCOL
        .long    PPFET+HEADSIZE
        .long    0x0
        .long    QBL
        .long    N_PPFET
        .long    0
        .long    0

        .long      SRC, CELLP, TFET
        .long      OVER, EQUAL
        .long      ZBRAN
        .long      PPFET1-.-CW
        .long      ZERO
        .long      BRAN
        .long      PPFET2-.-CW
PPFET1:
        .long      LDUP
        
        .long      CFET
        .long      ONE, PP, PSTOR
PPFET2:
        .long      SEMIS
#

#  ************
#  *   NAME   *
#  ************
#
        .balign    4,0x00
N_LNAME:
        .long      4
        .ASCII      "NAME"
        .balign    4,0x00
LNAME:
        .long    DOCOL
        .long    LNAME+HEADSIZE
        .long    0x0
        .long    PPFET
        .long    N_LNAME
        .long    0
        .long    0

        .long      X
PWORD0: .long      DROP
        .long      PPFET, QBL
        .long      OVER, SRC, CELLP, FETCH, LSUB # At end?
        .long      LAND, ZEQU
        .long      ZBRAN
        .long      PWORD0-.-CW

        .long      X
PWORD1: .long      DROP
        .long      PPFET, QBL
        .long      ZBRAN
        .long      PWORD1-.-CW

        .long      OVER, LSUB
        
        .long      SEMIS
#

#  *************
#  *   PARSE   *
#  *************
#
        .balign    4,0x00
N_PARSE:
        .long      5
        .ASCII      "PARSE"
        .balign    4,0x00
PARSE:
        .long    DOCOL
        .long    PARSE+HEADSIZE
        .long    0x0
        .long    LNAME
        .long    N_PARSE
        .long    0
        .long    0

        .long      SRC, CELLP, TFET
        .long      OVER, LSUB
        
        .long      ROT, SSLAS, TSWAP
        .long      ZEQU
        .long      ZBRAN
        .long      PARSE8-.-CW
        .long      DROP, SRC, CELLP, FETCH
PARSE8: 
        .long      PP, STOR
        .long SEMIS
#

#  ***********
#  *   SRC   *
#  ***********
#
        .balign    4,0x00
N_SRC:
        .long      3
        .ASCII      "SRC"
        .balign    4,0x00
SRC:
        .long    DOUSE
        .long    (CW*(27))
        .long    0x0
        .long    PARSE
        .long    N_SRC
        .long    0
        .long    0
      # And 28 and 29.
#


#  **************
#  *   SOURCE   *
#  **************
#
        .balign    4,0x00
N_SOURCE:
        .long      6
        .ASCII      "SOURCE"
        .balign    4,0x00
SOURCE:
        .long    DOCOL
        .long    SOURCE+HEADSIZE
        .long    0x0
        .long    SRC
        .long    N_SOURCE
        .long    0
        .long    0

        .long      SRC, TFET
        .long      SWAP, OVER, LSUB
        .long      SEMIS
#
# 
#

#  **********
#  *   CR   *
#  **********
#
        .balign    4,0x00
N_CR:
        .long      2
        .ASCII      "CR"
        .balign    4,0x00
CR:
        .long    DOCOL
        .long    CR+HEADSIZE
        .long    0x0
        .long    SOURCE
        .long    N_CR
        .long    0
        .long    0

        .long      LIT,ALF
        .long      EMIT
        .long      ZERO, LOUT, STOR
        .long      SEMIS
#

#  *************
#  *   CMOVE   *
#  *************
#
        .balign    4,0x00
N_LCMOVE:
        .long      5
        .ASCII      "CMOVE"
        .balign    4,0x00
LCMOVE:
        .long    LCMOVE+HEADSIZE
        .long    LCMOVE+HEADSIZE
        .long    0x0
        .long    CR
        .long    N_LCMOVE
        .long    0
        .long    0

        CLD             #direction
        MOV     %EBX,%ESI   #save 
        POP     %ECX      #count
        POP     %EDI      #dest
        POP     %ESI      #source
        REP     MOVSB
        MOV     %ESI,%EBX   #get back 
        LODSD                  # NEXT
        JMP     DWORD PTR[%EAX]   
#

#  ************
#  *   MOVE   *
#  ************
#
        .balign    4,0x00
N_LMOVE:
        .long      4
        .ASCII      "MOVE"
        .balign    4,0x00
LMOVE:
        .long    LMOVE+HEADSIZE
        .long    LMOVE+HEADSIZE
        .long    0x0
        .long    LCMOVE
        .long    N_LMOVE
        .long    0
        .long    0

        MOV     %EBX,%ESI   #SAVE 
        POP     %ECX      #count
        POP     %EDI      #dest
        POP     %ESI      #source
        CMP     %ESI,%EDI
        JC    MOVE1
        CLD             #INC DIRECTION
        JMP SHORT MOVE2
MOVE1:  STD
        ADD     %EDI,%ECX
        DEC     %EDI
        ADD     %ESI,%ECX
        DEC     %ESI
MOVE2:
        REP     MOVSB   #THAT'S THE MOVE
        CLD             #INC DIRECTION
        MOV     %ESI,%EBX   #GET BACK 
        LODSD                  # NEXT
        JMP     DWORD PTR[%EAX]   
#
  
#  ***************
#  *   FARMOVE   *
#  ***************
#
        .balign    4,0x00
N_FMOVE:
        .long      7
        .ASCII      "FARMOVE"
        .balign    4,0x00
FMOVE:
        .long    FMOVE+HEADSIZE
        .long    FMOVE+HEADSIZE
        .long    0x0
        .long    LMOVE
        .long    N_FMOVE
        .long    0
        .long    0

        CLD             #direction
        MOV     %EAX,%ESI   #save 
        MOV     %EBX,%DS    #save 
        POP     %ECX      #count
        POP     %EDI      #dest
        POP     %EDX
        AND     %DX,%DX
        JZ      FARMV1
        MOV     %ES,%DX
FARMV1:
        POP     %ESI      #source
        POP     %EDX
        AND     %DX,%DX
        JZ      FARMV2
        MOV     %DS,%DX
FARMV2:
        REP     MOVSB
        MOV     %ESI,%EAX   #restore 
        PUSH    %EBX
        PUSH    %EBX
        POP     %ES
        POP     %DS
        LODSD                  # NEXT
        JMP     DWORD PTR[%EAX]   
#
#

#  ***********
#  *   UM*   *
#  ***********
#
        .balign    4,0x00
N_USTAR:
        .long      3
        .ASCII      "UM*"
        .balign    4,0x00
USTAR:
        .long    USTAR+HEADSIZE
        .long    USTAR+HEADSIZE
        .long    0x0
        .long    FMOVE
        .long    N_USTAR
        .long    0
        .long    0

        POP     %EAX
        POP     %EBX
        MUL     %EBX      #UNSIGNED
        XCHG    %EAX,%EDX   #AX NOW = MSW
        PUSH    %EDX
        PUSH    %EAX
        LODSD                  # NEXT
        JMP     DWORD PTR[%EAX]             #STORE DOUBLE CELL
#

#  **************
#  *   UM/MOD   *
#  **************
#
        .balign    4,0x00
N_USLAS:
        .long      6
        .ASCII      "UM/MOD"
        .balign    4,0x00
USLAS:
        .long    USLAS+HEADSIZE
        .long    USLAS+HEADSIZE
        .long    0x0
        .long    USTAR
        .long    N_USLAS
        .long    0
        .long    0

        POP     %EBX      #DIVISOR
        POP     %EDX      #MSW OF DIVIDEND
        POP     %EAX      #LSW OF DIVIDEND
        DIV     %EBX      #16 BIT DIVIDE
        PUSH    %EDX
        PUSH    %EAX
        LODSD                  # NEXT
        JMP     DWORD PTR[%EAX]             #STORE QUOT/REM
#

#  ***********
#  *   AND   *
#  ***********
#
        .balign    4,0x00
N_LAND:
        .long      3
        .ASCII      "AND"
        .balign    4,0x00
LAND:
        .long    LAND+HEADSIZE
        .long    LAND+HEADSIZE
        .long    0x0
        .long    USLAS
        .long    N_LAND
        .long    0
        .long    0

        POP     %EAX
        POP     %EBX
        AND     %EAX,%EBX
        PUSH    %EAX
        LODSD                  # NEXT
        JMP     DWORD PTR[%EAX]   
#

#  **********
#  *   OR   *
#  **********
#
        .balign    4,0x00
N_LOR:
        .long      2
        .ASCII      "OR"
        .balign    4,0x00
LOR:
        .long    LOR+HEADSIZE
        .long    LOR+HEADSIZE
        .long    0x0
        .long    LAND
        .long    N_LOR
        .long    0
        .long    0

        POP     %EAX      # (S1) <- (S1) OR (S2)
        POP     %EBX
        OR      %EAX,%EBX
        PUSH    %EAX
        LODSD                  # NEXT
        JMP     DWORD PTR[%EAX]   
#

#  ***********
#  *   XOR   *
#  ***********
#
        .balign    4,0x00
N_LXOR:
        .long      3
        .ASCII      "XOR"
        .balign    4,0x00
LXOR:
        .long    LXOR+HEADSIZE
        .long    LXOR+HEADSIZE
        .long    0x0
        .long    LOR
        .long    N_LXOR
        .long    0
        .long    0

        POP     %EAX      # (S1) <- (S1) XOR (S2)
        POP     %EBX
        XOR     %EAX,%EBX
        PUSH    %EAX
        LODSD                  # NEXT
        JMP     DWORD PTR[%EAX]   
#

#  **************
#  *   INVERT   *
#  **************
#
        .balign    4,0x00
N_INVERT:
        .long      6
        .ASCII      "INVERT"
        .balign    4,0x00
INVERT:
        .long    INVERT+HEADSIZE
        .long    INVERT+HEADSIZE
        .long    0x0
        .long    LXOR
        .long    N_INVERT
        .long    0
        .long    0

        POP     %EAX      # (S1) <- (S1) XOR (S2)
        NOT     %EAX
        PUSH    %EAX
        LODSD                  # NEXT
        JMP     DWORD PTR[%EAX]   
#

#  ************
#  *   DSP@   *
#  ************
#
        .balign    4,0x00
N_SPFET:
        .long      4
        .ASCII      "DSP@"
        .balign    4,0x00
SPFET:
        .long    SPFET+HEADSIZE
        .long    SPFET+HEADSIZE
        .long    0x0
        .long    INVERT
        .long    N_SPFET
        .long    0
        .long    0

        MOV     %EAX,%ESP   # (S1) <- (SP)
        PUSH    %EAX
        LODSD                  # NEXT
        JMP     DWORD PTR[%EAX]   
#

#  ************
#  *   DSP!   *
#  ************
#
        .balign    4,0x00
N_SPSTO:
        .long      4
        .ASCII      "DSP!"
        .balign    4,0x00
SPSTO:
        .long    SPSTO+HEADSIZE
        .long    SPSTO+HEADSIZE
        .long    0x0
        .long    SPFET
        .long    N_SPSTO
        .long    0
        .long    0

        POP     %EAX
        MOV     %ESP,%EAX        #RESET PARAM STACK POINTER
        LODSD                  # NEXT
        JMP     DWORD PTR[%EAX]   
#

#  *************
#  *   DEPTH   *
#  *************
#
        .balign    4,0x00
N_DEPTH:
        .long      5
        .ASCII      "DEPTH"
        .balign    4,0x00
DEPTH:
        .long    DOCOL
        .long    DEPTH+HEADSIZE
        .long    0x0
        .long    SPSTO
        .long    N_DEPTH
        .long    0
        .long    0

        .long      SZERO, FETCH
        .long      SPFET
        .long      LSUB
        .long      LIT, CW, SLASH
        .long      ONEM
        .long      SEMIS
#

#  ***********
#  *   CLS   *
#  ***********
#
        .balign    4,0x00
N_CLS:
        .long      3
        .ASCII      "CLS"
        .balign    4,0x00
CLS:
        .long    DOCOL
        .long    CLS+HEADSIZE
        .long    0x0
        .long    DEPTH
        .long    N_CLS
        .long    0
        .long    0

        .long      SZERO, FETCH
        .long      SPSTO
        .long      SEMIS
#

#  ************
#  *   RSP@   *
#  ************
#
        .balign    4,0x00
N_RPFET:
        .long      4
        .ASCII      "RSP@"
        .balign    4,0x00
RPFET:
        .long    RPFET+HEADSIZE
        .long    RPFET+HEADSIZE
        .long    0x0
        .long    CLS
        .long    N_RPFET
        .long    0
        .long    0
      #(S1) <- (RP)
        PUSH    %EBP
        LODSD                  # NEXT
        JMP     DWORD PTR[%EAX]   
#

#  ************
#  *   RSP!   *
#  ************
#
        .balign    4,0x00
N_RPSTO:
        .long      4
        .ASCII      "RSP!"
        .balign    4,0x00
RPSTO:
        .long    RPSTO+HEADSIZE
        .long    RPSTO+HEADSIZE
        .long    0x0
        .long    RPFET
        .long    N_RPSTO
        .long    0
        .long    0

        POP     %EBP
        LODSD                  # NEXT
        JMP     DWORD PTR[%EAX]   
#

#  ************
#  *   EXIT   *
#  ************
#
        .balign    4,0x00
N_EXIT:
        .long      4
        .ASCII      "EXIT"
        .balign    4,0x00
EXIT:
        .long    EXIT+HEADSIZE
        .long    EXIT+HEADSIZE
        .long    0x0
        .long    RPSTO
        .long    N_EXIT
        .long    0
        .long    0

        MOV     %ESI,[%EBP] #(IP) <- (R1)
        LEA     %EBP,[%EBP+(CW*(1))]
        LODSD                  # NEXT
        JMP     DWORD PTR[%EAX]   
#

#  **********
#  *   CO   *
#  **********
#
        .balign    4,0x00
N_CO:
        .long      2
        .ASCII      "CO"
        .balign    4,0x00
CO:
        .long    CO+HEADSIZE
        .long    CO+HEADSIZE
        .long    0x0
        .long    EXIT
        .long    N_CO
        .long    0
        .long    0

        XCHG    %ESI,[%EBP]
        LODSD                  # NEXT
        JMP     DWORD PTR[%EAX]   
#

#  ***********
#  *   (;)   *
#  ***********
#
        .balign    4,0x00
N_SEMIS:
        .long      3
        .ASCII      "(;)"
        .balign    4,0x00
SEMIS:
        .long    (EXIT+HEADSIZE)
        .long    SEMIS+HEADSIZE
        .long    0x0
        .long    CO
        .long    N_SEMIS
        .long    0
        .long    0

#

#  *************
#  *   LEAVE   *
#  *************
#
        .balign    4,0x00
N_LLEAV:
        .long      5
        .ASCII      "LEAVE"
        .balign    4,0x00
LLEAV:
        .long    DOCOL
        .long    LLEAV+HEADSIZE
        .long    0x0
        .long    SEMIS
        .long    N_LLEAV
        .long    0
        .long    0
  #LIMIT <- INDEX
        .long      UNLOOP
        .long      SEMIS
#

#  **********
#  *   >R   *
#  **********
#
        .balign    4,0x00
N_TOR:
        .long      2
        .ASCII      ">R"
        .balign    4,0x00
TOR:
        .long    TOR+HEADSIZE
        .long    TOR+HEADSIZE
        .long    0x0
        .long    LLEAV
        .long    N_TOR
        .long    0
        .long    0
        # (R1) <- (S1)
        POP     %EBX      #GET STACK PARAMETER
        LEA     %EBP,[%EBP - (CW*(1))]    #MOVE RETURN STACK DOWN
        MOV     [%EBP],%EBX #ADD TO RETURN STACK
        LODSD                  # NEXT
        JMP     DWORD PTR[%EAX]   
#

#  **********
#  *   R>   *
#  **********
#
        .balign    4,0x00
N_FROMR:
        .long      2
        .ASCII      "R>"
        .balign    4,0x00
FROMR:
        .long    FROMR+HEADSIZE
        .long    FROMR+HEADSIZE
        .long    0x0
        .long    TOR
        .long    N_FROMR
        .long    0
        .long    0
      #(S1) <- (R1)
        MOV     %EAX,[%EBP] # GET RETURN STACK VALUE
        LEA     %EBP,[%EBP + (CW*(1))]
        PUSH    %EAX
        LODSD                  # NEXT
        JMP     DWORD PTR[%EAX]   
#

#  *************
#  *   RDROP   *
#  *************
#
        .balign    4,0x00
N_RDROP:
        .long      5
        .ASCII      "RDROP"
        .balign    4,0x00
RDROP:
        .long    RDROP+HEADSIZE
        .long    RDROP+HEADSIZE
        .long    0x0
        .long    FROMR
        .long    N_RDROP
        .long    0
        .long    0
      #(S1) <- (R1)
        LEA     %EBP,[%EBP+(CW*(1))]
        LODSD                  # NEXT
        JMP     DWORD PTR[%EAX]   
#

#  **********
#  *   R@   *
#  **********
#
        .balign    4,0x00
N_RR:
        .long      2
        .ASCII      "R@"
        .balign    4,0x00
RR:
        .long    (IDO+HEADSIZE)
        .long    RR+HEADSIZE
        .long    0x0
        .long    RDROP
        .long    N_RR
        .long    0
        .long    0

#

#  **********
#  *   0=   *
#  **********
#
        .balign    4,0x00
N_ZEQU:
        .long      2
        .ASCII      "0="
        .balign    4,0x00
ZEQU:
        .long    ZEQU+HEADSIZE
        .long    ZEQU+HEADSIZE
        .long    0x0
        .long    RR
        .long    N_ZEQU
        .long    0
        .long    0

        POP     %EAX      #S2
        AND     %EAX,%EAX
        SETZ    %AL
        MOVZX   %EAX,%AL
        NEG     %EAX
        PUSH    %EAX
        LODSD                  # NEXT
        JMP     DWORD PTR[%EAX]   
        POP     %EAX
#

#  **********
#  *   0<   *
#  **********
#
        .balign    4,0x00
N_ZLESS:
        .long      2
        .ASCII      "0<"
        .balign    4,0x00
ZLESS:
        .long    ZLESS+HEADSIZE
        .long    ZLESS+HEADSIZE
        .long    0x0
        .long    ZEQU
        .long    N_ZLESS
        .long    0
        .long    0

        POP     %EAX
        AND     %EAX,%EAX
        SETS    %AL
        MOVZX   %EAX,%AL
        NEG     %EAX
        PUSH    %EAX
        LODSD                  # NEXT
        JMP     DWORD PTR[%EAX]   
#

#  *********
#  *   +   *
#  *********
#
        .balign    4,0x00
N_PLUS:
        .long      1
        .ASCII      "+"
        .balign    4,0x00
PLUS:
        .long    PLUS+HEADSIZE
        .long    PLUS+HEADSIZE
        .long    0x0
        .long    ZLESS
        .long    N_PLUS
        .long    0
        .long    0

        POP     %EAX      #(S1) <- (S1) + (S2)
        POP     %EBX
        ADD     %EAX,%EBX
        PUSH    %EAX
        LODSD                  # NEXT
        JMP     DWORD PTR[%EAX]   
#

#  **********
#  *   D+   *
#  **********
#
        .balign    4,0x00
N_DPLUS:
        .long      2
        .ASCII      "D+"
        .balign    4,0x00
DPLUS:
        .long    DPLUS+HEADSIZE
        .long    DPLUS+HEADSIZE
        .long    0x0
        .long    PLUS
        .long    N_DPLUS
        .long    0
        .long    0

        POP     %EAX      # YHC
        POP     %EDX      # YLC
        POP     %EBX      # XHC
        POP     %ECX      # XLC
        ADD     %EDX,%ECX   # SLC
        ADC     %EAX,%EBX   # SHC
        PUSH    %EDX
        PUSH    %EAX
        LODSD                  # NEXT
        JMP     DWORD PTR[%EAX]   
#

#  **************
#  *   NEGATE   *
#  **************
#
        .balign    4,0x00
N_NEGATE:
        .long      6
        .ASCII      "NEGATE"
        .balign    4,0x00
NEGATE:
        .long    NEGATE+HEADSIZE
        .long    NEGATE+HEADSIZE
        .long    0x0
        .long    DPLUS
        .long    N_NEGATE
        .long    0
        .long    0

        POP     %EAX
        NEG     %EAX
        PUSH    %EAX
        LODSD                  # NEXT
        JMP     DWORD PTR[%EAX]   
#

#  ***************
#  *   DNEGATE   *
#  ***************
#
        .balign    4,0x00
N_DNEGA:
        .long      7
        .ASCII      "DNEGATE"
        .balign    4,0x00
DNEGA:
        .long    DNEGA+HEADSIZE
        .long    DNEGA+HEADSIZE
        .long    0x0
        .long    NEGATE
        .long    N_DNEGA
        .long    0
        .long    0

        POP     %EBX
        POP     %ECX
        SUB     %EAX,%EAX
        MOV     %EDX,%EAX
        SUB     %EDX,%ECX   # MAKE 2'S COMPLEMENT
        SBB     %EAX,%EBX   # HIGH CELL
        PUSH    %EDX
        PUSH    %EAX
        LODSD                  # NEXT
        JMP     DWORD PTR[%EAX]   
        #
#

#  ************
#  *   OVER   *
#  ************
#
        .balign    4,0x00
N_OVER:
        .long      4
        .ASCII      "OVER"
        .balign    4,0x00
OVER:
        .long    OVER+HEADSIZE
        .long    OVER+HEADSIZE
        .long    0x0
        .long    DNEGA
        .long    N_OVER
        .long    0
        .long    0

        POP     %EDX
        POP     %EAX
        PUSH    %EAX
        PUSH    %EDX
        PUSH    %EAX
        LODSD                  # NEXT
        JMP     DWORD PTR[%EAX]   
#

#  ************
#  *   DROP   *
#  ************
#
        .balign    4,0x00
N_DROP:
        .long      4
        .ASCII      "DROP"
        .balign    4,0x00
DROP:
        .long    DROP+HEADSIZE
        .long    DROP+HEADSIZE
        .long    0x0
        .long    OVER
        .long    N_DROP
        .long    0
        .long    0

        POP     %EAX
        LODSD                  # NEXT
        JMP     DWORD PTR[%EAX]   
#

#  ***********
#  *   NIP   *
#  ***********
#
        .balign    4,0x00
N_NIP:
        .long      3
        .ASCII      "NIP"
        .balign    4,0x00
NIP:
        .long    NIP+HEADSIZE
        .long    NIP+HEADSIZE
        .long    0x0
        .long    DROP
        .long    N_NIP
        .long    0
        .long    0

        POP     %EAX
        POP     %EBX
        PUSH    %EAX
        LODSD                  # NEXT
        JMP     DWORD PTR[%EAX]   
#

#  *************
#  *   2DROP   *
#  *************
#
        .balign    4,0x00
N_TDROP:
        .long      5
        .ASCII      "2DROP"
        .balign    4,0x00
TDROP:
        .long    TDROP+HEADSIZE
        .long    TDROP+HEADSIZE
        .long    0x0
        .long    NIP
        .long    N_TDROP
        .long    0
        .long    0

        POP     %EAX
        POP     %EAX
        LODSD                  # NEXT
        JMP     DWORD PTR[%EAX]   
#

#  ************
#  *   SWAP   *
#  ************
#
        .balign    4,0x00
N_SWAP:
        .long      4
        .ASCII      "SWAP"
        .balign    4,0x00
SWAP:
        .long    SWAP+HEADSIZE
        .long    SWAP+HEADSIZE
        .long    0x0
        .long    TDROP
        .long    N_SWAP
        .long    0
        .long    0

        POP     %EDX
        POP     %EAX
        PUSH    %EDX
        PUSH    %EAX
        LODSD                  # NEXT
        JMP     DWORD PTR[%EAX]   
#

#  ***********
#  *   DUP   *
#  ***********
#
        .balign    4,0x00
N_LDUP:
        .long      3
        .ASCII      "DUP"
        .balign    4,0x00
LDUP:
        .long    LDUP+HEADSIZE
        .long    LDUP+HEADSIZE
        .long    0x0
        .long    SWAP
        .long    N_LDUP
        .long    0
        .long    0

        POP     %EAX
        PUSH    %EAX
        PUSH    %EAX
        LODSD                  # NEXT
        JMP     DWORD PTR[%EAX]   
#

#  ************
#  *   2DUP   *
#  ************
#
        .balign    4,0x00
N_TDUP:
        .long      4
        .ASCII      "2DUP"
        .balign    4,0x00
TDUP:
        .long    TDUP+HEADSIZE
        .long    TDUP+HEADSIZE
        .long    0x0
        .long    LDUP
        .long    N_TDUP
        .long    0
        .long    0

        POP     %EAX
        POP     %EDX
        PUSH    %EDX
        PUSH    %EAX
        PUSH    %EDX
        PUSH    %EAX
        LODSD                  # NEXT
        JMP     DWORD PTR[%EAX]   
#

#  *************
#  *   2SWAP   *
#  *************
#
        .balign    4,0x00
N_TSWAP:
        .long      5
        .ASCII      "2SWAP"
        .balign    4,0x00
TSWAP:
        .long    TSWAP+HEADSIZE
        .long    TSWAP+HEADSIZE
        .long    0x0
        .long    TDUP
        .long    N_TSWAP
        .long    0
        .long    0

        POP     %EBX
        POP     %ECX
        POP     %EAX
        POP     %EDX
        PUSH     %ECX
        PUSH     %EBX
        PUSH    %EDX
        PUSH    %EAX
        LODSD                  # NEXT
        JMP     DWORD PTR[%EAX]   
#

#  *************
#  *   2OVER   *
#  *************
#
        .balign    4,0x00
N_TOVER:
        .long      5
        .ASCII      "2OVER"
        .balign    4,0x00
TOVER:
        .long    TOVER+HEADSIZE
        .long    TOVER+HEADSIZE
        .long    0x0
        .long    TSWAP
        .long    N_TOVER
        .long    0
        .long    0

        POP     %EBX
        POP     %ECX
        POP     %EAX
        POP     %EDX
        PUSH     %EDX
        PUSH     %EAX
        PUSH     %ECX
        PUSH     %EBX
        PUSH    %EDX
        PUSH    %EAX
        LODSD                  # NEXT
        JMP     DWORD PTR[%EAX]   
#

#  **********
#  *   +!   *
#  **********
#
        .balign    4,0x00
N_PSTOR:
        .long      2
        .ASCII      "+!"
        .balign    4,0x00
PSTOR:
        .long    PSTOR+HEADSIZE
        .long    PSTOR+HEADSIZE
        .long    0x0
        .long    TOVER
        .long    N_PSTOR
        .long    0
        .long    0

        POP     %EBX      #ADDRESS
        POP     %EAX      #INCREMENT
        ADD     [%EBX],%EAX
        LODSD                  # NEXT
        JMP     DWORD PTR[%EAX]   
#

#  **************
#  *   TOGGLE   *
#  **************
#
        .balign    4,0x00
N_TOGGL:
        .long      6
        .ASCII      "TOGGLE"
        .balign    4,0x00
TOGGL:
        .long    TOGGL+HEADSIZE
        .long    TOGGL+HEADSIZE
        .long    0x0
        .long    PSTOR
        .long    N_TOGGL
        .long    0
        .long    0

        POP     %EAX      #BIT PATTERN
        POP     %EBX      #ADDR
        XOR     [%EBX],%EAX #
        LODSD                  # NEXT
        JMP     DWORD PTR[%EAX]   
#

#  *********
#  *   @   *
#  *********
#
        .balign    4,0x00
N_FETCH:
        .long      1
        .ASCII      "@"
        .balign    4,0x00
FETCH:
        .long    FETCH+HEADSIZE
        .long    FETCH+HEADSIZE
        .long    0x0
        .long    TOGGL
        .long    N_FETCH
        .long    0
        .long    0

        POP     %EBX
        MOV     %EAX,[%EBX]
        PUSH    %EAX
        LODSD                  # NEXT
        JMP     DWORD PTR[%EAX]   
#
#

#  **********
#  *   C@   *
#  **********
#
        .balign    4,0x00
N_CFET:
        .long      2
        .ASCII      "C@"
        .balign    4,0x00
CFET:
        .long    CFET+HEADSIZE
        .long    CFET+HEADSIZE
        .long    0x0
        .long    FETCH
        .long    N_CFET
        .long    0
        .long    0

        POP     %EBX
        XOR     %EAX,%EAX
        MOV     %AL,[%EBX]
        PUSH    %EAX
        LODSD                  # NEXT
        JMP     DWORD PTR[%EAX]   
#

#  **********
#  *   2@   *
#  **********
#
        .balign    4,0x00
N_TFET:
        .long      2
        .ASCII      "2@"
        .balign    4,0x00
TFET:
        .long    TFET+HEADSIZE
        .long    TFET+HEADSIZE
        .long    0x0
        .long    CFET
        .long    N_TFET
        .long    0
        .long    0

        POP     %EBX      #ADDR
        MOV     %EAX,[%EBX] #MSW
        MOV     %EDX,[%EBX+(CW*(1))]        #LSW
        PUSH    %EDX
        PUSH    %EAX
        LODSD                  # NEXT
        JMP     DWORD PTR[%EAX]   
#

#  *********
#  *   !   *
#  *********
#
        .balign    4,0x00
N_STOR:
        .long      1
        .ASCII      "!"
        .balign    4,0x00
STOR:
        .long    STOR+HEADSIZE
        .long    STOR+HEADSIZE
        .long    0x0
        .long    TFET
        .long    N_STOR
        .long    0
        .long    0

        POP     %EBX      #ADDR
        POP     %EAX      #DATA
        MOV     [%EBX],%EAX
        LODSD                  # NEXT
        JMP     DWORD PTR[%EAX]   
#
#

#  **********
#  *   C!   *
#  **********
#
        .balign    4,0x00
N_CSTOR:
        .long      2
        .ASCII      "C!"
        .balign    4,0x00
CSTOR:
        .long    CSTOR+HEADSIZE
        .long    CSTOR+HEADSIZE
        .long    0x0
        .long    STOR
        .long    N_CSTOR
        .long    0
        .long    0

        POP     %EBX      #ADDR
        POP     %EAX      #DATA
        MOV     [%EBX],%AL
        LODSD                  # NEXT
        JMP     DWORD PTR[%EAX]   
#

#  **********
#  *   2!   *
#  **********
#
        .balign    4,0x00
N_TSTOR:
        .long      2
        .ASCII      "2!"
        .balign    4,0x00
TSTOR:
        .long    TSTOR+HEADSIZE
        .long    TSTOR+HEADSIZE
        .long    0x0
        .long    CSTOR
        .long    N_TSTOR
        .long    0
        .long    0

        POP     %EBX      #ADDR
        POP     %EAX      #MSW
        MOV     [%EBX],%EAX
        POP     %EAX      #LSW
        MOV     [%EBX+(CW*(1))],%EAX
        LODSD                  # NEXT
        JMP     DWORD PTR[%EAX]   
#

#  **************
#  *   WITHIN   *
#  **************
#
        .balign    4,0x00
N_WITHIN:
        .long      6
        .ASCII      "WITHIN"
        .balign    4,0x00
WITHIN:
        .long    DOCOL
        .long    WITHIN+HEADSIZE
        .long    0x0
        .long    TSTOR
        .long    N_WITHIN
        .long    0
        .long    0

        .long      OVER, LSUB, TOR
        .long      LSUB, FROMR
        .long      ULESS
        .long      SEMIS
#


#  ************
#  *   FAR@   *
#  ************
#
        .balign    4,0x00
N_LFET:
        .long      4
        .ASCII      "FAR@"
        .balign    4,0x00
LFET:
        .long    LFET+HEADSIZE
        .long    LFET+HEADSIZE
        .long    0x0
        .long    WITHIN
        .long    N_LFET
        .long    0
        .long    0

#

        POP     %EBX      #MEM LOC
        MOV     %DX,%FS
        POP     %FS      #SEG REG VAL
        MOV     %EBX,%FS:[%EBX]
        MOV     %FS,%DX
        PUSH    %EBX
        LODSD                  # NEXT
        JMP     DWORD PTR[%EAX]   
#
        LODSD                  # NEXT
        JMP     DWORD PTR[%EAX]   
#

#  ************
#  *   FAR!   *
#  ************
#
        .balign    4,0x00
N_LSTOR:
        .long      4
        .ASCII      "FAR!"
        .balign    4,0x00
LSTOR:
        .long    LSTOR+HEADSIZE
        .long    LSTOR+HEADSIZE
        .long    0x0
        .long    LFET
        .long    N_LSTOR
        .long    0
        .long    0

#

        POP     %EBX
        MOV     %DX,%FS
        POP     %FS      #SEG REG VAL
        POP     %EAX
        MOV     %FS:[%EBX],%EAX
        MOV     %FS,%DX
        LODSD                  # NEXT
        JMP     DWORD PTR[%EAX]   
#
#
#

#  *********
#  *   :   *
#  *********
#
        .balign    4,0x00
N_COLON:
        .long      1
        .ASCII      ":"
        .balign    4,0x00
COLON:
        .long    DOCOL
        .long    COLON+HEADSIZE
        .long    0x0
        .long    LSTOR
        .long    N_COLON
        .long    0
        .long    0

        .long      SCSP
        .long      LNAME
        .long      PCREAT
        .long      LATEST, HIDDEN
        .long      RBRAC
        .long      PSCOD
DOCOL:  LEA     %EBP,[%EBP - (CW*(1))]  #Push HIP
        MOV     [%EBP],%ESI #R1 <- (IP)
         MOV     %ESI,[%EAX+(CW*(D_HOFFSET - C_HOFFSET))]  #(IP) <- (PFA)
#        CALL    DISPLAYSI
# 
        LODSD                  # NEXT
        JMP     DWORD PTR[%EAX]   
#

#  *********
#  *   ;   *
#  *********
#
        .balign    4,0x00
N_SEMI:
        .long      1
        .ASCII      ";"
        .balign    4,0x00
SEMI:
        .long    DOCOL
        .long    SEMI+HEADSIZE
        .long    B_IMMED
        .long    COLON
        .long    N_SEMI
        .long    0
        .long    0

        .long      QCSP
        .long      LIT, SEMIS, COMMA
        .long      LATEST, HIDDEN
        .long      LBRAC
        .long      SEMIS
#

#  ****************
#  *   CONSTANT   *
#  ****************
#
        .balign    4,0x00
N_LCONST:
        .long      8
        .ASCII      "CONSTANT"
        .balign    4,0x00
LCONST:
        .long    DOCOL
        .long    LCONST+HEADSIZE
        .long    0x0
        .long    SEMI
        .long    N_LCONST
        .long    0
        .long    0

        .long      LNAME
        .long      PCREAT
        .long      LATEST, TDFA, STOR
        .long      PSCOD
DOCON:  MOV     %EAX,[%EAX+(CW*((D_HOFFSET)))] #GET DATA FROM PFA
        PUSH    %EAX
        LODSD                  # NEXT
        JMP     DWORD PTR[%EAX]   
#

#  ****************
#  *   VARIABLE   *
#  ****************
#
        .balign    4,0x00
N_VAR:
        .long      8
        .ASCII      "VARIABLE"
        .balign    4,0x00
VAR:
        .long    DOCOL
        .long    VAR+HEADSIZE
        .long    0x0
        .long    LCONST
        .long    N_VAR
        .long    0
        .long    0

        .long      LDATA
        .long      ZERO, COMMA
        .long      SEMIS
#

#  ************
#  *   DATA   *
#  ************
#
        .balign    4,0x00
N_LDATA:
        .long      4
        .ASCII      "DATA"
        .balign    4,0x00
LDATA:
        .long    DOCOL
        .long    LDATA+HEADSIZE
        .long    0x0
        .long    VAR
        .long    N_LDATA
        .long    0
        .long    0

        .long      LNAME
        .long      PCREAT
        .long      PSCOD
DOVAR:  MOV     %EAX,[%EAX+(CW*((D_HOFFSET)))] #(AX) <- PFA
        PUSH    %EAX
        LODSD                  # NEXT
        JMP     DWORD PTR[%EAX]   
#

#  ************
#  *   USER   *
#  ************
#
        .balign    4,0x00
N_USER:
        .long      4
        .ASCII      "USER"
        .balign    4,0x00
USER:
        .long    DOCOL
        .long    USER+HEADSIZE
        .long    0x0
        .long    LDATA
        .long    N_USER
        .long    0
        .long    0

        .long      LCONST
        .long      PSCOD
DOUSE:  MOV     %EBX,[%EAX+(CW*((D_HOFFSET)))] #PFA  
       MOV     %EAX, %EBP
        OR      %EAX, OFFSET STACKSIZE-1  #Requires it is power of 2.
        INC     %EAX
        ADD     %EAX,%EBX      #ADDR OF VARIABLE
        PUSH    %EAX
        LODSD                  # NEXT
        JMP     DWORD PTR[%EAX]   
#
#

#  *********
#  *   _   *
#  *********
#
        .balign    4,0x00
N_X:
        .long      1
        .ASCII      "_"
        .balign    4,0x00
X:
        .long    X+HEADSIZE
        .long    X+HEADSIZE
        .long    0x0
        .long    USER
        .long    N_X
        .long    0
        .long    0

        PUSH    %EAX
        LODSD                  # NEXT
        JMP     DWORD PTR[%EAX]    #Whatever happens to be in %EAX, i.e. the dea of ``_''.
#

#  *********
#  *   0   *
#  *********
#
        .balign    4,0x00
N_ZERO:
        .long      1
        .ASCII      "0"
        .balign    4,0x00
ZERO:
        .long    DOCON
        .long    0
        .long    0x0
        .long    X
        .long    N_ZERO
        .long    0
        .long    0

# The data field of 0 is used for an empty string.
#

#  *********
#  *   1   *
#  *********
#
        .balign    4,0x00
N_ONE:
        .long      1
        .ASCII      "1"
        .balign    4,0x00
ONE:
        .long    DOCON
        .long    1
        .long    0x0
        .long    ZERO
        .long    N_ONE
        .long    0
        .long    0

#

#  *********
#  *   2   *
#  *********
#
        .balign    4,0x00
N_TWO:
        .long      1
        .ASCII      "2"
        .balign    4,0x00
TWO:
        .long    DOCON
        .long    2
        .long    0x0
        .long    ONE
        .long    N_TWO
        .long    0
        .long    0

#

#  **********
#  *   BL   *
#  **********
#
        .balign    4,0x00
N_LBL:
        .long      2
        .ASCII      "BL"
        .balign    4,0x00
LBL:
        .long    DOCON
        .long    ABL
        .long    0x0
        .long    TWO
        .long    N_LBL
        .long    0
        .long    0

#

#
#  **********
#  *   $@   *
#  **********
#
        .balign    4,0x00
N_SFET:
        .long      2
        .ASCII      "$@"
        .balign    4,0x00
SFET:
        .long    SFET+HEADSIZE
        .long    SFET+HEADSIZE
        .long    0x0
        .long    LBL
        .long    N_SFET
        .long    0
        .long    0

        POP   %EBX
        MOV   %EAX,[%EBX]
        LEA   %EDX, [%EBX+(CW*(1))]
        PUSH    %EDX
        PUSH    %EAX
        LODSD                  # NEXT
        JMP     DWORD PTR[%EAX]   
#

#  **********
#  *   $!   *
#  **********
#
        .balign    4,0x00
N_SSTOR:
        .long      2
        .ASCII      "$!"
        .balign    4,0x00
SSTOR:
        .long    DOCOL
        .long    SSTOR+HEADSIZE
        .long    0x0
        .long    SFET
        .long    N_SSTOR
        .long    0
        .long    0

        .long TDUP, STOR, CELLP, SWAP, LMOVE
        .long SEMIS
#

# 

#  ***********
#  *   $+!   *
#  ***********
#
        .balign    4,0x00
N_SADD:
        .long      3
        .ASCII      "$+!"
        .balign    4,0x00
SADD:
        .long    DOCOL
        .long    SADD+HEADSIZE
        .long    0x0
        .long    SSTOR
        .long    N_SADD
        .long    0
        .long    0

        .long   LDUP, FETCH, TOR # Remember old count.
        .long   TDUP, PSTOR
        .long   CELLP, FROMR, PLUS, SWAP, LMOVE
        .long SEMIS
#

#  ***********
#  *   $C+   *
#  ***********
#
        .balign    4,0x00
N_CHAPP:
        .long      3
        .ASCII      "$C+"
        .balign    4,0x00
CHAPP:
        .long    DOCOL
        .long    CHAPP+HEADSIZE
        .long    0x0
        .long    SADD
        .long    N_CHAPP
        .long    0
        .long    0

        .long   LDUP, TOR
        .long   LDUP, FETCH, PLUS, CELLP, CSTOR
        .long   ONE, FROMR, PSTOR
        .long SEMIS
#

#  **********
#  *   $,   *
#  **********
#
        .balign    4,0x00
N_SCOMMA:
        .long      2
        .ASCII      "$,"
        .balign    4,0x00
SCOMMA:
        .long    DOCOL
        .long    SCOMMA+HEADSIZE
        .long    0x0
        .long    CHAPP
        .long    N_SCOMMA
        .long    0
        .long    0

        .long HERE, TOR
        .long LDUP, CELLP, ALLOT
        .long RR, SSTOR, FROMR
 .long LALIGN
        .long SEMIS
#

#  ***********
#  *   C/L   *
#  ***********
#
        .balign    4,0x00
N_CSLL:
        .long      3
        .ASCII      "C/L"
        .balign    4,0x00
CSLL:
        .long    DOCON
        .long    64
        .long    0x0
        .long    SCOMMA
        .long    N_CSLL
        .long    0
        .long    0

#


#  **************
#  *   _FIRST   *
#  **************
#
        .balign    4,0x00
N__FIRST:
        .long      6
        .ASCII      "_FIRST"
        .balign    4,0x00
_FIRST:
        .long    DOCON
        .long    BUF1
        .long    0x0
        .long    CSLL
        .long    N__FIRST
        .long    0
        .long    0

 #  
#
# 
#

#  **************
#  *   _LIMIT   *
#  **************
#
        .balign    4,0x00
N__LIMIT:
        .long      6
        .ASCII      "_LIMIT"
        .balign    4,0x00
_LIMIT:
        .long    DOCON
        .long    BUF1+(KBBUF+2*CW)*NBUF
        .long    0x0
        .long    _FIRST
        .long    N__LIMIT
        .long    0
        .long    0

# THE END  OF THE MEMORY 

#  **********
#  *   EM   *
#  **********
#
        .balign    4,0x00
N_LEM:
        .long      2
        .ASCII      "EM"
        .balign    4,0x00
LEM:
        .long    DOCON
        .long    ACTUAL_EM
        .long    0x0
        .long    _LIMIT
        .long    N_LEM
        .long    0
        .long    0

#

#  **********
#  *   BM   *
#  **********
#
        .balign    4,0x00
N_LBM:
        .long      2
        .ASCII      "BM"
        .balign    4,0x00
LBM:
        .long    DOCON
        .long    _ORIG
        .long    0x0
        .long    LEM
        .long    N_LBM
        .long    0
        .long    0

#


#  *****************
#  *   TASK-SIZE   *
#  *****************
#
        .balign    4,0x00
N_LTAS:
        .long      9
        .ASCII      "TASK-SIZE"
        .balign    4,0x00
LTAS:
        .long    DOCON
        .long    TAS
        .long    0x0
        .long    LBM
        .long    N_LTAS
        .long    0
        .long    0

#

#  *************
#  *   B/BUF   *
#  *************
#
        .balign    4,0x00
N_BBUF:
        .long      5
        .ASCII      "B/BUF"
        .balign    4,0x00
BBUF:
        .long    DOCON
        .long    KBBUF
        .long    0x0
        .long    LTAS
        .long    N_BBUF
        .long    0
        .long    0

#
# All user variables are initialised 
# with the values from USINI.
# The implementation relies on the initialisation of 
# those with numbers (1..11), so change in concord with USINI.

#  **********
#  *   U0   *
#  **********
#
        .balign    4,0x00
N_UZERO:
        .long      2
        .ASCII      "U0"
        .balign    4,0x00
UZERO:
        .long    DOUSE
        .long    (CW*(1))
        .long    0x0
        .long    BBUF
        .long    N_UZERO
        .long    0
        .long    0

#

#  **********
#  *   S0   *
#  **********
#
        .balign    4,0x00
N_SZERO:
        .long      2
        .ASCII      "S0"
        .balign    4,0x00
SZERO:
        .long    DOUSE
        .long    (CW*(2))
        .long    0x0
        .long    UZERO
        .long    N_SZERO
        .long    0
        .long    0

#

#  **********
#  *   R0   *
#  **********
#
        .balign    4,0x00
N_RZERO:
        .long      2
        .ASCII      "R0"
        .balign    4,0x00
RZERO:
        .long    DOUSE
        .long    (CW*(3))
        .long    0x0
        .long    SZERO
        .long    N_RZERO
        .long    0
        .long    0

#

#  ***********
#  *   TIB   *
#  ***********
#
        .balign    4,0x00
N_TIB:
        .long      3
        .ASCII      "TIB"
        .balign    4,0x00
TIB:
        .long    DOUSE
        .long    (CW*(4))
        .long    0x0
        .long    RZERO
        .long    N_TIB
        .long    0
        .long    0

#

#  **************
#  *   RUBOUT   *
#  **************
#
        .balign    4,0x00
N_RUBOUT:
        .long      6
        .ASCII      "RUBOUT"
        .balign    4,0x00
RUBOUT:
        .long    DOUSE
        .long    (CW*(5))
        .long    0x0
        .long    TIB
        .long    N_RUBOUT
        .long    0
        .long    0

#

#  ***************
#  *   WARNING   *
#  ***************
#
        .balign    4,0x00
N_LWARN:
        .long      7
        .ASCII      "WARNING"
        .balign    4,0x00
LWARN:
        .long    DOUSE
        .long    (CW*(7))
        .long    0x0
        .long    RUBOUT
        .long    N_LWARN
        .long    0
        .long    0

#

#  *************
#  *   FENCE   *
#  *************
#
        .balign    4,0x00
N_FENCE:
        .long      5
        .ASCII      "FENCE"
        .balign    4,0x00
FENCE:
        .long    DOUSE
        .long    (CW*(8))
        .long    0x0
        .long    LWARN
        .long    N_FENCE
        .long    0
        .long    0

#

#  **********
#  *   DP   *
#  **********
#
        .balign    4,0x00
N_LDP:
        .long      2
        .ASCII      "DP"
        .balign    4,0x00
LDP:
        .long    DOUSE
        .long    (CW*(9))
        .long    0x0
        .long    FENCE
        .long    N_LDP
        .long    0
        .long    0

#

#  ****************
#  *   VOC-LINK   *
#  ****************
#
        .balign    4,0x00
N_VOCL:
        .long      8
        .ASCII      "VOC-LINK"
        .balign    4,0x00
VOCL:
        .long    DOUSE
        .long    (CW*(10))
        .long    0x0
        .long    LDP
        .long    N_VOCL
        .long    0
        .long    0

#

#  ****************
#  *   MAX-USER   *
#  ****************
#
        .balign    4,0x00
N_MAXUSER:
        .long      8
        .ASCII      "MAX-USER"
        .balign    4,0x00
MAXUSER:
        .long    DOUSE
        .long    (CW*(11))
        .long    0x0
        .long    VOCL
        .long    N_MAXUSER
        .long    0
        .long    0

#
#
#

#  *************
#  *   WHERE   *
#  *************
#
        .balign    4,0x00
N_LWHERE:
        .long      5
        .ASCII      "WHERE"
        .balign    4,0x00
LWHERE:
        .long    DOUSE
        .long    (CW*(12))
        .long    0x0
        .long    MAXUSER
        .long    N_LWHERE
        .long    0
        .long    0
    #  Occupies two CELLS! 
#

#  ***********
#  *   SCR   *
#  ***********
#
        .balign    4,0x00
N_SCR:
        .long      3
        .ASCII      "SCR"
        .balign    4,0x00
SCR:
        .long    DOUSE
        .long    (CW*(33))
        .long    0x0
        .long    LWHERE
        .long    N_SCR
        .long    0
        .long    0

#

#  *************
#  *   STATE   *
#  *************
#
        .balign    4,0x00
N_STATE:
        .long      5
        .ASCII      "STATE"
        .balign    4,0x00
STATE:
        .long    DOUSE
        .long    (CW*(18))
        .long    0x0
        .long    SCR
        .long    N_STATE
        .long    0
        .long    0

#

#  ************
#  *   BASE   *
#  ************
#
        .balign    4,0x00
N_BASE:
        .long      4
        .ASCII      "BASE"
        .balign    4,0x00
BASE:
        .long    DOUSE
        .long    (CW*(19))
        .long    0x0
        .long    STATE
        .long    N_BASE
        .long    0
        .long    0

#

#  ***********
#  *   DPL   *
#  ***********
#
        .balign    4,0x00
N_DPL:
        .long      3
        .ASCII      "DPL"
        .balign    4,0x00
DPL:
        .long    DOUSE
        .long    (CW*(20))
        .long    0x0
        .long    BASE
        .long    N_DPL
        .long    0
        .long    0

#

#  ***********
#  *   ECL   *
#  ***********
#
        .balign    4,0x00
N_ECL:
        .long      3
        .ASCII      "ECL"
        .balign    4,0x00
ECL:
        .long    DOUSE
        .long    (CW*(38))
        .long    0x0
        .long    DPL
        .long    N_ECL
        .long    0
        .long    0

#

#  ***********
#  *   FLD   *
#  ***********
#
        .balign    4,0x00
N_LFLD:
        .long      3
        .ASCII      "FLD"
        .balign    4,0x00
LFLD:
        .long    DOUSE
        .long    (CW*(21))
        .long    0x0
        .long    ECL
        .long    N_LFLD
        .long    0
        .long    0

#


#  ***********
#  *   CSP   *
#  ***********
#
        .balign    4,0x00
N_LCSP:
        .long      3
        .ASCII      "CSP"
        .balign    4,0x00
LCSP:
        .long    DOUSE
        .long    (CW*(22))
        .long    0x0
        .long    LFLD
        .long    N_LCSP
        .long    0
        .long    0

#
#

#  **********
#  *   R#   *
#  **********
#
        .balign    4,0x00
N_RNUM:
        .long      2
        .ASCII      "R#"
        .balign    4,0x00
RNUM:
        .long    DOUSE
        .long    (CW*(23))
        .long    0x0
        .long    LCSP
        .long    N_RNUM
        .long    0
        .long    0

#

#  ***********
#  *   HLD   *
#  ***********
#
        .balign    4,0x00
N_HLD:
        .long      3
        .ASCII      "HLD"
        .balign    4,0x00
HLD:
        .long    DOUSE
        .long    (CW*(24))
        .long    0x0
        .long    RNUM
        .long    N_HLD
        .long    0
        .long    0

#

#  ***********
#  *   OUT   *
#  ***********
#
        .balign    4,0x00
N_LOUT:
        .long      3
        .ASCII      "OUT"
        .balign    4,0x00
LOUT:
        .long    DOUSE
        .long    (CW*(25))
        .long    0x0
        .long    HLD
        .long    N_LOUT
        .long    0
        .long    0

#

#  *************
#  *   (BLK)   *
#  *************
#
        .balign    4,0x00
N_PBLK:
        .long      5
        .ASCII      "(BLK)"
        .balign    4,0x00
PBLK:
        .long    DOUSE
        .long    (CW*(26))
        .long    0x0
        .long    LOUT
        .long    N_PBLK
        .long    0
        .long    0

#

#  **********
#  *   PP   *
#  **********
#
        .balign    4,0x00
N_PP:
        .long      2
        .ASCII      "PP"
        .balign    4,0x00
PP:
        .long    DOUSE
        .long    (CW*(29))
        .long    0x0
        .long    PBLK
        .long    N_PP
        .long    0
        .long    0

#


#  *************
#  *   (>IN)   *
#  *************
#
        .balign    4,0x00
N_PIIN:
        .long      5
        .ASCII      "(>IN)"
        .balign    4,0x00
PIIN:
        .long    DOUSE
        .long    (CW*(30))
        .long    0x0
        .long    PP
        .long    N_PIIN
        .long    0
        .long    0

#
#

#  ************
#  *   ARGS   *
#  ************
#
        .balign    4,0x00
N_ARGS:
        .long      4
        .ASCII      "ARGS"
        .balign    4,0x00
ARGS:
        .long    DOUSE
        .long    (CW*(31))
        .long    0x0
        .long    PIIN
        .long    N_ARGS
        .long    0
        .long    0

#

#  ***************
#  *   HANDLER   *
#  ***************
#
        .balign    4,0x00
N_HANDLER:
        .long      7
        .ASCII      "HANDLER"
        .balign    4,0x00
HANDLER:
        .long    DOUSE
        .long    (CW*(32))
        .long    0x0
        .long    ARGS
        .long    N_HANDLER
        .long    0
        .long    0

#

#  ***************
#  *   CURRENT   *
#  ***************
#
        .balign    4,0x00
N_CURR:
        .long      7
        .ASCII      "CURRENT"
        .balign    4,0x00
CURR:
        .long    DOUSE
        .long    (CW*(34))
        .long    0x0
        .long    HANDLER
        .long    N_CURR
        .long    0
        .long    0

#

#  *****************
#  *   REMAINDER   *
#  *****************
#
        .balign    4,0x00
N_REMAIND:
        .long      9
        .ASCII      "REMAINDER"
        .balign    4,0x00
REMAIND:
        .long    DOUSE
        .long    (CW*(14))
        .long    0x0
        .long    CURR
        .long    N_REMAIND
        .long    0
        .long    0

#      IMPORTANT
# REQUIRES ONE MORE CELL!
#
#

#  **************
#  *   OFFSET   *
#  **************
#
        .balign    4,0x00
N_LOFFSET:
        .long      6
        .ASCII      "OFFSET"
        .balign    4,0x00
LOFFSET:
        .long    DOUSE
        .long    (CW*(37))
        .long    0x0
        .long    REMAIND
        .long    N_LOFFSET
        .long    0
        .long    0

#
#
#

#  ***************
#  *   CONTEXT   *
#  ***************
#
        .balign    4,0x00
N_CONTEXT:
        .long      7
        .ASCII      "CONTEXT"
        .balign    4,0x00
CONTEXT:
        .long    DOUSE
        .long    (CW*(39))
        .long    0x0
        .long    LOFFSET
        .long    N_CONTEXT
        .long    0
        .long    0
 # Up to  39+16
#      IMPORTANT
#     16 USER SPACE CELLS MUST BE KEPT FREE
#     IN ADDITION TO THE ONE FOR CONTEXT
#
#========== END USER VARIABLES =============
#

#  **********
#  *   1+   *
#  **********
#
        .balign    4,0x00
N_ONEP:
        .long      2
        .ASCII      "1+"
        .balign    4,0x00
ONEP:
        .long    DOCOL
        .long    ONEP+HEADSIZE
        .long    0x0
        .long    CONTEXT
        .long    N_ONEP
        .long    0
        .long    0

        .long      ONE
        .long      PLUS
        .long      SEMIS
#

#  *************
#  *   CELL+   *
#  *************
#
        .balign    4,0x00
N_CELLP:
        .long      5
        .ASCII      "CELL+"
        .balign    4,0x00
CELLP:
        .long    DOCOL
        .long    CELLP+HEADSIZE
        .long    0x0
        .long    ONEP
        .long    N_CELLP
        .long    0
        .long    0

        .long      LIT, CW
        .long      PLUS
        .long      SEMIS
#

#
#  *************
#  *   CELLS   *
#  *************
#
        .balign    4,0x00
N_LCELLS:
        .long      5
        .ASCII      "CELLS"
        .balign    4,0x00
LCELLS:
        .long    DOCOL
        .long    LCELLS+HEADSIZE
        .long    0x0
        .long    CELLP
        .long    N_LCELLS
        .long    0
        .long    0

        .long       TWO 
        .long      LSHIFT
        .long      SEMIS
#

#  *************
#  *   CHAR+   *
#  *************
#
        .balign    4,0x00
N_CHARP:
        .long      5
        .ASCII      "CHAR+"
        .balign    4,0x00
CHARP:
        .long    DOCOL
        .long    (ONEP+HEADSIZE)
        .long    0x0
        .long    LCELLS
        .long    N_CHARP
        .long    0
        .long    0

#

#  *************
#  *   CHARS   *
#  *************
#
        .balign    4,0x00
N_CHARS:
        .long      5
        .ASCII      "CHARS"
        .balign    4,0x00
CHARS:
        .long    DOCOL 
        .long    CHARS+HEADSIZE
        .long    0x0
        .long    CHARP
        .long    N_CHARS
        .long    0
        .long    0

        .long      SEMIS
#

#  *************
#  *   ALIGN   *
#  *************
#
        .balign    4,0x00
N_LALIGN:
        .long      5
        .ASCII      "ALIGN"
        .balign    4,0x00
LALIGN:
        .long    DOCOL 
        .long    LALIGN+HEADSIZE
        .long    0x0
        .long    CHARS
        .long    N_LALIGN
        .long    0
        .long    0


        .long      LDP, FETCH
        .long      ALIGNED
        .long      LDP, STOR
#
        .long      SEMIS
#

#  ***************
#  *   ALIGNED   *
#  ***************
#
        .balign    4,0x00
N_ALIGNED :
        .long      7
        .ASCII      "ALIGNED"
        .balign    4,0x00
ALIGNED :
        .long    ALIGNED +HEADSIZE
        .long    ALIGNED +HEADSIZE
        .long    0x0
        .long    LALIGN
        .long    N_ALIGNED 
        .long    0
        .long    0


        POP     %EAX
        DEC     %EAX
        OR      %EAX, OFFSET (CW*(1))-1
        INC     %EAX
        PUSH    %EAX
#
        LODSD                  # NEXT
        JMP     DWORD PTR[%EAX]   
#

#  ************
#  *   HERE   *
#  ************
#
        .balign    4,0x00
N_HERE:
        .long      4
        .ASCII      "HERE"
        .balign    4,0x00
HERE:
        .long    DOCOL
        .long    HERE+HEADSIZE
        .long    0x0
        .long    ALIGNED 
        .long    N_HERE
        .long    0
        .long    0

        .long      LDP
        .long      FETCH
        .long      SEMIS
#

#  *************
#  *   ALLOT   *
#  *************
#
        .balign    4,0x00
N_ALLOT:
        .long      5
        .ASCII      "ALLOT"
        .balign    4,0x00
ALLOT:
        .long    DOCOL
        .long    ALLOT+HEADSIZE
        .long    0x0
        .long    HERE
        .long    N_ALLOT
        .long    0
        .long    0

        .long      LDP
        .long      PSTOR
        .long      SEMIS
#

#  *********
#  *   ,   *
#  *********
#
        .balign    4,0x00
N_COMMA:
        .long      1
        .ASCII      ","
        .balign    4,0x00
COMMA:
        .long    DOCOL
        .long    COMMA+HEADSIZE
        .long    0x0
        .long    ALLOT
        .long    N_COMMA
        .long    0
        .long    0

        .long      HERE
        .long      LIT
        .long      CW
        .long      ALLOT
        .long      STOR
        .long      SEMIS
#

#  **********
#  *   C,   *
#  **********
#
        .balign    4,0x00
N_CCOMM:
        .long      2
        .ASCII      "C,"
        .balign    4,0x00
CCOMM:
        .long    DOCOL
        .long    CCOMM+HEADSIZE
        .long    0x0
        .long    COMMA
        .long    N_CCOMM
        .long    0
        .long    0

        .long      HERE
        .long      ONE
        .long      ALLOT
        .long      CSTOR
        .long      SEMIS
#

#  **********
#  *   2,   *
#  **********
#
        .balign    4,0x00
N_TCOMM:
        .long      2
        .ASCII      "2,"
        .balign    4,0x00
TCOMM:
        .long    DOCOL
        .long    TCOMM+HEADSIZE
        .long    0x0
        .long    CCOMM
        .long    N_TCOMM
        .long    0
        .long    0

        .long      HERE
        .long      LIT
        .long      CW*2
        .long      ALLOT
        .long      TSTOR
        .long      SEMIS
#

#  *********
#  *   -   *
#  *********
#
        .balign    4,0x00
N_LSUB:
        .long      1
        .ASCII      "-"
        .balign    4,0x00
LSUB:
        .long    LSUB+HEADSIZE
        .long    LSUB+HEADSIZE
        .long    0x0
        .long    TCOMM
        .long    N_LSUB
        .long    0
        .long    0

        POP     %EDX      #S1
        POP     %EAX
        SUB     %EAX,%EDX
        PUSH    %EAX
        LODSD                  # NEXT
        JMP     DWORD PTR[%EAX]      #S1 = S2 - S1
#

#  *********
#  *   =   *
#  *********
#
        .balign    4,0x00
N_EQUAL:
        .long      1
        .ASCII      "="
        .balign    4,0x00
EQUAL:
        .long    EQUAL+HEADSIZE
        .long    EQUAL+HEADSIZE
        .long    0x0
        .long    LSUB
        .long    N_EQUAL
        .long    0
        .long    0

         POP     %EAX
         POP     %EBX
         CMP     %EAX,%EBX
         SETZ    %AL
         MOVZX   %EAX,%AL
         NEG     %EAX
         PUSH    %EAX
        LODSD                  # NEXT
        JMP     DWORD PTR[%EAX]   
#

#  *********
#  *   <   *
#  *********
#
        .balign    4,0x00
N_LESS:
        .long      1
        .ASCII      "<"
        .balign    4,0x00
LESS:
        .long    LESS+HEADSIZE
        .long    LESS+HEADSIZE
        .long    0x0
        .long    EQUAL
        .long    N_LESS
        .long    0
        .long    0

        POP     %EAX
        POP     %EBX
        CMP     %EBX,%EAX
        SETL    %AL
        MOVZX   %EAX,%AL
        NEG     %EAX
        PUSH    %EAX
        LODSD                  # NEXT
        JMP     DWORD PTR[%EAX]   
#

#  **********
#  *   U<   *
#  **********
#
        .balign    4,0x00
N_ULESS:
        .long      2
        .ASCII      "U<"
        .balign    4,0x00
ULESS:
        .long    ULESS+HEADSIZE
        .long    ULESS+HEADSIZE
        .long    0x0
        .long    LESS
        .long    N_ULESS
        .long    0
        .long    0

       POP     %EAX
               POP     %EDX
               SUB     %EDX,%EAX
               SBB     %EAX,%EAX
               PUSH    %EAX
        LODSD                  # NEXT
        JMP     DWORD PTR[%EAX]   
#

#  *********
#  *   >   *
#  *********
#
        .balign    4,0x00
N_GREAT:
        .long      1
        .ASCII      ">"
        .balign    4,0x00
GREAT:
        .long    DOCOL
        .long    GREAT+HEADSIZE
        .long    0x0
        .long    ULESS
        .long    N_GREAT
        .long    0
        .long    0

        .long      SWAP
        .long      LESS
        .long      SEMIS
#

#  **********
#  *   <>   *
#  **********
#
        .balign    4,0x00
N_UNEQ:
        .long      2
        .ASCII      "<>"
        .balign    4,0x00
UNEQ:
        .long    UNEQ+HEADSIZE
        .long    UNEQ+HEADSIZE
        .long    0x0
        .long    GREAT
        .long    N_UNEQ
        .long    0
        .long    0

         POP     %EAX
         POP     %EBX
         CMP     %EAX,%EBX
         SETNZ    %AL
         MOVZX   %EAX,%AL
         NEG     %EAX
         PUSH    %EAX
        LODSD                  # NEXT
        JMP     DWORD PTR[%EAX]   
#

#  ***********
#  *   ROT   *
#  ***********
#
        .balign    4,0x00
N_ROT:
        .long      3
        .ASCII      "ROT"
        .balign    4,0x00
ROT:
        .long    ROT+HEADSIZE
        .long    ROT+HEADSIZE
        .long    0x0
        .long    UNEQ
        .long    N_ROT
        .long    0
        .long    0

        POP     %EDX      #S1
        POP     %EBX      #S2
        POP     %EAX      #S3
        PUSH    %EBX
        PUSH    %EDX
        PUSH    %EAX
        LODSD                  # NEXT
        JMP     DWORD PTR[%EAX]   
#

#  *************
#  *   SPACE   *
#  *************
#
        .balign    4,0x00
N_SPACE:
        .long      5
        .ASCII      "SPACE"
        .balign    4,0x00
SPACE:
        .long    DOCOL
        .long    SPACE+HEADSIZE
        .long    0x0
        .long    ROT
        .long    N_SPACE
        .long    0
        .long    0

        .long      LBL
        .long      EMIT
        .long      SEMIS
#

#  ************
#  *   ?DUP   *
#  ************
#
        .balign    4,0x00
N_QDUP:
        .long      4
        .ASCII      "?DUP"
        .balign    4,0x00
QDUP:
        .long    DOCOL
        .long    QDUP+HEADSIZE
        .long    0x0
        .long    SPACE
        .long    N_QDUP
        .long    0
        .long    0

        .long      LDUP
        .long      ZBRAN
        .long      QDUP1-.-CW # IF
        .long      LDUP    #THEN
QDUP1:  .long      SEMIS
#

#  **************
#  *   LATEST   *
#  **************
#
        .balign    4,0x00
N_LATEST:
        .long      6
        .ASCII      "LATEST"
        .balign    4,0x00
LATEST:
        .long    DOCOL
        .long    LATEST+HEADSIZE
        .long    0x0
        .long    QDUP
        .long    N_LATEST
        .long    0
        .long    0

        .long      CURR
        .long      FETCH
        .long      TLFA
        .long      FETCH
        .long      SEMIS
#

#  ************
#  *   >CFA   *
#  ************
#
        .balign    4,0x00
N_TCFA:
        .long      4
        .ASCII      ">CFA"
        .balign    4,0x00
TCFA:
        .long    DOCOL
        .long    TCFA+HEADSIZE
        .long    0x0
        .long    LATEST
        .long    N_TCFA
        .long    0
        .long    0

        .long      LIT, (CW*(C_HOFFSET))
        .long      PLUS
        .long      SEMIS
#

#  ************
#  *   >DFA   *
#  ************
#
        .balign    4,0x00
N_TDFA:
        .long      4
        .ASCII      ">DFA"
        .balign    4,0x00
TDFA:
        .long    DOCOL
        .long    TDFA+HEADSIZE
        .long    0x0
        .long    TCFA
        .long    N_TDFA
        .long    0
        .long    0

        .long      LIT, (CW*(D_HOFFSET))
        .long      PLUS
        .long      SEMIS
#

#  ************
#  *   >FFA   *
#  ************
#
        .balign    4,0x00
N_TFFA:
        .long      4
        .ASCII      ">FFA"
        .balign    4,0x00
TFFA:
        .long    DOCOL
        .long    TFFA+HEADSIZE
        .long    0x0
        .long    TDFA
        .long    N_TFFA
        .long    0
        .long    0

        .long      LIT, (CW*(F_HOFFSET))
        .long      PLUS
        .long      SEMIS
#

#  ************
#  *   >LFA   *
#  ************
#
        .balign    4,0x00
N_TLFA:
        .long      4
        .ASCII      ">LFA"
        .balign    4,0x00
TLFA:
        .long    DOCOL
        .long    TLFA+HEADSIZE
        .long    0x0
        .long    TFFA
        .long    N_TLFA
        .long    0
        .long    0

        .long      LIT, (CW*(L_HOFFSET))
        .long      PLUS
        .long      SEMIS
#

#  ************
#  *   >NFA   *
#  ************
#
        .balign    4,0x00
N_TNFA:
        .long      4
        .ASCII      ">NFA"
        .balign    4,0x00
TNFA:
        .long    DOCOL
        .long    TNFA+HEADSIZE
        .long    0x0
        .long    TLFA
        .long    N_TNFA
        .long    0
        .long    0

        .long      LIT,(CW*(N_HOFFSET))
        .long      PLUS
        .long      SEMIS
#


#
#  ************
#  *   >SFA   *
#  ************
#
        .balign    4,0x00
N_TSFA:
        .long      4
        .ASCII      ">SFA"
        .balign    4,0x00
TSFA:
        .long    DOCOL
        .long    TSFA+HEADSIZE
        .long    0x0
        .long    TNFA
        .long    N_TSFA
        .long    0
        .long    0

        .long      LIT,(CW*(S_HOFFSET))
        .long      PLUS
        .long      SEMIS
#
#


#
#  ************
#  *   >XFA   *
#  ************
#
        .balign    4,0x00
N_TXFA:
        .long      4
        .ASCII      ">XFA"
        .balign    4,0x00
TXFA:
        .long    DOCOL
        .long    TXFA+HEADSIZE
        .long    0x0
        .long    TSFA
        .long    N_TXFA
        .long    0
        .long    0

        .long      LIT,(CW*(X_HOFFSET))
        .long      PLUS
        .long      SEMIS
#
#

#  ************
#  *   >PHA   *
#  ************
#
        .balign    4,0x00
N_TPHA:
        .long      4
        .ASCII      ">PHA"
        .balign    4,0x00
TPHA:
        .long    DOCOL
        .long    TPHA+HEADSIZE
        .long    0x0
        .long    TXFA
        .long    N_TPHA
        .long    0
        .long    0

        .long      LIT,(CW*(PH_OFFSET))
        .long      PLUS
        .long      SEMIS
#

#  *************
#  *   >BODY   *
#  *************
#
        .balign    4,0x00
N_TOBODY:
        .long      5
        .ASCII      ">BODY"
        .balign    4,0x00
TOBODY:
        .long    DOCOL
        .long    TOBODY+HEADSIZE
        .long    0x0
        .long    TPHA
        .long    N_TOBODY
        .long    0
        .long    0

        .long      TDFA, FETCH
        .long      CELLP           # Skip DOES> pointer.
        .long      SEMIS
#

#  *************
#  *   BODY>   *
#  *************
#
        .balign    4,0x00
N_BODYF:
        .long      5
        .ASCII      "BODY>"
        .balign    4,0x00
BODYF:
        .long    DOCOL
        .long    BODYF+HEADSIZE
        .long    0x0
        .long    TOBODY
        .long    N_BODYF
        .long    0
        .long    0

        .long      LIT,(CW*(BD_OFFSET))
        .long      LSUB
        .long      SEMIS
#

#  ************
#  *   >WID   *
#  ************
#
        .balign    4,0x00
N_TWID:
        .long      4
        .ASCII      ">WID"
        .balign    4,0x00
TWID:
        .long    DOCOL
        .long    TWID+HEADSIZE
        .long    0x0
        .long    BODYF
        .long    N_TWID
        .long    0
        .long    0

        .long      TOBODY
        .long      CELLP # Skip vfa link.
        .long      SEMIS
#

#  ************
#  *   >VFA   *
#  ************
#
        .balign    4,0x00
N_TVFA:
        .long      4
        .ASCII      ">VFA"
        .balign    4,0x00
TVFA:
        .long    DOCOL
        .long    TVFA+HEADSIZE
        .long    0x0
        .long    TWID
        .long    N_TVFA
        .long    0
        .long    0

        .long      TOBODY
        .long      SEMIS
#


#  ************
#  *   !CSP   *
#  ************
#
        .balign    4,0x00
N_SCSP:
        .long      4
        .ASCII      "!CSP"
        .balign    4,0x00
SCSP:
        .long    DOCOL
        .long    SCSP+HEADSIZE
        .long    0x0
        .long    TVFA
        .long    N_SCSP
        .long    0
        .long    0

        .long      SPFET
        .long      LCSP
        .long      STOR
        .long      SEMIS
#
#

#  **************
#  *   ?ERROR   *
#  **************
#
        .balign    4,0x00
N_QERR:
        .long      6
        .ASCII      "?ERROR"
        .balign    4,0x00
QERR:
        .long    DOCOL
        .long    QERR+HEADSIZE
        .long    0x0
        .long    SCSP
        .long    N_QERR
        .long    0
        .long    0

        .long      SWAP
        .long      ZBRAN
        .long      QERR1-.-CW #IF
        .long      PP, FETCH
        .long      SRC, FETCH
        .long      LWHERE, TSTOR
        .long      THROW
        .long      BRAN
        .long      QERR2-.-CW  #ELSE
QERR1:  .long      DROP    #THEN
QERR2:  .long      SEMIS
#

#  **************
#  *   ?ERRUR   *
#  **************
#
        .balign    4,0x00
N_QERRUR:
        .long      6
        .ASCII      "?ERRUR"
        .balign    4,0x00
QERRUR:
        .long    DOCOL
        .long    QERRUR+HEADSIZE
        .long    0x0
        .long    QERR
        .long    N_QERRUR
        .long    0
        .long    0

        .long      ZERO, MIN, LDUP, QERR
        .long      SEMIS
#


#  **************
#  *   ?DELIM   *
#  **************
#
        .balign    4,0x00
N_QDELIM:
        .long      6
        .ASCII      "?DELIM"
        .balign    4,0x00
QDELIM:
        .long    DOCOL
        .long    QDELIM+HEADSIZE
        .long    0x0
        .long    QERRUR
        .long    N_QDELIM
        .long    0
        .long    0

        .long      PPFET
        .long      QBL
        .long      ZEQU
        .long      LIT, 10, QERR
        .long      DROP
        .long      SEMIS
#

#  ************
#  *   ?CSP   *
#  ************
#
        .balign    4,0x00
N_QCSP:
        .long      4
        .ASCII      "?CSP"
        .balign    4,0x00
QCSP:
        .long    DOCOL
        .long    QCSP+HEADSIZE
        .long    0x0
        .long    QDELIM
        .long    N_QCSP
        .long    0
        .long    0

        .long      SPFET
        .long      LCSP
        .long      FETCH
        .long      LSUB
        .long      LIT, 20, QERR
        .long      SEMIS
#

#  *************
#  *   ?COMP   *
#  *************
#
        .balign    4,0x00
N_QCOMP:
        .long      5
        .ASCII      "?COMP"
        .balign    4,0x00
QCOMP:
        .long    DOCOL
        .long    QCOMP+HEADSIZE
        .long    0x0
        .long    QCSP
        .long    N_QCOMP
        .long    0
        .long    0

        .long      STATE
        .long      FETCH
        .long      ZEQU
        .long      LIT, 17, QERR
        .long      SEMIS
#

# 

#  **************
#  *   ?PAIRS   *
#  **************
#
        .balign    4,0x00
N_QPAIR:
        .long      6
        .ASCII      "?PAIRS"
        .balign    4,0x00
QPAIR:
        .long    DOCOL
        .long    QPAIR+HEADSIZE
        .long    0x0
        .long    QCOMP
        .long    N_QPAIR
        .long    0
        .long    0

        .long      LSUB
        .long      LIT, 19, QERR
        .long      SEMIS
#


# 
#
#
#
#

#  *********
#  *   [   *
#  *********
#
        .balign    4,0x00
N_LBRAC:
        .long      1
        .ASCII      "["
        .balign    4,0x00
LBRAC:
        .long    DOCOL
        .long    LBRAC+HEADSIZE
        .long    B_IMMED
        .long    QPAIR
        .long    N_LBRAC
        .long    0
        .long    0

        .long      ZERO
        .long      STATE
        .long      STOR
        .long      SEMIS
#

#  *********
#  *   ]   *
#  *********
#
        .balign    4,0x00
N_RBRAC:
        .long      1
        .ASCII      "]"
        .balign    4,0x00
RBRAC:
        .long    DOCOL
        .long    RBRAC+HEADSIZE
        .long    0x0
        .long    LBRAC
        .long    N_RBRAC
        .long    0
        .long    0

        .long      ONE
        .long      STATE
        .long      STOR
        .long      SEMIS
#

#  **************
#  *   HIDDEN   *
#  **************
#
        .balign    4,0x00
N_HIDDEN:
        .long      6
        .ASCII      "HIDDEN"
        .balign    4,0x00
HIDDEN:
        .long    DOCOL
        .long    HIDDEN+HEADSIZE
        .long    0x0
        .long    RBRAC
        .long    N_HIDDEN
        .long    0
        .long    0

        .long      TFFA
        .long      LIT,B_INVIS
        .long      TOGGL
        .long      SEMIS
#

#  ***********
#  *   HEX   *
#  ***********
#
        .balign    4,0x00
N_HEX:
        .long      3
        .ASCII      "HEX"
        .balign    4,0x00
HEX:
        .long    DOCOL
        .long    HEX+HEADSIZE
        .long    0x0
        .long    HIDDEN
        .long    N_HEX
        .long    0
        .long    0

        .long      LIT,16
        .long      BASE
        .long      STOR
        .long      SEMIS
#

#  ***************
#  *   DECIMAL   *
#  ***************
#
        .balign    4,0x00
N_DECA:
        .long      7
        .ASCII      "DECIMAL"
        .balign    4,0x00
DECA:
        .long    DOCOL
        .long    DECA+HEADSIZE
        .long    0x0
        .long    HEX
        .long    N_DECA
        .long    0
        .long    0

        .long      LIT,10
        .long      BASE
        .long      STOR
        .long      SEMIS
#

#  ***************
#  *   (;CODE)   *
#  ***************
#
        .balign    4,0x00
N_PSCOD:
        .long      7
        .ASCII      "(;CODE)"
        .balign    4,0x00
PSCOD:
        .long    DOCOL
        .long    PSCOD+HEADSIZE
        .long    0x0
        .long    DECA
        .long    N_PSCOD
        .long    0
        .long    0

        .long      FROMR
        .long      LATEST
        .long      TCFA
        .long      STOR
        .long      SEMIS
#
# 

#  **************
#  *   CREATE   *
#  **************
#
        .balign    4,0x00
N_LCREATE:
        .long      6
        .ASCII      "CREATE"
        .balign    4,0x00
LCREATE:
        .long    DOCOL
        .long    LCREATE+HEADSIZE
        .long    0x0
        .long    PSCOD
        .long    N_LCREATE
        .long    0
        .long    0

        .long      LNAME
        .long      PCREAT
        .long      LIT, HLNOOP, COMMA
        .long      PSCOD
DODOE:  LEA     %EBP,[%EBP - (CW*(1))] #Push HIP.
        MOV     [%EBP],%ESI
        MOV     %ESI,[%EAX+(CW*((D_HOFFSET)))] #NEW IP 
        LEA     %EAX,[%ESI+(CW*(1))]
        MOV     %ESI,[%ESI]
        PUSH    %EAX
        LODSD                  # NEXT
        JMP     DWORD PTR[%EAX]   
        .long      0       # In behalf of FIND-HEAD in decompilation
HLNOOP: .long      SEMIS
#

#  *************
#  *   DOES>   *
#  *************
#
        .balign    4,0x00
N_DOES:
        .long      5
        .ASCII      "DOES>"
        .balign    4,0x00
DOES:
        .long    DOCOL
        .long    DOES+HEADSIZE
        .long    0x0
        .long    LCREATE
        .long    N_DOES
        .long    0
        .long    0

        .long      FROMR
        .long      LATEST
        .long      TDFA
        .long      FETCH
        .long      STOR
        .long      SEMIS
#

#  *************
#  *   COUNT   *
#  *************
#
        .balign    4,0x00
N_COUNT:
        .long      5
        .ASCII      "COUNT"
        .balign    4,0x00
COUNT:
        .long    DOCOL
        .long    COUNT+HEADSIZE
        .long    0x0
        .long    DOES
        .long    N_COUNT
        .long    0
        .long    0

        .long      LDUP
        .long      ONEP
        .long      SWAP
        .long      CFET
        .long      SEMIS
#

#  *****************
#  *   -TRAILING   *
#  *****************
#
        .balign    4,0x00
N_DTRAI:
        .long      9
        .ASCII      "-TRAILING"
        .balign    4,0x00
DTRAI:
        .long    DOCOL
        .long    DTRAI+HEADSIZE
        .long    0x0
        .long    COUNT
        .long    N_DTRAI
        .long    0
        .long    0

        .long      LDUP
        .long      ZERO
        .long     XQDO
        .long      DTRA4-.-CW
DTRA1:  .long      OVER
        .long      OVER
        .long      PLUS
        .long      ONE
        .long      LSUB
        .long      CFET
        .long      QBL
        .long      ZEQU
        .long      ZBRAN
        .long      DTRA2-.-CW #IF
        .long      LLEAV
DTRA2:  .long      ONE
        .long      LSUB    # THEN
        .long     ONE, XPLOOP, ZBRAN
        .long      DTRA1-.-CW
        .long      UNLOOP     # LOOP
DTRA4:
        .long      SEMIS
#


#  **********
#  *   S"   *
#  **********
#
        .balign    4,0x00
N_SQUOT:
        .long      2
        .ASCII      "S\""
        .balign    4,0x00
SQUOT:
        .long    DOCOL
        .long    SQUOT+HEADSIZE
        .long    B_IMMED
        .long    DTRAI
        .long    N_SQUOT
        .long    0
        .long    0

        .long      DENQ
        .long      SEMIS
#
#

#  **********
#  *   ."   *
#  **********
#
        .balign    4,0x00
N_DOTQ:
        .long      2
        .ASCII      ".\""
        .balign    4,0x00
DOTQ:
        .long    DOCOL
        .long    DOTQ+HEADSIZE
        .long    B_IMMED
        .long    SQUOT
        .long    N_DOTQ
        .long    0
        .long    0

        .long      DENQ
        .long      STATE
        .long      FETCH
        .long      ZBRAN
        .long      DOTQ1-.-CW # IF
        .long      LIT, LTYPE, COMMA
        .long      BRAN
        .long      DOTQ2-.-CW
DOTQ1:
        .long      LTYPE
DOTQ2:
        .long      SEMIS   # THEN
#

#  **********
#  *   .(   *
#  **********
#
        .balign    4,0x00
N_DOTP:
        .long      2
        .ASCII      ".("
        .balign    4,0x00
DOTP:
        .long    DOCOL
        .long    DOTP+HEADSIZE
        .long    B_IMMED
        .long    DOTQ
        .long    N_DOTP
        .long    0
        .long    0

        .long      LIT, ')'
        .long      PARSE
        .long      LTYPE
        .long      SEMIS
#

#  ***************
#  *   SET-SRC   *
#  ***************
#
        .balign    4,0x00
N_SETSRC:
        .long      7
        .ASCII      "SET-SRC"
        .balign    4,0x00
SETSRC:
        .long    DOCOL
        .long    SETSRC+HEADSIZE
        .long    0x0
        .long    DOTP
        .long    N_SETSRC
        .long    0
        .long    0

        .long      OVER, PLUS
        .long      OVER, SRC, TSTOR
        .long      PP, STOR #  IN

#       DC      DOTS
        .long      SEMIS
#

#  ****************
#  *   EVALUATE   *
#  ****************
#
        .balign    4,0x00
N_EVALUATE:
        .long      8
        .ASCII      "EVALUATE"
        .balign    4,0x00
EVALUATE:
        .long    DOCOL
        .long    EVALUATE+HEADSIZE
        .long    0x0
        .long    SETSRC
        .long    N_EVALUATE
        .long    0
        .long    0

        .long      SAVE
        .long      SETSRC
        .long      LIT, INTER, CATCH
        .long      RESTO
        .long      THROW
        .long      SEMIS
#

#  ************
#  *   FILL   *
#  ************
#
        .balign    4,0x00
N_FILL:
        .long      4
        .ASCII      "FILL"
        .balign    4,0x00
FILL:
        .long    FILL+HEADSIZE
        .long    FILL+HEADSIZE
        .long    0x0
        .long    EVALUATE
        .long    N_FILL
        .long    0
        .long    0

#       Assume the extra segment points to the data segment 
        POP     %EAX      # FILL CHAR
        POP     %ECX      # FILL COUNT
        POP     %EDI      # BEGIN ADDR
        CLD             # INC DIRECTION
        REP     STOSB   #STORE BYTE
        LODSD                  # NEXT
        JMP     DWORD PTR[%EAX]   
#

#  ************
#  *   CORA   *
#  ************
#
        .balign    4,0x00
N_CORA:
        .long      4
        .ASCII      "CORA"
        .balign    4,0x00
CORA:
        .long    CORA+HEADSIZE
        .long    CORA+HEADSIZE
        .long    0x0
        .long    FILL
        .long    N_CORA
        .long    0
        .long    0

#       Assume the extra segment points to the data segment 
        MOV     %EDX,%ESI   #SAVE
        XOR     %EAX,%EAX   # Result
        POP     %ECX      # count
        POP     %EDI      # addr2
        POP     %ESI      # addr1
        CLD             # INC DIRECTION
        REPZ    CMPSB   # Compare BYTE
        JZ      CORA3
        MOV     %AL,1    #Remainder is already 0
        JNC     CORA3
        NEG     %EAX
CORA3:
        MOV     %ESI,%EDX  #Restore
        PUSH    %EAX
        LODSD                  # NEXT
        JMP     DWORD PTR[%EAX]   
#

#  **********
#  *   $^   *
#  **********
#
        .balign    4,0x00
N_SINDEX:
        .long      2
        .ASCII      "$^"
        .balign    4,0x00
SINDEX:
        .long    SINDEX+HEADSIZE
        .long    SINDEX+HEADSIZE
        .long    0x0
        .long    CORA
        .long    N_SINDEX
        .long    0
        .long    0

#       Assume the extra segment points to the data segment 
        POP     %EAX      # char
        POP     %ECX      # count
        POP     %EDI      # addr
        OR      %ESI,%ESI   #Clear zero flag.
        CLD             # INC DIRECTION
        REPNZ     SCASB   # Compare BYTE
        JZ      SINDEX1
        XOR     %EDI,%EDI    #Not found: 0
        INC     %EDI
SINDEX1:
        DEC     %EDI
        PUSH    %EDI
        LODSD                  # NEXT
        JMP     DWORD PTR[%EAX]   
#

#  **********
#  *   $/   *
#  **********
#
        .balign    4,0x00
N_SSLAS:
        .long      2
        .ASCII      "$/"
        .balign    4,0x00
SSLAS:
        .long    SSLAS+HEADSIZE
        .long    SSLAS+HEADSIZE
        .long    0x0
        .long    SINDEX
        .long    N_SSLAS
        .long    0
        .long    0

#       Assume the extra segment points to the data segment 
        POP     %EAX      # char
        POP     %ECX      # count
        MOV     %EBX,%ECX
        POP     %EDI      # addr
        OR      %EDI,%EDI   #Clear zero flag.
        MOV     %EDX,%EDI   # Copy
        CLD             # INC DIRECTION
        REPNZ     SCASB   # Compare BYTE
        JZ      SSLAS1
# Not present.
        PUSH    %ECX   # Nil pointer.
        JMP SHORT SSLAS2
SSLAS1:
        PUSH    %EDI
        SUB     %EBX,%ECX
        DEC     %EBX      #Delimiter is not part of first string.
SSLAS2:
        PUSH    %ECX   #Remaining length
        PUSH    %EDX   #Start of first string.
        PUSH    %EBX   #Skipped length.
        LODSD                  # NEXT
        JMP     DWORD PTR[%EAX]   
#

#  **********
#  *   $\   *
#  **********
#
        .balign    4,0x00
N_BSLAS:
        .long      2
        .ASCII      "$\\"
        .balign    4,0x00
BSLAS:
        .long    BSLAS+HEADSIZE
        .long    BSLAS+HEADSIZE
        .long    0x0
        .long    SSLAS
        .long    N_BSLAS
        .long    0
        .long    0

#       Assume the extra segment points to the data segment 
        POP     %EAX      # char
        POP     %ECX      # count
        MOV     %EBX,%ECX
        POP     %EDI      # addr
        MOV     %EDX,%EDI   # Copy
        ADD     %EDI,%ECX
        DEC     %EDI      #Last char & clear zero flag.
        STD    # DEC DIRECTION
        REPNZ     SCASB   # Compare BYTE
        JZ      BSLAS1
# Not present.
        PUSH    %ECX   # Nil pointer.
        INC     %EDI   #Start of end string
        JMP SHORT BSLAS2
BSLAS1:
        PUSH    %EDX   #Start of start string.
        INC     %EDI
        INC     %EDI   #Start of end string
        SUB     %EBX,%ECX
        DEC     %EBX      #Delimiter is not part of ending string.
BSLAS2:
        PUSH    %ECX   #Remaining length
        PUSH    %EDI
        PUSH    %EBX
        CLD
        LODSD                  # NEXT
        JMP     DWORD PTR[%EAX]   
#

#  *************
#  *   ERASE   *
#  *************
#
        .balign    4,0x00
N_LERASE:
        .long      5
        .ASCII      "ERASE"
        .balign    4,0x00
LERASE:
        .long    DOCOL
        .long    LERASE+HEADSIZE
        .long    0x0
        .long    BSLAS
        .long    N_LERASE
        .long    0
        .long    0

        .long      ZERO
        .long      FILL
        .long      SEMIS
#


#  *************
#  *   BLANK   *
#  *************
#
        .balign    4,0x00
N_BLANK:
        .long      5
        .ASCII      "BLANK"
        .balign    4,0x00
BLANK:
        .long    DOCOL
        .long    BLANK+HEADSIZE
        .long    0x0
        .long    LERASE
        .long    N_BLANK
        .long    0
        .long    0

        .long      LBL
        .long      FILL
        .long      SEMIS
#
#

#  ************
#  *   HOLD   *
#  ************
#
        .balign    4,0x00
N_HOLD:
        .long      4
        .ASCII      "HOLD"
        .balign    4,0x00
HOLD:
        .long    DOCOL
        .long    HOLD+HEADSIZE
        .long    0x0
        .long    BLANK
        .long    N_HOLD
        .long    0
        .long    0

        .long      LIT,-1
        .long      HLD
        .long      PSTOR
        .long      HLD
        .long      FETCH
        .long      CSTOR
        .long      SEMIS
#

#  ***********
#  *   PAD   *
#  ***********
#
        .balign    4,0x00
N_PAD:
        .long      3
        .ASCII      "PAD"
        .balign    4,0x00
PAD:
        .long    DOCOL
        .long    PAD+HEADSIZE
        .long    0x0
        .long    HOLD
        .long    N_PAD
        .long    0
        .long    0

        .long      HERE
# Allow for a one line name, a double binary number and some hold char's
        .long      LIT,84+128+64
        .long      PLUS
        .long      SEMIS
#


# 
#
#

#  ************
#  *   CHAR   *
#  ************
#
        .balign    4,0x00
N_LCHAR:
        .long      4
        .ASCII      "CHAR"
        .balign    4,0x00
LCHAR:
        .long    DOCOL
        .long    LCHAR+HEADSIZE
        .long    0x0
        .long    PAD
        .long    N_LCHAR
        .long    0
        .long    0

        .long      LNAME, DROP, CFET
        .long      SEMIS
#

#  **************
#  *   [CHAR]   *
#  **************
#
        .balign    4,0x00
N_BCHAR:
        .long      6
        .ASCII      "[CHAR]"
        .balign    4,0x00
BCHAR:
        .long    DOCOL
        .long    BCHAR+HEADSIZE
        .long    B_IMMED
        .long    LCHAR
        .long    N_BCHAR
        .long    0
        .long    0

        .long      LCHAR, LITER
        .long      SEMIS
#

#  ****************
#  *   (NUMBER)   *
#  ****************
#
        .balign    4,0x00
N_PNUMB:
        .long      8
        .ASCII      "(NUMBER)"
        .balign    4,0x00
PNUMB:
        .long    DOCOL
        .long    PNUMB+HEADSIZE
        .long    0x0
        .long    BCHAR
        .long    N_PNUMB
        .long    0
        .long    0

        .long      ZERO, ZERO
        .long      ZERO, DPL, STOR
NPNUM1:  .long      PPFET   # BEGIN
        .long      LDUP, LIT, ADOT, EQUAL
        .long      ZBRAN
        .long      NPNUM2-.-CW # case 1 
        .long      DROP, DPL, STOR
        .long      BRAN
        .long      NPNUM1-.-CW # continue 
NPNUM2:
        .long      LDUP, LIT, ',', EQUAL
        .long      ZBRAN
        .long      NPNUM3-.-CW # case 2 
        .long      TDROP
        .long      BRAN
        .long      NPNUM1-.-CW # continue 
NPNUM3:
        .long      LDUP, QBL, ZEQU
        .long      ZBRAN
        .long      NPNUM5-.-CW  # while 
        .long      NIP
        .long      BASE, FETCH, DIGIT
        .long      ZEQU
        .long      LIT, 10, QERR

        .long      SWAP
        .long      BASE
        .long      FETCH
        .long      USTAR
        .long      DROP
        .long      ROT
        .long      BASE
        .long      FETCH
        .long      USTAR
        .long      DPLUS
        .long      BRAN
        .long      NPNUM1-.-CW # repeat 
NPNUM5:
        .long      DROP, PP, STOR
        .long      SEMIS
#

#  **************
#  *   NUMBER   *
#  **************
#
        .balign    4,0x00
N_NUMB:
        .long      6
        .ASCII      "NUMBER"
        .balign    4,0x00
NUMB:
        .long    DOCOL
        .long    NUMB+HEADSIZE
        .long    0x0
        .long    PNUMB
        .long    N_NUMB
        .long    0
        .long    0

LNUMB:
        .long      LIT, -1, PP, PSTOR
        .long      PNUMB, SDLITE
        .long      SEMIS
#


#  ***************
#  *   >NUMBER   *
#  ***************
#
        .balign    4,0x00
N_TONUMB:
        .long      7
        .ASCII      ">NUMBER"
        .balign    4,0x00
TONUMB:
        .long    DOCOL
        .long    TONUMB+HEADSIZE
        .long    0x0
        .long    NUMB
        .long    N_TONUMB
        .long    0
        .long    0

        .long      TDUP, PLUS, TOR     # End available on return stack.
        .long      ZERO
        .long     XQDO
        .long      TONUM9-.-CW
TONUM1:
        .long      LDUP, CFET, BASE, FETCH, DIGIT
        .long      ZEQU
        .long      ZBRAN
        .long      TONUM4-.-CW # IF
        .long      DROP
        .long      LLEAV
TONUM4:
        .long      SWAP, TOR # Address out of the way.
        .long      SWAP
        .long      BASE
        .long      FETCH
        .long      USTAR
        .long      DROP
        .long      ROT
        .long      BASE
        .long      FETCH
        .long      USTAR
        .long      DPLUS
        .long      FROMR, ONEP     # Address back.
        .long     ONE, XPLOOP, ZBRAN
        .long      TONUM1-.-CW
        .long      UNLOOP 
TONUM9:
        .long      FROMR
        .long      OVER, LSUB
        .long      SEMIS
#
#

#  *************
#  *   FOUND   *
#  *************
#
        .balign    4,0x00
N_FOUND:
        .long      5
        .ASCII      "FOUND"
        .balign    4,0x00
FOUND:
        .long    DOCOL
        .long    FOUND+HEADSIZE
        .long    0x0
        .long    TONUMB
        .long    N_FOUND
        .long    0
        .long    0

        .long      CONTEXT, TOR
FOUND1: .long      RR, FETCH
#        DC      DOTS
        .long      PFIND, LDUP, ZEQU
        .long      ZBRAN
        .long      FOUND3-.-CW
        .long      DROP
        .long      RR, FETCH, LIT, ONLYBODY, LSUB #Was this ONLY?
        .long      ZBRAN
        .long      FOUND2-.-CW
        .long      FROMR, CELLP, TOR
        .long      BRAN
        .long      FOUND1-.-CW
FOUND2: .long      ZERO
FOUND3: .long      RDROP
        .long      NIP,NIP
        .long      SEMIS
#

#  ***************
#  *   PRESENT   *
#  ***************
#
        .balign    4,0x00
N_PRESENT:
        .long      7
        .ASCII      "PRESENT"
        .balign    4,0x00
PRESENT:
        .long    DOCOL
        .long    PRESENT+HEADSIZE
        .long    0x0
        .long    FOUND
        .long    N_PRESENT
        .long    0
        .long    0

        .long      LDUP, TOR
        .long      FOUND
        .long      LDUP
        .long      ZBRAN
        .long      PRES1-.-CW
        .long      LDUP
        .long      TNFA, FETCH, FETCH #  Get precise length.
        .long      RR, EQUAL
        .long      LAND
PRES1:
        .long      RDROP
        .long      SEMIS
#


# 
#


#  ***********
#  *   OPT   *
#  ***********
#
        .balign    4,0x00
N_OPT:
        .long      3
        .ASCII      "OPT"
        .balign    4,0x00
OPT:
        .long    DOCOL
        .long    OPT+HEADSIZE
        .long    0x0
        .long    PRESENT
        .long    N_OPT
        .long    0
        .long    0

        .long      SEMIS

#

#  **************
#  *   (FIND)   *
#  **************
#
        .balign    4,0x00
N_PFIND:
        .long      6
        .ASCII      "(FIND)"
        .balign    4,0x00
PFIND:
        .long    DOCOL
        .long    PFIND+HEADSIZE
        .long    0x0
        .long    OPT
        .long    N_PFIND
        .long    0
        .long    0

# The idea is to have a fast slim loop.
# The optimisation tries to skip, using the extra links.
# At all labels the stack is (sc, dea).
# PFIND0 : nothing known yet about dea
# PFCONTINUE : dea is disapproved, go on
# PFEXIT : dea is the match, or null.

PFIND0:                       #Start of loop.
        .long      NMATCH
        .long      ZEQU
        .long      ZBRAN
        .long      PFCONTINUE-.-CW
# ~MATCH is sure in determining a no-match,
# a possible match must be investigated closely.
#
# dea's name and sc have equal length
        .long      TDUP, TNFA, FETCH, FETCH
        .long      EQUAL
# dea is a prefix 
        .long      OVER, TFFA, FETCH
        .long      LIT, B_DENOT, LAND
        .long      LOR             # One of those must apply.
        .long      ZBRAN
        .long      PFCONTINUE-.-CW

# Discard still, if invisible ("smudged") or dummy.
        .long      LDUP, TFFA, FETCH
        .long      LIT, B_INVIS | B_DUMMY, LAND
        .long      ZEQU
        .long      ZBRAN
        .long      PFCONTINUE-.-CW
        .long      BRAN
        .long      PFEXIT-.-CW         # Match! 
PFCONTINUE:
        .long      TLFA, FETCH
        .long      LDUP, ZEQU
        .long      ZBRAN
        .long      PFIND0-.-CW      #End of loop.
# Fall through leaves zero, as required.

PFEXIT:
        .long      SEMIS
#

#  *************
#  *   ERROR   *
#  *************
#
        .balign    4,0x00
N_LERROR:
        .long      5
        .ASCII      "ERROR"
        .balign    4,0x00
LERROR:
        .long    DOCOL
        .long    LERROR+HEADSIZE
        .long    0x0
        .long    PFIND
        .long    N_LERROR
        .long    0
        .long    0

# Make sure the whole word is shown in case of denotations. 
        .long      LWHERE, TFET
        .long      OVER
        .long      ZBRAN
        .long      ERR2-.-CW      # Virgin: print empty string.
        .long      OVER, LIT, 20, LSUB
        .long      MAX
        .long      SWAP
        .long      ONEM         #Backup for if we are at next word.
ERR1:   .long      LDUP, CFET, QBL, ZEQU  #Show whole word!
        .long      ZBRAN
        .long      ERR2 -.-CW
        .long      ONEP
        .long      BRAN
        .long      ERR1 -.-CW
ERR2:
        .long      OVER, LSUB
        .long      ETYPE
        .long      SKIP
         .long      19
SB3: .ASCII      " ? ciforth ERROR # "
       .balign    4,0x00
        .long      LIT, SB3
        .long      LIT, 19
        .long      ETYPE
        .long      BASE, FETCH
        .long      DECA
        .long      OVER
        .long      STOD, ZERO, PDDOTR      #This is about (.) 
        .long      ETYPE
        .long      BASE, STOR
        .long      LIT, -256, MAX
        .long      LIT, 63, MIN
        .long      LIT, MESS, CATCH, DROP  #Prevent error loops 
        .long      SEMIS
#

#  *************
#  *   CATCH   *
#  *************
#
        .balign    4,0x00
N_CATCH:
        .long      5
        .ASCII      "CATCH"
        .balign    4,0x00
CATCH:
        .long    DOCOL
        .long    CATCH+HEADSIZE
        .long    0x0
        .long    LERROR
        .long    N_CATCH
        .long    0
        .long    0

        .long      SPFET, CELLP, TOR
        .long      HANDLER, FETCH, TOR
        .long      RPFET, HANDLER, STOR
        .long      EXEC
        .long      FROMR, HANDLER, STOR
        .long      RDROP, ZERO
        .long      SEMIS
#

#  *************
#  *   THROW   *
#  *************
#
        .balign    4,0x00
N_THROW:
        .long      5
        .ASCII      "THROW"
        .balign    4,0x00
THROW:
        .long    DOCOL
        .long    THROW+HEADSIZE
        .long    0x0
        .long    CATCH
        .long    N_THROW
        .long    0
        .long    0

        .long      LDUP
        .long      ZBRAN
        .long      THROW1-.-CW
        .long      HANDLER, FETCH, ZEQU
        .long      ZBRAN
        .long      THROW2-.-CW
        .long      LERROR
        .long      CLS
        .long      QUIT
THROW2:
        .long      HANDLER, FETCH, RPSTO
        .long      FROMR, HANDLER, STOR
        .long      FROMR, SWAP, TOR
        .long      SPSTO
        .long      FROMR
        .long      X
THROW1:
        .long      DROP
        .long      SEMIS
#


# 
#

# 

#

#  ***********
#  *   ID.   *
#  ***********
#
        .balign    4,0x00
N_IDDOT:
        .long      3
        .ASCII      "ID."
        .balign    4,0x00
IDDOT:
        .long    DOCOL
        .long    IDDOT+HEADSIZE
        .long    0x0
        .long    THROW
        .long    N_IDDOT
        .long    0
        .long    0

        .long      LDUP, TFFA, FETCH
        .long      LIT, B_DUMMY, LAND
        .long      ZBRAN
        .long      IDDOT1-.-CW
        .long      DROP
        .long      BRAN
        .long      IDDOT2-.-CW
IDDOT1:
        .long      TNFA
        .long      FETCH
        .long      SFET
        .long      LTYPE
        .long      SPACE
        .long      SPACE
        .long      SPACE
IDDOT2:
        .long      SEMIS
#
#

#  **************
#  *   HEADER   *
#  **************
#
        .balign    4,0x00
N_LHEAD:
        .long      6
        .ASCII      "HEADER"
        .balign    4,0x00
LHEAD:
        .long    DOCOL
        .long    LHEAD+HEADSIZE
        .long    0x0
        .long    IDDOT
        .long    N_LHEAD
        .long    0
        .long    0

        .long      LDUP
        .long      ZEQU
        .long      LIT, 5, QERR
        .long      TDUP
        .long      PRESENT
        .long      LDUP
        .long      ZBRAN
        .long      CREA1-.-CW #IF
        .long      TNFA, FETCH, SFET
        .long      ETYPE
        .long      LIT,4
        .long      MESS
        .long      X       #THEN
CREA1:  .long      DROP
 .long      LALIGN
        .long      SCOMMA
 .long      LALIGN
        .long      HERE,TOR

        .long      RR, TPHA, COMMA         #Code field.
        .long      RR, TPHA, COMMA         #Data field.
        .long      ZERO, COMMA #Flag field.
        .long      ZERO, COMMA #Link field.
        .long      COMMA   #Name field.


        .long      BLK, FETCH, LDUP, ZEQU
        .long      ZBRAN
        .long      CREA2-.-CW
        .long      DROP, PP, FETCH
CREA2:  .long      COMMA  #Source field.

        .long      ZERO, COMMA
        .long      FROMR
        .long      SEMIS
#

#  ************
#  *   LINK   *
#  ************
#
        .balign    4,0x00
N_LLINK:
        .long      4
        .ASCII      "LINK"
        .balign    4,0x00
LLINK:
        .long    DOCOL
        .long    LLINK+HEADSIZE
        .long    0x0
        .long    LHEAD
        .long    N_LLINK
        .long    0
        .long    0

        .long      TLFA
        .long      TDUP
        .long      FETCH
        .long      SWAP, TLFA, STOR
        .long      STOR
        .long      SEMIS
#

#  ****************
#  *   (CREATE)   *
#  ****************
#
        .balign    4,0x00
N_PCREAT:
        .long      8
        .ASCII      "(CREATE)"
        .balign    4,0x00
PCREAT:
        .long    DOCOL
        .long    PCREAT+HEADSIZE
        .long    0x0
        .long    LLINK
        .long    N_PCREAT
        .long    0
        .long    0

        .long      LHEAD
        .long      CURR, FETCH
        .long      LLINK
        .long      SEMIS
#

#  *****************
#  *   [COMPILE]   *
#  *****************
#
        .balign    4,0x00
N_BCOMP:
        .long      9
        .ASCII      "[COMPILE]"
        .balign    4,0x00
BCOMP:
        .long    DOCOL
        .long    BCOMP+HEADSIZE
        .long    B_IMMED
        .long    PCREAT
        .long    N_BCOMP
        .long    0
        .long    0

        .long      LNAME
        .long      PRESENT
        .long      LDUP
        .long      ZEQU
        .long      LIT, 16, QERR
        .long      TCFA
        .long      COMMA
        .long      SEMIS
#

#  ****************
#  *   POSTPONE   *
#  ****************
#
        .balign    4,0x00
N_POSTP:
        .long      8
        .ASCII      "POSTPONE"
        .balign    4,0x00
POSTP:
        .long    DOCOL
        .long    POSTP+HEADSIZE
        .long    B_IMMED
        .long    BCOMP
        .long    N_POSTP
        .long    0
        .long    0

        .long      LNAME
        .long      PRESENT
        .long      LDUP
        .long      ZEQU
        .long      LIT, 15, QERR
        .long      LDUP, TFFA, FETCH
        .long      LIT, B_IMMED, LAND, ZEQU
        .long      ZBRAN
        .long      POSTP1-.-CW
         .long      LIT, LIT, COMMA
         .long      COMMA
         .long      LIT, COMMA
POSTP1:
         .long      COMMA
        .long      SEMIS
#

#  ***************
#  *   LITERAL   *
#  ***************
#
        .balign    4,0x00
N_LITER:
        .long      7
        .ASCII      "LITERAL"
        .balign    4,0x00
LITER:
        .long    DOCOL
        .long    LITER+HEADSIZE
        .long    B_IMMED
        .long    POSTP
        .long    N_LITER
        .long    0
        .long    0

        .long      STATE
        .long      FETCH
        .long      ZBRAN
        .long      LITE1-.-CW #IF
        .long      LIT, LIT, COMMA
        .long      COMMA   #THEN
LITE1:  .long      SEMIS
#

#  ****************
#  *   DLITERAL   *
#  ****************
#
        .balign    4,0x00
N_DLITE:
        .long      8
        .ASCII      "DLITERAL"
        .balign    4,0x00
DLITE:
        .long    DOCOL
        .long    DLITE+HEADSIZE
        .long    B_IMMED
        .long    LITER
        .long    N_DLITE
        .long    0
        .long    0

        .long      STATE
        .long      FETCH
        .long      ZBRAN
        .long      DLIT1-.-CW # IF
        .long      SWAP
        .long      LITER
        .long      LITER   # THEN
DLIT1:  .long      SEMIS
#
#

#  *****************
#  *   SDLITERAL   *
#  *****************
#
        .balign    4,0x00
N_SDLITE:
        .long      9
        .ASCII      "SDLITERAL"
        .balign    4,0x00
SDLITE:
        .long    DOCOL
        .long    SDLITE+HEADSIZE
        .long    B_IMMED
        .long    DLITE
        .long    N_SDLITE
        .long    0
        .long    0

        .long      DPL
        .long      FETCH
        .long      ZBRAN
        .long      SDLIT1-.-CW # IF
        .long      DLITE
        .long      BRAN
        .long      SDLIT2-.-CW # IF
SDLIT1:
        .long      DROP, LITER
SDLIT2:
        .long      SEMIS
#


#  **************
#  *   ?STACK   *
#  **************
#
        .balign    4,0x00
N_QSTAC:
        .long      6
        .ASCII      "?STACK"
        .balign    4,0x00
QSTAC:
        .long    DOCOL
        .long    QSTAC+HEADSIZE
        .long    0x0
        .long    SDLITE
        .long    N_QSTAC
        .long    0
        .long    0

        .long      SPFET
        .long      SZERO
        .long      FETCH
        .long      SWAP
        .long      ULESS
        .long      ONE, QERR
        .long      SPFET
        .long      HERE
        .long      LIT,0x80
        .long      PLUS
        .long      ULESS
        .long      LIT, 7, QERR
        .long      SEMIS
        #
#
#

#  *****************
#  *   INTERPRET   *
#  *****************
#
        .balign    4,0x00
N_INTER:
        .long      9
        .ASCII      "INTERPRET"
        .balign    4,0x00
INTER:
        .long    DOCOL
        .long    INTER+HEADSIZE
        .long    0x0
        .long    QSTAC
        .long    N_INTER
        .long    0
        .long    0

INTE1:
        .long      LNAME
        .long      LDUP      # Zero length.
        .long      ZBRAN
        .long      INTE8-.-CW #WHILE
#       DC      DOTS
#       DC      TDUP, LTYPE
        .long      OVER, TOR       # Save old parse pointer.
        .long      FOUND
        .long      LDUP, ZEQU
        .long      LIT, 12, QERR
        .long      LDUP, TFFA, FETCH
        .long      LDUP, LIT, B_DENOT, LAND #Retain copy of flags.
        .long      ZBRAN
        .long      INTE3B-.-CW #IF
        .long      OVER, TNFA, FETCH, FETCH
        .long      RR, PLUS, PP, STOR  #Skip over prefix.
INTE3B:                  #THEN 
        .long      RDROP           # Drop old parse pointer.
        .long      LIT, B_IMMED, LAND
        .long      STATE, FETCH, ZEQU, LOR
        .long      ZBRAN
        .long      INTE3-.-CW #IF
        .long      EXEC
        .long      BRAN
        .long      INTE4-.-CW #IF
INTE3:
        .long      COMMA
                        #THEN
INTE4:
        .long      QSTAC
        .long      BRAN
        .long      INTE1-.-CW  #AGAIN
INTE8:  .long      DROP, DROP
        .long      SEMIS
#

#  *****************
#  *   IMMEDIATE   *
#  *****************
#
        .balign    4,0x00
N_IMMED:
        .long      9
        .ASCII      "IMMEDIATE"
        .balign    4,0x00
IMMED:
        .long    DOCOL
        .long    IMMED+HEADSIZE
        .long    0x0
        .long    INTER
        .long    N_IMMED
        .long    0
        .long    0

        .long      LATEST
        .long      TFFA
        .long      LIT, B_IMMED
        .long      TOGGL
        .long      SEMIS
#

#  **************
#  *   PREFIX   *
#  **************
#
        .balign    4,0x00
N_PREFX:
        .long      6
        .ASCII      "PREFIX"
        .balign    4,0x00
PREFX:
        .long    DOCOL
        .long    PREFX+HEADSIZE
        .long    0x0
        .long    IMMED
        .long    N_PREFX
        .long    0
        .long    0

        .long      LATEST
        .long      TFFA
        .long      LIT, B_DENOT
        .long      TOGGL
        .long      SEMIS
#

#  *****************
#  *   NAMESPACE   *
#  *****************
#
        .balign    4,0x00
N_VOCAB:
        .long      9
        .ASCII      "NAMESPACE"
        .balign    4,0x00
VOCAB:
        .long    DOCOL
        .long    VOCAB+HEADSIZE
        .long    0x0
        .long    PREFX
        .long    N_VOCAB
        .long    0
        .long    0

        .long      LCREATE
        .long      LATEST   # Link this DEA into VOC-LINK chain.
        .long      VOCL
        .long      FETCH
        .long      COMMA
        .long      VOCL
        .long      STOR
        .long      WORDLIST, DROP
        .long      DOES
DOVOC:
        .long      ALSO
        .long      CELLP   # Now points to WID. 
        .long      CONTEXT
        .long      STOR
        .long      SEMIS
        #
#
#   The link to task is a cold start value only.
#   It is updated each time a definition is
#   appended to the 'FORTH' namespace.
#

#  ****************
#  *   WORDLIST   *
#  ****************
#
        .balign    4,0x00
N_WORDLIST:
        .long      8
        .ASCII      "WORDLIST"
        .balign    4,0x00
WORDLIST:
        .long    DOCOL
        .long    WORDLIST+HEADSIZE
        .long    0x0
        .long    VOCAB
        .long    N_WORDLIST
        .long    0
        .long    0

        .long      HERE
        .long      ZERO, COMMA   # Dummy code field
        .long      ZERO, COMMA   # Dummy data field
        .long      LIT, B_DUMMY, COMMA # Dummy flag field
        .long      ZERO, COMMA #Link field: empty chain
        .long      LIT, (ZERO+(CW*(D_HOFFSET))), COMMA #Empty string for name.
        .long      ZERO, COMMA
        .long      ZERO, COMMA
        .long      SEMIS
        #
#

#

#  *******************
#  *   DEFINITIONS   *
#  *******************
#
        .balign    4,0x00
N_DEFIN:
        .long      11
        .ASCII      "DEFINITIONS"
        .balign    4,0x00
DEFIN:
        .long    DOCOL
        .long    DEFIN+HEADSIZE
        .long    0x0
        .long    WORDLIST
        .long    N_DEFIN
        .long    0
        .long    0

        .long      CONTEXT
        .long      FETCH
        .long      CURR
        .long      STOR
        .long      SEMIS
#

#  ************
#  *   ALSO   *
#  ************
#
        .balign    4,0x00
N_ALSO:
        .long      4
        .ASCII      "ALSO"
        .balign    4,0x00
ALSO:
        .long    DOCOL
        .long    ALSO+HEADSIZE
        .long    0x0
        .long    DEFIN
        .long    N_ALSO
        .long    0
        .long    0

        .long      CONTEXT, LDUP, CELLP
        .long      LIT, (CW*(16-1))
        .long      LMOVE
        .long      LIT, ONLYBODY  #End sentinel for array of word lists.
        .long      CONTEXT, LIT, (CW*(16)), PLUS
        .long      STOR #Trim sets of wordset.
        .long      SEMIS
#

#  ****************
#  *   PREVIOUS   *
#  ****************
#
        .balign    4,0x00
N_PREVI:
        .long      8
        .ASCII      "PREVIOUS"
        .balign    4,0x00
PREVI:
        .long    DOCOL
        .long    PREVI+HEADSIZE
        .long    0x0
        .long    ALSO
        .long    N_PREVI
        .long    0
        .long    0

        .long      CONTEXT, LDUP, CELLP, SWAP
        .long      LIT, (CW*(16))
        .long      LMOVE
        .long      SEMIS
#

#  *********
#  *   (   *
#  *********
#
        .balign    4,0x00
N_PAREN:
        .long      1
        .ASCII      "("
        .balign    4,0x00
PAREN:
        .long    DOCOL
        .long    PAREN+HEADSIZE
        .long    B_IMMED
        .long    PREVI
        .long    N_PAREN
        .long    0
        .long    0

        .long      LIT,')'
        .long      PARSE
        .long      TDROP
        .long      SEMIS
#
#

#  *********
#  *   \   *
#  *********
#
        .balign    4,0x00
N_BACKS:
        .long      1
        .ASCII      "\\"
        .balign    4,0x00
BACKS:
        .long    DOCOL
        .long    BACKS+HEADSIZE
        .long    B_IMMED
        .long    PAREN
        .long    N_BACKS
        .long    0
        .long    0

# Backup one character, just in case we are at the end of a line.
        .long      LIT, -1, PP, PSTOR
        .long      LIT,ALF
        .long      PARSE
        .long      TDROP
        .long      SEMIS
#

#  ************
#  *   QUIT   *
#  ************
#
        .balign    4,0x00
N_QUIT:
        .long      4
        .ASCII      "QUIT"
        .balign    4,0x00
QUIT:
        .long    DOCOL
        .long    QUIT+HEADSIZE
        .long    0x0
        .long    BACKS
        .long    N_QUIT
        .long    0
        .long    0

        .long      LBRAC
QUIT1:                  #BEGIN
        .long      RZERO
        .long      FETCH
        .long      RPSTO
        .long      LIT
        .long      PACCEP
        .long      CATCH
        .long      LDUP, LIT, -EPIPE, EQUAL
        .long      ZBRAN
        .long      ENDIF7-.-CW
        .long      BYE     #End of input, no error!
ENDIF7:
        .long      QERRUR
        .long      ONEP
        .long      SETSRC
        .long      INTER
        .long      OK
        .long      BRAN
        .long      QUIT1-.-CW  #AGAIN
        .long      SEMIS   #Unnecessary, but helpful for decompilation.
#

#  **********
#  *   OK   *
#  **********
#
        .balign    4,0x00
N_OK:
        .long      2
        .ASCII      "OK"
        .balign    4,0x00
OK:
        .long    DOCOL
        .long    OK+HEADSIZE
        .long    0x0
        .long    QUIT
        .long    N_OK
        .long    0
        .long    0

        .long      STATE
        .long      FETCH
        .long      ZEQU
        .long      ZBRAN
        .long      OK2-.-CW #IF
        .long      SKIP
         .long      3
SB4: .ASCII      " OK"
       .balign    4,0x00
        .long      LIT, SB4
        .long      LIT, 3
        .long      LTYPE
        .long      CR
OK2:
        .long      SEMIS
#

#  *************
#  *   ABORT   *
#  *************
#
        .balign    4,0x00
N_ABORT:
        .long      5
        .ASCII      "ABORT"
        .balign    4,0x00
ABORT:
        .long    DOCOL
        .long    ABORT+HEADSIZE
        .long    0x0
        .long    OK
        .long    N_ABORT
        .long    0
        .long    0

        .long      SZERO, FETCH, SPSTO
        .long      ZERO, HANDLER, STOR
        .long      QUIT
        .long      SEMIS   #Unnecessary, but helpful for decompilation.
#
#      WARM START VECTOR COMES HERE
#      For booting code we enter here, real mode and using the switchsegment.
#      BY control BREAK.
WARM_ENTRY:
# 

        
        JMP USINI+(CW*(-2))
#

#  ************
#  *   WARM   *
#  ************
#
        .balign    4,0x00
N_WARM:
        .long      4
        .ASCII      "WARM"
        .balign    4,0x00
WARM:
        .long    DOCOL
        .long    WARM+HEADSIZE
        .long    0x0
        .long    ABORT
        .long    N_WARM
        .long    0
        .long    0

        .long      INIT
        .long      SIGNON
        .long      ABORT
        .long      SEMIS   #Unnecessary, but helpful for decompilation.
#


#  ***************
#  *   OPTIONS   *
#  ***************
#
        .balign    4,0x00
N_OPTIONS:
        .long      7
        .ASCII      "OPTIONS"
        .balign    4,0x00
OPTIONS:
        .long    DOCOL
        .long    OPTIONS+HEADSIZE
        .long    0x0
        .long    WARM
        .long    N_OPTIONS
        .long    0
        .long    0

#       Execute option.
        .long      ARGS, FETCH, CELLP, CELLP, FETCH, LDUP
        .long      ZBRAN
        .long      OPT4-.-CW # No options  



#Supposedly on the stack is now a pointer to the option.
        .long      COUNT

        .long      LIT, '-', UNEQ
        .long      ZBRAN
        .long      OPT3-.-CW
        .long      LIT, 3, LDUP, LERROR
        .long      XCODE, STOR, BYE
OPT3:
        .long      CFET
        .long      LIT, 0x1F, LAND
        .long      LLOAD       #Supposedly stack-neutral.
        .long      ZERO, SWAP # Sign on suppressed.
OPT4:
        .long      DROP
        .long      SEMIS
#


#  ************
#  *   COLD   *
#  ************
#
        .balign    4,0x00
N_COLD:
        .long      4
        .ASCII      "COLD"
        .balign    4,0x00
COLD:
        .long    DOCOL
        .long    COLD5
        .long    0x0
        .long    OPTIONS
        .long    N_COLD
        .long    0
        .long    0

# Fill user area for single task.
        .long      LIT, USINI      # Fails if SP/RP are not usable!
        .long      LIT, USINI+(CW*(1)), FETCH
        .long      LIT, US
        .long      LCMOVE
        .long      INIT
COLD0:  .long      ONE            # Sign on wanted.
#

        .long      ZERO, LIT, TCGETS, TERMIO
        .long      LIT, ioctl, XOS, ZLESS
        .long      ZBRAN
        .long      COLD1-.-CW
        # Not a terminal. Cannot set the terminal, get rid of "OK".
        .long      DROP, ZERO      # Sign on suppressed.
        .long      LIT,TASK  # Suppress OK for pipes.
        .long      LIT,OK
        .long      LIT, (CW*(3)), LCMOVE
COLD1:
#
        .long      OPTIONS
        .long      ZBRAN
        .long      COLD5-.-CW
        .long      SIGNON    # Suppressed for scripting! Or any options.
COLD5:
        .long      ABORT
        .long      BYE     # In case of turnkey programs.
        .long      SEMIS   # Unnecessary, but helpful for decompilation.
#

#  *************
#  *   _INIT   *
#  *************
#
        .balign    4,0x00
N_INIT:
        .long      5
        .ASCII      "_INIT"
        .balign    4,0x00
INIT:
        .long    DOCOL
        .long    INIT+HEADSIZE
        .long    0x0
        .long    COLD
        .long    N_INIT
        .long    0
        .long    0

        .long      SZERO, FETCH, SPSTO
        .long      ZERO, HANDLER, STOR
        .long      DECA
        .long      ONLY, FORTH, DEFIN

        .long      LIT, 0, BLINI  #Default, open library file read-only!#
#
        .long      SEMIS
#
#

#  ***********
#  *   S>D   *
#  ***********
#
        .balign    4,0x00
N_STOD:
        .long      3
        .ASCII      "S>D"
        .balign    4,0x00
STOD:
        .long    STOD+HEADSIZE
        .long    STOD+HEADSIZE
        .long    0x0
        .long    INIT
        .long    N_STOD
        .long    0
        .long    0

        POP     %EDX      #S1
        SUB     %EAX,%EAX
        OR      %EDX,%EDX
        JNS     STOD1   #POS
        DEC     %EAX      #NEG
STOD1:
        PUSH    %EDX
        PUSH    %EAX
        LODSD                  # NEXT
        JMP     DWORD PTR[%EAX]   
#

#  ***********
#  *   ABS   *
#  ***********
#
        .balign    4,0x00
N_LABS:
        .long      3
        .ASCII      "ABS"
        .balign    4,0x00
LABS:
        .long    DOCOL
        .long    LABS+HEADSIZE
        .long    0x0
        .long    STOD
        .long    N_LABS
        .long    0
        .long    0

        .long      LDUP
        .long      ZLESS
        .long      ZBRAN
        .long      PM1-.-CW   #IF
        .long      NEGATE   #THEN
PM1:
        .long      SEMIS
#

#  ************
#  *   DABS   *
#  ************
#
        .balign    4,0x00
N_DABS:
        .long      4
        .ASCII      "DABS"
        .balign    4,0x00
DABS:
        .long    DOCOL
        .long    DABS+HEADSIZE
        .long    0x0
        .long    LABS
        .long    N_DABS
        .long    0
        .long    0

        .long      LDUP
        .long      ZLESS
        .long      ZBRAN
        .long      DPM1-.-CW  #IF
        .long      DNEGA   #THEN
DPM1:
        .long      SEMIS
#

#  ***********
#  *   MIN   *
#  ***********
#
        .balign    4,0x00
N_MIN:
        .long      3
        .ASCII      "MIN"
        .balign    4,0x00
MIN:
        .long    DOCOL
        .long    MIN+HEADSIZE
        .long    0x0
        .long    DABS
        .long    N_MIN
        .long    0
        .long    0

        .long      TDUP
        .long      GREAT
        .long      ZBRAN
        .long      MIN1-.-CW  #IF
        .long      SWAP    #THEN
MIN1:   .long      DROP
        .long      SEMIS
#

#  ***********
#  *   MAX   *
#  ***********
#
        .balign    4,0x00
N_MAX:
        .long      3
        .ASCII      "MAX"
        .balign    4,0x00
MAX:
        .long    DOCOL
        .long    MAX+HEADSIZE
        .long    0x0
        .long    MIN
        .long    N_MAX
        .long    0
        .long    0

        .long      TDUP
        .long      LESS
        .long      ZBRAN
        .long      MAX1-.-CW  #IF
        .long      SWAP    #THEN
MAX1:   .long      DROP
        .long      SEMIS
#

#  **************
#  *   LSHIFT   *
#  **************
#
        .balign    4,0x00
N_LSHIFT:
        .long      6
        .ASCII      "LSHIFT"
        .balign    4,0x00
LSHIFT:
        .long    LSHIFT+HEADSIZE
        .long    LSHIFT+HEADSIZE
        .long    0x0
        .long    MAX
        .long    N_LSHIFT
        .long    0
        .long    0

        POP     %ECX
        POP     %EAX
        SHL     %EAX,%CL
        PUSH    %EAX
        LODSD                  # NEXT
        JMP     DWORD PTR[%EAX]   
#

#  **************
#  *   RSHIFT   *
#  **************
#
        .balign    4,0x00
N_RSHIFT:
        .long      6
        .ASCII      "RSHIFT"
        .balign    4,0x00
RSHIFT:
        .long    RSHIFT+HEADSIZE
        .long    RSHIFT+HEADSIZE
        .long    0x0
        .long    LSHIFT
        .long    N_RSHIFT
        .long    0
        .long    0

        POP     %ECX
        POP     %EAX
        SHR     %EAX,%CL
        PUSH    %EAX
        LODSD                  # NEXT
        JMP     DWORD PTR[%EAX]   
#

#  **********
#  *   M*   *
#  **********
#
        .balign    4,0x00
N_MSTAR:
        .long      2
        .ASCII      "M*"
        .balign    4,0x00
MSTAR:
        .long    MSTAR+HEADSIZE
        .long    MSTAR+HEADSIZE
        .long    0x0
        .long    RSHIFT
        .long    N_MSTAR
        .long    0
        .long    0

        POP     %EAX
        POP     %EBX
        IMUL     %EBX      #SIGNED
        XCHG    %EAX,%EDX   #AX NOW = MSW
        PUSH    %EDX
        PUSH    %EAX
        LODSD                  # NEXT
        JMP     DWORD PTR[%EAX]             #STORE DOUBLE CELL
#

#  **************
#  *   SM/REM   *
#  **************
#
        .balign    4,0x00
N_MSLAS:
        .long      6
        .ASCII      "SM/REM"
        .balign    4,0x00
MSLAS:
        .long    MSLAS+HEADSIZE
        .long    MSLAS+HEADSIZE
        .long    0x0
        .long    MSTAR
        .long    N_MSLAS
        .long    0
        .long    0

        POP     %EBX      #DIVISOR
        POP     %EDX      #MSW OF DIVIDEND
        POP     %EAX      #LSW OF DIVIDEND
        IDIV     %EBX      # 32 BIT DIVIDE
        PUSH    %EDX
        PUSH    %EAX
        LODSD                  # NEXT
        JMP     DWORD PTR[%EAX]             #STORE QUOT/REM
#

#
#  **********
#  *   2/   *
#  **********
#
        .balign    4,0x00
N_TWOSL:
        .long      2
        .ASCII      "2/"
        .balign    4,0x00
TWOSL:
        .long    TWOSL+HEADSIZE
        .long    TWOSL+HEADSIZE
        .long    0x0
        .long    MSLAS
        .long    N_TWOSL
        .long    0
        .long    0

        POP     %EAX      #(S1) <- (S1) + (S2)
        SAR     %EAX,1
        PUSH    %EAX
        LODSD                  # NEXT
        JMP     DWORD PTR[%EAX]   

#

#
#  **********
#  *   2*   *
#  **********
#
        .balign    4,0x00
N_TWOST:
        .long      2
        .ASCII      "2*"
        .balign    4,0x00
TWOST:
        .long    TWOST+HEADSIZE
        .long    TWOST+HEADSIZE
        .long    0x0
        .long    TWOSL
        .long    N_TWOST
        .long    0
        .long    0

        POP     %EAX      #(S1) <- (S1) + (S2)
        SAL     %EAX,1
        PUSH    %EAX
        LODSD                  # NEXT
        JMP     DWORD PTR[%EAX]   
#

#  **********
#  *   1-   *
#  **********
#
        .balign    4,0x00
N_ONEM:
        .long      2
        .ASCII      "1-"
        .balign    4,0x00
ONEM:
        .long    DOCOL
        .long    ONEM+HEADSIZE
        .long    0x0
        .long    TWOST
        .long    N_ONEM
        .long    0
        .long    0

        .long      ONE, LSUB
        .long      SEMIS
#

#  **************
#  *   FM/MOD   *
#  **************
#
        .balign    4,0x00
N_FMSLAS:
        .long      6
        .ASCII      "FM/MOD"
        .balign    4,0x00
FMSLAS:
        .long    DOCOL
        .long    FMSLAS+HEADSIZE
        .long    0x0
        .long    ONEM
        .long    N_FMSLAS
        .long    0
        .long    0

        .long      LDUP, TOR
        .long      TDUP, LXOR, TOR
        .long      MSLAS
        .long      FROMR, ZLESS
        .long      ZBRAN
        .long      FMMOD1-.-CW
        .long      OVER
        .long      ZBRAN
        .long      FMMOD1-.-CW
        .long      ONE, LSUB
        .long      SWAP, FROMR, PLUS, SWAP
        .long      BRAN
        .long      FMMOD2-.-CW
FMMOD1:
        .long      RDROP
FMMOD2:
        .long      SEMIS
#

#  *********
#  *   *   *
#  *********
#
        .balign    4,0x00
N_STAR:
        .long      1
        .ASCII      "*"
        .balign    4,0x00
STAR:
        .long    DOCOL
        .long    STAR+HEADSIZE
        .long    0x0
        .long    FMSLAS
        .long    N_STAR
        .long    0
        .long    0

        .long      MSTAR
        .long      DROP
        .long      SEMIS
#

#  ************
#  *   /MOD   *
#  ************
#
        .balign    4,0x00
N_SLMOD:
        .long      4
        .ASCII      "/MOD"
        .balign    4,0x00
SLMOD:
        .long    DOCOL
        .long    SLMOD+HEADSIZE
        .long    0x0
        .long    STAR
        .long    N_SLMOD
        .long    0
        .long    0

        .long      TOR
        .long      STOD
        .long      FROMR
        .long      MSLAS
        .long      SEMIS
#

#  *********
#  *   /   *
#  *********
#
        .balign    4,0x00
N_SLASH:
        .long      1
        .ASCII      "/"
        .balign    4,0x00
SLASH:
        .long    DOCOL
        .long    SLASH+HEADSIZE
        .long    0x0
        .long    SLMOD
        .long    N_SLASH
        .long    0
        .long    0

        .long      SLMOD
        .long      NIP
        .long      SEMIS
#

#  ***********
#  *   MOD   *
#  ***********
#
        .balign    4,0x00
N_LMOD:
        .long      3
        .ASCII      "MOD"
        .balign    4,0x00
LMOD:
        .long    DOCOL
        .long    LMOD+HEADSIZE
        .long    0x0
        .long    SLASH
        .long    N_LMOD
        .long    0
        .long    0

        .long      SLMOD
        .long      DROP
        .long      SEMIS
#

#  *************
#  *   */MOD   *
#  *************
#
        .balign    4,0x00
N_SSMOD:
        .long      5
        .ASCII      "*/MOD"
        .balign    4,0x00
SSMOD:
        .long    DOCOL
        .long    SSMOD+HEADSIZE
        .long    0x0
        .long    LMOD
        .long    N_SSMOD
        .long    0
        .long    0

        .long      TOR
        .long      MSTAR
        .long      FROMR
        .long      MSLAS
        .long      SEMIS
#

#  **********
#  *   */   *
#  **********
#
        .balign    4,0x00
N_SSLA:
        .long      2
        .ASCII      "*/"
        .balign    4,0x00
SSLA:
        .long    DOCOL
        .long    SSLA+HEADSIZE
        .long    0x0
        .long    SSMOD
        .long    N_SSLA
        .long    0
        .long    0

        .long      SSMOD
        .long      NIP
        .long      SEMIS
#


#  *************
#  *   M/MOD   *
#  *************
#
        .balign    4,0x00
N_MSMOD1:
        .long      5
        .ASCII      "M/MOD"
        .balign    4,0x00
MSMOD1:
        .long    DOCOL
        .long    UDMSMOD 
        .long    0x0
        .long    SSLA
        .long    N_MSMOD1
        .long    0
        .long    0

#  ***************
#  *   UDM/MOD   *
#  ***************
#
        .balign    4,0x00
N_MSMOD:
        .long      7
        .ASCII      "UDM/MOD"
        .balign    4,0x00
MSMOD:
        .long    DOCOL
        .long    MSMOD+HEADSIZE
        .long    0x0
        .long    MSMOD1
        .long    N_MSMOD
        .long    0
        .long    0

UDMSMOD:
        .long      TOR
        .long      ZERO
        .long      RR
        .long      USLAS
        .long      FROMR
        .long      SWAP
        .long      TOR
        .long      USLAS
        .long      FROMR
        .long      SEMIS
#

#  **************
#  *   (LINE)   *
#  **************
#
        .balign    4,0x00
N_PLINE:
        .long      6
        .ASCII      "(LINE)"
        .balign    4,0x00
PLINE:
        .long    DOCOL
        .long    PLINE+HEADSIZE
        .long    0x0
        .long    MSMOD
        .long    N_PLINE
        .long    0
        .long    0

        .long      TOR
        .long      LIT,64
        .long      MSTAR
        .long      BBUF
        .long      FMSLAS
        .long      FROMR # This blocks, so is screens.
        .long      PLUS
        .long      BLOCK
        .long      PLUS
        .long      LIT,63
        .long      SEMIS
#

#  **************
#  *   ERRSCR   *
#  **************
#
        .balign    4,0x00
N_ERRSCR:
        .long      6
        .ASCII      "ERRSCR"
        .balign    4,0x00
ERRSCR:
        .long    DOVAR
        .long    ERRSCR+HEADSIZE
        .long    0x0
        .long    PLINE
        .long    N_ERRSCR
        .long    0
        .long    0

        .long ERRORSCREEN
#

#  ***************
#  *   MESSAGE   *
#  ***************
#
        .balign    4,0x00
N_MESS:
        .long      7
        .ASCII      "MESSAGE"
        .balign    4,0x00
MESS:
        .long    DOCOL
        .long    MESS+HEADSIZE
        .long    0x0
        .long    ERRSCR
        .long    N_MESS
        .long    0
        .long    0

        .long      LWARN
        .long      FETCH
        .long      ZBRAN
        .long      MESS1-.-CW #IF
        .long      ERRSCR, FETCH
        .long      PLINE, ONEP     # Also print the '\n' !
        .long      ETYPE
        .long      X
MESS1:                  #THEN
        .long      DROP
        .long      SEMIS
#

#  ***********
#  *   PC@   *
#  ***********
#
        .balign    4,0x00
N_PCFET:
        .long      3
        .ASCII      "PC@"
        .balign    4,0x00
PCFET:
        .long    PCFET+HEADSIZE
        .long    PCFET+HEADSIZE
        .long    0x0
        .long    MESS
        .long    N_PCFET
        .long    0
        .long    0

# FETCH CHARACTER (BYTE) FROM PORT
        POP     %EDX      # PORT ADDR
        XOR     %EAX,%EAX
        IN      %AL,%DX  # BYTE INPUT
        PUSH    %EAX
        LODSD                  # NEXT
        JMP     DWORD PTR[%EAX]   
#

#  ***********
#  *   PC!   *
#  ***********
#
        .balign    4,0x00
N_PCSTO:
        .long      3
        .ASCII      "PC!"
        .balign    4,0x00
PCSTO:
        .long    PCSTO+HEADSIZE
        .long    PCSTO+HEADSIZE
        .long    0x0
        .long    PCFET
        .long    N_PCSTO
        .long    0
        .long    0

        POP     %EDX      #PORT ADDR
        POP     %EAX      #DATA
        OUT     %DX,%AL   # BYTE OUTPUT
        LODSD                  # NEXT
        JMP     DWORD PTR[%EAX]   
#

#  ***********
#  *   PW@   *
#  ***********
#
        .balign    4,0x00
N_PFET:
        .long      3
        .ASCII      "PW@"
        .balign    4,0x00
PFET:
        .long    PFET+HEADSIZE
        .long    PFET+HEADSIZE
        .long    0x0
        .long    PCSTO
        .long    N_PFET
        .long    0
        .long    0

        POP     %EDX      #PORT ADDR
        IN      %AX,%DX  #WORD INPUT
        PUSH    %EAX
        LODSD                  # NEXT
        JMP     DWORD PTR[%EAX]   
#

#  ***********
#  *   PW!   *
#  ***********
#
        .balign    4,0x00
N_PSTO:
        .long      3
        .ASCII      "PW!"
        .balign    4,0x00
PSTO:
        .long    PSTO+HEADSIZE
        .long    PSTO+HEADSIZE
        .long    0x0
        .long    PFET
        .long    N_PSTO
        .long    0
        .long    0

        POP     %EDX      #PORT ADDR
        POP     %EAX      #DATA
        OUT     %DX,%AX   #WORD OUTPUT
        LODSD                  # NEXT
        JMP     DWORD PTR[%EAX]   
#

#  *************
#  *   _PREV   *
#  *************
#
        .balign    4,0x00
N__PREV:
        .long      5
        .ASCII      "_PREV"
        .balign    4,0x00
_PREV:
        .long    DOVAR
        .long    _PREV+HEADSIZE
        .long    0x0
        .long    PSTO
        .long    N__PREV
        .long    0
        .long    0

        .long      BUF1
#

#  *************
#  *   #BUFF   *
#  *************
#
        .balign    4,0x00
N_NOBUF:
        .long      5
        .ASCII      "#BUFF"
        .balign    4,0x00
NOBUF:
        .long    DOCON
        .long    NBUF
        .long    0x0
        .long    _PREV
        .long    N_NOBUF
        .long    0
        .long    0

#

#  ************
#  *   +BUF   *
#  ************
#
        .balign    4,0x00
N_PBUF:
        .long      4
        .ASCII      "+BUF"
        .balign    4,0x00
PBUF:
        .long    DOCOL
        .long    PBUF+HEADSIZE
        .long    0x0
        .long    NOBUF
        .long    N_PBUF
        .long    0
        .long    0

        .long      BBUF, CELLP, CELLP
        .long      PLUS,LDUP
        .long      _LIMIT,EQUAL
        .long      ZBRAN
        .long      PBUF1-.-CW
        .long      DROP,_FIRST
PBUF1:  .long      LDUP, _PREV, FETCH, LSUB
        .long      SEMIS
#

#  **************
#  *   UPDATE   *
#  **************
#
        .balign    4,0x00
N_UPDAT:
        .long      6
        .ASCII      "UPDATE"
        .balign    4,0x00
UPDAT:
        .long    DOCOL
        .long    UPDAT+HEADSIZE
        .long    0x0
        .long    PBUF
        .long    N_UPDAT
        .long    0
        .long    0

        .long      _PREV, FETCH
        .long      LDUP, CELLP,CELLP
        .long      SWAP, FETCH
        .long      LOFFSET,  FETCH, PLUS
        .long      BLWRITE
        .long      SEMIS
#

#  *********************
#  *   EMPTY-BUFFERS   *
#  *********************
#
        .balign    4,0x00
N_MTBUF:
        .long      13
        .ASCII      "EMPTY-BUFFERS"
        .balign    4,0x00
MTBUF:
        .long    DOCOL
        .long    MTBUF+HEADSIZE
        .long    0x0
        .long    UPDAT
        .long    N_MTBUF
        .long    0
        .long    0

        .long      _FIRST
        .long      _LIMIT,OVER
        .long      LSUB,LERASE
        .long      _FIRST, _PREV, STOR
        .long      SEMIS
        #
#

#  ****************
#  *   (BUFFER)   *
#  ****************
#
        .balign    4,0x00
N_BUFFER:
        .long      8
        .ASCII      "(BUFFER)"
        .balign    4,0x00
BUFFER:
        .long    DOCOL
        .long    BUFFER+HEADSIZE
        .long    0x0
        .long    MTBUF
        .long    N_BUFFER
        .long    0
        .long    0

# Find the buffer, if it is already here.
    .long      _PREV, FETCH
BUFFER1:
    .long          TOR, RR, FETCH, OVER, EQUAL
    .long      ZBRAN
        .long      BUFFER3-.-CW
    .long        DROP, FROMR, EXIT
BUFFER3:
    .long          FROMR
    .long      PBUF, ZEQU
    .long      ZBRAN
        .long      BUFFER1-.-CW
# We are left with the previous buffer.
BUFFER2:
    .long       PBUF, ZEQU, LIT, 48, QERR  # No free buffers available.
    .long       LDUP, CELLP, FETCH
    .long       LIT, -1, GREAT        # Unlocked buffer?
    .long      ZBRAN
        .long      BUFFER2-.-CW
    .long       TOR
# Fill in the house keeping.
    .long       RR, STOR
    .long       ZERO, RR, CELLP, STOR

    .long       RR, _PREV, STOR
    .long       FROMR
    .long  SEMIS
#


#  *************
#  *   BLOCK   *
#  *************
#
        .balign    4,0x00
N_BLOCK:
        .long      5
        .ASCII      "BLOCK"
        .balign    4,0x00
BLOCK:
        .long    DOCOL
        .long    BLOCK+HEADSIZE
        .long    0x0
        .long    BUFFER
        .long    N_BLOCK
        .long    0
        .long    0


        .long      BUFFER
        .long      LDUP, CELLP, FETCH, ONE, LAND, ZEQU
        .long      ZBRAN
        .long      BLOCK1-.-CW
        .long      LDUP, CELLP, CELLP
        .long      OVER, FETCH
        .long      LOFFSET,  FETCH, PLUS
        .long      BLREAD
        .long      LDUP, CELLP , ONE, TOGGL
BLOCK1:
        .long      LDUP, _PREV, STOR
        .long      CELLP, CELLP
        .long      SEMIS
#

#  *************
#  *   FLUSH   *
#  *************
#
        .balign    4,0x00
N_FLUSH:
        .long      5
        .ASCII      "FLUSH"
        .balign    4,0x00
FLUSH:
        .long    DOCOL
        .long    (MTBUF+HEADSIZE)
        .long    0x0
        .long    BLOCK
        .long    N_FLUSH
        .long    0
        .long    0


# Unlock all buffers
        .long      _LIMIT
        .long      _FIRST, CELLP
        .long     XDO
        .long      FLUS2-.-CW
FLUS1:  .long      ZERO, IDO, STOR
        .long      LIT,(KBBUF+2*CW)
        .long      PLOOP
        .long      (FLUS1-.)
FLUS2:
        .long      SEMIS
#

#  ************
#  *   SAVE   *
#  ************
#
        .balign    4,0x00
N_SAVE:
        .long      4
        .ASCII      "SAVE"
        .balign    4,0x00
SAVE:
        .long    DOCOL
        .long    SAVE+HEADSIZE
        .long    0x0
        .long    FLUSH
        .long    N_SAVE
        .long    0
        .long    0

        .long      FROMR
        .long      SRC, TFET
        .long      PP, FETCH
        .long      TOR, TOR, TOR
        .long      TOR
        .long SEMIS
#

#  ***************
#  *   RESTORE   *
#  ***************
#
        .balign    4,0x00
N_RESTO:
        .long      7
        .ASCII      "RESTORE"
        .balign    4,0x00
RESTO:
        .long    DOCOL
        .long    RESTO+HEADSIZE
        .long    0x0
        .long    SAVE
        .long    N_RESTO
        .long    0
        .long    0

        .long      FROMR
        .long      FROMR, FROMR, FROMR
        .long      PP, STOR
        .long      SRC, TSTOR
        .long      TOR
        .long SEMIS
#


# 
#

# 
#
#

#  ************
#  *   LOCK   *
#  ************
#
        .balign    4,0x00
N_LLOCK:
        .long      4
        .ASCII      "LOCK"
        .balign    4,0x00
LLOCK:
        .long    DOCOL
        .long    LLOCK+HEADSIZE
        .long    0x0
        .long    RESTO
        .long    N_LLOCK
        .long    0
        .long    0

        .long      BUFFER
        .long      CELLP
        .long      LIT, -2, SWAP, PSTOR
        .long      SEMIS
#

#  **************
#  *   UNLOCK   *
#  **************
#
        .balign    4,0x00
N_LUNLOCK:
        .long      6
        .ASCII      "UNLOCK"
        .balign    4,0x00
LUNLOCK:
        .long    DOCOL
        .long    LUNLOCK+HEADSIZE
        .long    0x0
        .long    LLOCK
        .long    N_LUNLOCK
        .long    0
        .long    0

        .long      BUFFER
        .long      CELLP
        .long      TWO, SWAP, PSTOR
        .long      SEMIS
#

#  ************
#  *   LOAD   *
#  ************
#
        .balign    4,0x00
N_LLOAD:
        .long      4
        .ASCII      "LOAD"
        .balign    4,0x00
LLOAD:
        .long    DOCOL
        .long    LLOAD+HEADSIZE
        .long    0x0
        .long    LUNLOCK
        .long    N_LLOAD
        .long    0
        .long    0

        .long      SAVE
        .long      TOR
        .long      RR, LIT, LLOCK, CATCH  # Be careful about always unlocking.
        .long      LDUP, ZEQU
        .long      ZBRAN
        .long      LOAD5-.-CW
        .long      DROP
        .long      RR, BLOCK     # Can't fail anymore. 
        .long      BBUF
        .long      SETSRC
        .long      LIT, INTER, CATCH
        .long      RR, LUNLOCK
LOAD5:
        .long      RDROP
        .long      RESTO
        .long      THROW
        .long      SEMIS
#

#  ************
#  *   THRU   *
#  ************
#
        .balign    4,0x00
N_THRU:
        .long      4
        .ASCII      "THRU"
        .balign    4,0x00
THRU:
        .long    DOCOL
        .long    THRU+HEADSIZE
        .long    0x0
        .long    LLOAD
        .long    N_THRU
        .long    0
        .long    0

        .long      ONEP, SWAP
        .long     XDO
        .long      THRU2-.-CW
THRU1:
        .long      IDO, LLOAD
        .long     ONE, XPLOOP, ZBRAN
        .long      THRU1-.-CW
        .long      UNLOOP 
THRU2:
        .long      SEMIS
#

#

#  ***********
#  *   BLK   *
#  ***********
#
        .balign    4,0x00
N_BLK:
        .long      3
        .ASCII      "BLK"
        .balign    4,0x00
BLK:
        .long    DOCOL
        .long    BLK+HEADSIZE
        .long    0x0
        .long    THRU
        .long    N_BLK
        .long    0
        .long    0

        .long      PP, FETCH
        .long      _FIRST, _LIMIT, WITHIN
        .long      SRC, TFET, LSUB
        .long      LIT, 1024, EQUAL, LAND
        .long      ZBRAN
        .long      BLK1-.-CW
        .long      SRC, FETCH, TWO, LCELLS, LSUB, FETCH
        .long      BRAN
        .long      BLK2-.-CW
BLK1:
        .long      ZERO
BLK2:
        .long      PBLK, STOR
        .long      PBLK
        .long      SEMIS
#

#
#
# 



#  ***********
#  *   XOS   *
#  ***********
#
        .balign    4,0x00
N_XOS:
        .long      3
        .ASCII      "XOS"
        .balign    4,0x00
XOS:
        .long    XOS+HEADSIZE
        .long    XOS+HEADSIZE
        .long    0x0
        .long    BLK
        .long    N_XOS
        .long    0
        .long    0


        POP     %EAX        # Function number
        POP     %EDX        # Third parameter, or dummy
        POP     %ECX        # Second parameter, or dummy
        POP     %EBX        # First parameter.
        INT     0x80        # Generic call on LINUX 
#
#
        PUSH    %EAX
        LODSD                  # NEXT
        JMP     DWORD PTR[%EAX]        # Positive means okay. Negative means -errno.
#
 # 
#


#  ************
#  *   XOS5   *
#  ************
#
        .balign    4,0x00
N_XOS5:
        .long      4
        .ASCII      "XOS5"
        .balign    4,0x00
XOS5:
        .long    XOS5+HEADSIZE
        .long    XOS5+HEADSIZE
        .long    0x0
        .long    XOS
        .long    N_XOS5
        .long    0
        .long    0


        LEA     %EBP,[%EBP - (CW*(1))] #Save HIP on return stack.
        MOV     [%EBP],%ESI

        POP     %EAX        # Function number
        POP     %EDI        # 5th parameter, or dummy
        POP     %ESI        # 4th parameter, or dummy
        POP     %EDX        # Third parameter, or dummy
        POP     %ECX        # Second parameter, or dummy
        POP     %EBX        # First parameter.
        INT     0x80        # Generic call on LINUX 
        MOV     %ESI,[%EBP]    # Restore
        LEA     %EBP,[%EBP+(CW*(1))]
#
#

        PUSH    %EAX
        LODSD                  # NEXT
        JMP     DWORD PTR[%EAX]        # Positive means okay. Negative means -errno.
#
 # 
#

#  **********
#  *   MS   *
#  **********
#
        .balign    4,0x00
N_MS:
        .long      2
        .ASCII      "MS"
        .balign    4,0x00
MS:
        .long    DOCOL
        .long    MS+HEADSIZE
        .long    0x0
        .long    XOS5
        .long    N_MS
        .long    0
        .long    0

# Use select because nanosleep has a 20 MS resolution. 
        .long      LIT, 1000, LIT, 1000000, SSMOD # (ms -- us s) 
        .long      LIT, TV, TSTOR
        .long      ZERO, ZERO
        .long      ZERO, ZERO
        .long      LIT, TV
       .long      LIT, _newselect, XOS5

        .long      QERRUR
        .long      SEMIS
TV:     .SPACE   (CW*(2))    # A time_t and a long, both cells.
#
# 
# 
        #
#------------------------------------
#       CONSOLE RELEATED I/O         
#------------------------------------
#


# 
# 


#  *****************
#  *   RW-BUFFER   *
#  *****************
#
        .balign    4,0x00
N_RWBUF:
        .long      9
        .ASCII      "RW-BUFFER"
        .balign    4,0x00
RWBUF:
        .long    DOVAR
        .long    RWBUF+HEADSIZE
        .long    0x0
        .long    MS
        .long    N_RWBUF
        .long    0
        .long    0

        .SPACE   0x1000
# 
# 
#


#  ***********
#  *   ZEN   *
#  ***********
#
        .balign    4,0x00
N_ZEN:
        .long      3
        .ASCII      "ZEN"
        .balign    4,0x00
ZEN:
        .long    DOCOL
        .long    ZEN+HEADSIZE
        .long    0x0
        .long    RWBUF
        .long    N_ZEN
        .long    0
        .long    0

        .long      RWBUF, SSTOR
        .long      ZERO, RWBUF, CHAPP
        .long      RWBUF, CELLP
        .long      SEMIS
# 
# 
# 
# 
#
#


#  ******************
#  *   WRITE-FILE   *
#  ******************
#
        .balign    4,0x00
N_WFILE:
        .long      10
        .ASCII      "WRITE-FILE"
        .balign    4,0x00
WFILE:
        .long    DOCOL
        .long    WFILE+HEADSIZE
        .long    0x0
        .long    ZEN
        .long    N_WFILE
        .long    0
        .long    0

        .long      ROT, ROT
        .long      LIT, write, XOS
        .long      ZERO, MIN
        .long      SEMIS
# 
# 
# 
# 
#


#  *****************
#  *   OPEN-FILE   *
#  *****************
#
        .balign    4,0x00
N_OFILE:
        .long      9
        .ASCII      "OPEN-FILE"
        .balign    4,0x00
OFILE:
        .long    DOCOL
        .long    OFILE+HEADSIZE
        .long    0x0
        .long    WFILE
        .long    N_OFILE
        .long    0
        .long    0

        .long      TOR, ZEN, FROMR
#
        .long      X
        .long      LIT, open, XOS
        .long      LDUP, ZERO, MIN
        .long      SEMIS
#
#
#
#


#  ******************
#  *   CLOSE-FILE   *
#  ******************
#
        .balign    4,0x00
N_CFILE:
        .long      10
        .ASCII      "CLOSE-FILE"
        .balign    4,0x00
CFILE:
        .long    DOCOL
        .long    CFILE+HEADSIZE
        .long    0x0
        .long    OFILE
        .long    N_CFILE
        .long    0
        .long    0

        .long      X, X
        .long      LIT, close, XOS
        .long      SEMIS
#
#
#
#
#


#  *******************
#  *   CREATE-FILE   *
#  *******************
#
        .balign    4,0x00
N_CREATEF:
        .long      11
        .ASCII      "CREATE-FILE"
        .balign    4,0x00
CREATEF:
        .long    DOCOL
        .long    CREATEF+HEADSIZE
        .long    0x0
        .long    CFILE
        .long    N_CREATEF
        .long    0
        .long    0

        .long      TOR, TDUP, DFILE, DROP, ZEN
        .long      LIT, 0755, FROMR, LIT, 8, RSHIFT, LXOR
        .long      X
        .long      LIT, creat, XOS
        .long      LDUP, ZERO, MAX, SWAP, ZERO, MIN
        .long      SEMIS
#
#
#
#


#  *******************
#  *   DELETE-FILE   *
#  *******************
#
        .balign    4,0x00
N_DFILE:
        .long      11
        .ASCII      "DELETE-FILE"
        .balign    4,0x00
DFILE:
        .long    DOCOL
        .long    DFILE+HEADSIZE
        .long    0x0
        .long    CREATEF
        .long    N_DFILE
        .long    0
        .long    0

        .long      ZEN, X, X
        .long      LIT, unlink, XOS
        .long      SEMIS
#
#
#
#


#  *****************
#  *   READ-FILE   *
#  *****************
#
        .balign    4,0x00
N_RFILE:
        .long      9
        .ASCII      "READ-FILE"
        .balign    4,0x00
RFILE:
        .long    DOCOL
        .long    RFILE+HEADSIZE
        .long    0x0
        .long    DFILE
        .long    N_RFILE
        .long    0
        .long    0

        .long      ROT, ROT
        .long      LIT, read, XOS
        .long      LDUP, ZERO, MAX, SWAP, ZERO, MIN
        .long      SEMIS
# 
# 
# 
# 
#

#

#  *********************
#  *   FILE-POSITION   *
#  *********************
#
        .balign    4,0x00
N_FPFILE:
        .long      13
        .ASCII      "FILE-POSITION"
        .balign    4,0x00
FPFILE:
        .long    DOCOL
        .long    FPFILE+HEADSIZE
        .long    0x0
        .long    RFILE
        .long    N_FPFILE
        .long    0
        .long    0

        .long      ZERO
        .long      LIT, SEEK_CUR
        .long      LIT, lseek, XOS
        .long      ZERO     #Double precision return result
        .long      OVER, LDUP
        .long      NEGATE, LIT, 256, ULESS, LAND # Heuristic 
        .long      SEMIS
 # 
#
#
#


#  ***********************
#  *   REPOSITION-FILE   *
#  ***********************
#
        .balign    4,0x00
N_PFILE:
        .long      15
        .ASCII      "REPOSITION-FILE"
        .balign    4,0x00
PFILE:
        .long    DOCOL
        .long    PFILE+HEADSIZE
        .long    0x0
        .long    FPFILE
        .long    N_PFILE
        .long    0
        .long    0

        .long      TOR, DROP, FROMR   #Convert to single precision.
        .long      SWAP
        .long      LIT, SEEK_SET
        .long      LIT, lseek, XOS
        .long      ZERO, MIN
        .long      SEMIS
 # 
#
#
#
#
# 

#  ****************
#  *   GET-FILE   *
#  ****************
#
        .balign    4,0x00
N_GETFILE:
        .long      8
        .ASCII      "GET-FILE"
        .balign    4,0x00
GETFILE:
        .long    DOCOL
        .long    GETFILE+HEADSIZE
        .long    0x0
        .long    PFILE
        .long    N_GETFILE
        .long    0
        .long    0

    .long LALIGN
        .long      TDUP, SCOMMA, DROP
        .long      LIT, 'F'+(0x100*('i'+0x100*('L'+0x100*'e'))), COMMA #Magic number.
        .long      ZERO, OFILE, THROW, TOR
        .long      HERE, LDUP
        .long      SPFET, HERE, LSUB
        .long      LIT, 6, SLASH, LDUP, ALLOT
        .long      LIT, 1000, LSUB
        .long      RR, RFILE, THROW
        .long      FROMR, CFILE, THROW
        .long      TDUP, PLUS, LDP, STOR        # No allocation if it fails.
    .long LALIGN
        .long      SEMIS
#

#  ****************
#  *   PUT-FILE   *
#  ****************
#
        .balign    4,0x00
N_PUTFILE:
        .long      8
        .ASCII      "PUT-FILE"
        .balign    4,0x00
PUTFILE:
        .long    DOCOL
        .long    PUTFILE+HEADSIZE
        .long    0x0
        .long    GETFILE
        .long    N_PUTFILE
        .long    0
        .long    0

        .long      ZERO
        .long      CREATEF, THROW
        .long      LDUP, TOR
        .long      WFILE, THROW
        .long      FROMR, CFILE, THROW
        .long      SEMIS
#

#  ****************
#  *   INCLUDED   *
#  ****************
#
        .balign    4,0x00
N_INCLUDED:
        .long      8
        .ASCII      "INCLUDED"
        .balign    4,0x00
INCLUDED:
        .long    DOCOL
        .long    INCLUDED+HEADSIZE
        .long    0x0
        .long    PUTFILE
        .long    N_INCLUDED
        .long    0
        .long    0

        .long      HERE, TOR
        .long      LIT, GETFILE, CATCH
        .long      LDUP
        .long      ZBRAN
        .long      INCLUD1-.-CW
        .long      FROMR, LDP, STOR
        .long      THROW
        .long      BRAN
        .long      INCLUD2-.-CW
INCLUD1:
        .long      RDROP, DROP
INCLUD2:
        .long      EVALUATE
        .long      SEMIS
#

#  ***************
#  *   INCLUDE   *
#  ***************
#
        .balign    4,0x00
N_LINCLUD:
        .long      7
        .ASCII      "INCLUDE"
        .balign    4,0x00
LINCLUD:
        .long    DOCOL
        .long    LINCLUD+HEADSIZE
        .long    0x0
        .long    INCLUDED
        .long    N_LINCLUD
        .long    0
        .long    0

        .long      LNAME, INCLUDED
        .long      SEMIS
#

#  ******************
#  *   REFILL-TIB   *
#  ******************
#
        .balign    4,0x00
N_REFTIB:
        .long      10
        .ASCII      "REFILL-TIB"
        .balign    4,0x00
REFTIB:
        .long    DOCOL
        .long    REFTIB+HEADSIZE
        .long    0x0
        .long    LINCLUD
        .long    N_REFTIB
        .long    0
        .long    0

        .long      REMAIND, FETCH, TOR
        .long      TIB, FETCH, RR, PLUS
        .long      LIT, TIBSIZE, RR, LSUB
        .long      ZERO
        .long      RFILE
        .long      QERRUR
        .long      LDUP, ZEQU, LIT, -EPIPE, LAND # Presumably end of pipe.
        .long      QERRUR
        .long      TIB, FETCH, SWAP, FROMR, PLUS
        .long      REMAIND, TSTOR
        .long      SEMIS
#
#
#
#

# All I/O goes through the modern device (Unix-standard)
# device I/O. Code fields are filled in during bootup. 
# Lower case labels starting with "c_.." are c-supplied facilities.

#  **************
#  *   ACCEPT   *
#  **************
#
        .balign    4,0x00
N_ACCEP:
        .long      6
        .ASCII      "ACCEPT"
        .balign    4,0x00
ACCEP:
        .long    DOCOL
        .long    ACCEP+HEADSIZE
        .long    0x0
        .long    REFTIB
        .long    N_ACCEP
        .long    0
        .long    0

        .long      PACCEP
        .long      TSWAP, ROT, MIN
        .long      LDUP, TOR, LMOVE, FROMR
        .long      SEMIS
#

#  ****************
#  *   (ACCEPT)   *
#  ****************
#
        .balign    4,0x00
N_PACCEP:
        .long      8
        .ASCII      "(ACCEPT)"
        .balign    4,0x00
PACCEP:
        .long    DOCOL
        .long    PACCEP+HEADSIZE
        .long    0x0
        .long    ACCEP
        .long    N_PACCEP
        .long    0
        .long    0

PACCEP2:
#        DC      NOLF
#        DC       LIT, 'a', EMIT
        .long      REMAIND
        .long      TFET
        .long      LIT, ALF, SINDEX
        .long      ZEQU
        .long      ZBRAN
        .long      PACCEP1-.-CW
#        DC      RETOBO
        .long      REMAIND, TFET
        .long      TIB, FETCH
        .long      SWAP, LMOVE
        .long      TIB, FETCH
        .long      REMAIND, CELLP, STOR
        .long      REFTIB
        .long      BRAN
        .long      PACCEP2-.-CW
PACCEP1:
#        DC      SPLOFF
        .long      REMAIND, TFET
        .long      LIT, ALF, SSLAS
        .long      TSWAP, REMAIND, TSTOR
        .long      SEMIS


#  ***********
#  *   KEY   *
#  ***********
#
        .balign    4,0x00
N_KEY:
        .long      3
        .ASCII      "KEY"
        .balign    4,0x00
KEY:
        .long    DOCOL
        .long    KEY+HEADSIZE
        .long    0x0
        .long    PACCEP
        .long    N_KEY
        .long    0
        .long    0

        .long      ONE, LIT, RAWIO, STTERM # Do not wait for CR
        .long      ZERO, SPFET     # Expect single key on stack
        .long      ZERO            # Standard input
        .long      SWAP, ONE       # Buffer and length
        .long      LIT, read, XOS
        .long      LDUP, QERRUR
        .long      ZEQU, LIT, -EPIPE, LAND, QERRUR # Presumably end of pipe.
        .long      ONE, LIT, RAWIO, STTERM       # Toggle back
        .long      SEMIS
#

#  ************
#  *   KEY?   *
#  ************
#
        .balign    4,0x00
N_KEYQ:
        .long      4
        .ASCII      "KEY?"
        .balign    4,0x00
KEYQ:
        .long    DOCOL
        .long    KEYQ+HEADSIZE
        .long    0x0
        .long    KEY
        .long    N_KEYQ
        .long    0
        .long    0

        .long      ZERO, LIT, RAWIO, STTERM
        .long      ZERO, ZERO     # Time out of zero. See MS.
         .long      LIT, TV, TSTOR
        .long      ONE # Waiting for standard in.
        .long      LIT, RFDS, STOR
        .long      ONE     # # of highest channel to watch.
        .long      LIT, RFDS
        .long      ZERO, ZERO  # Not output, not events.
        .long      LIT, TV
        .long      LIT, _newselect, XOS5
        .long      LDUP,  QERRUR
        .long      NEGATE        # 1 -> flag.
        .long      ZERO, LIT, RAWIO, STTERM
        .long      SEMIS
        .balign    4,0x00
RFDS:   .SPACE   (CW*(32))     #32 channels maximum.
#

#  ************
#  *   TYPE   *
#  ************
#
        .balign    4,0x00
N_LTYPE:
        .long      4
        .ASCII      "TYPE"
        .balign    4,0x00
LTYPE:
        .long    DOCOL
        .long    LTYPE+HEADSIZE
        .long    0x0
        .long    KEYQ
        .long    N_LTYPE
        .long    0
        .long    0

        .long      LDUP, LOUT, PSTOR
        .long      ONE, WFILE, THROW
        .long      SEMIS
#

#  *************
#  *   ETYPE   *
#  *************
#
        .balign    4,0x00
N_ETYPE:
        .long      5
        .ASCII      "ETYPE"
        .balign    4,0x00
ETYPE:
        .long    DOCOL
        .long    ETYPE+HEADSIZE
        .long    0x0
        .long    LTYPE
        .long    N_ETYPE
        .long    0
        .long    0

        .long      TWO, WFILE, THROW
        .long      SEMIS
#

#  ************
#  *   EMIT   *
#  ************
#
        .balign    4,0x00
N_EMIT:
        .long      4
        .ASCII      "EMIT"
        .balign    4,0x00
EMIT:
        .long    DOCOL
        .long    EMIT+HEADSIZE
        .long    0x0
        .long    ETYPE
        .long    N_EMIT
        .long    0
        .long    0

        .long      SPFET, ONE, LTYPE
        .long      DROP
        .long      SEMIS
#

#  **************
#  *   TERMIO   *
#  **************
#
        .balign    4,0x00
N_TERMIO:
        .long      6
        .ASCII      "TERMIO"
        .balign    4,0x00
TERMIO:
        .long    DOVAR
        .long    TERMIO+HEADSIZE
        .long    0x0
        .long    EMIT
        .long    N_TERMIO
        .long    0
        .long    0

        .SPACE   SIZE_TERMIO     # Filled in during cold boot.
#

#  ****************
#  *   SET-TERM   *
#  ****************
#
        .balign    4,0x00
N_STTERM:
        .long      8
        .ASCII      "SET-TERM"
        .balign    4,0x00
STTERM:
        .long    DOCOL
        .long    STTERM+HEADSIZE
        .long    0x0
        .long    TERMIO
        .long    N_STTERM
        .long    0
        .long    0

SIZEL   = 4     # Also for 64 bits. 
        .long      LIT, (TERMIO+HEADSIZE)+SIZEL*3
        .long      SWAP, TOGGL
        .long      LIT, (TERMIO+HEADSIZE)+SIZEL*4+7
        .long      CSTOR
        .long      ZERO, LIT, TCSETS, TERMIO
        .long      LIT, ioctl, XOS, QERRUR
        .long      SEMIS
#
# 
#
#
#
#
#
#

        #
#------------------------------------
#       SYSTEM DEPENDANT DISK I/O
#------------------------------------

#  ******************
#  *   DISK-ERROR   *
#  ******************
#
        .balign    4,0x00
N_DERR:
        .long      10
        .ASCII      "DISK-ERROR"
        .balign    4,0x00
DERR:
        .long    DOVAR
        .long    DERR+HEADSIZE
        .long    0x0
        .long    STTERM
        .long    N_DERR
        .long    0
        .long    0

        .long      -1
#


#  ******************
#  *   BLOCK-FILE   *
#  ******************
#
        .balign    4,0x00
N_BLFL:
        .long      10
        .ASCII      "BLOCK-FILE"
        .balign    4,0x00
BLFL:
        .long    DOVAR
        .long    BLFL+HEADSIZE
        .long    0x0
        .long    DERR
        .long    N_BLFL
        .long    0
        .long    0

        # Allow for some path, at most 80 char's
        .long      80
        .ASCII      "forth.lab                                                                       "

#  ********************
#  *   BLOCK-HANDLE   *
#  ********************
#
        .balign    4,0x00
N_BHAN:
        .long      12
        .ASCII      "BLOCK-HANDLE"
        .balign    4,0x00
BHAN:
        .long    DOVAR
        .long    BHAN+HEADSIZE
        .long    0x0
        .long    BLFL
        .long    N_BHAN
        .long    0
        .long    0

        .long      -1
#

#  *******************
#  *   ?DISK-ERROR   *
#  *******************
#
        .balign    4,0x00
N_QDSKER:
        .long      11
        .ASCII      "?DISK-ERROR"
        .balign    4,0x00
QDSKER:
        .long    DOCOL
        .long    QDSKER+HEADSIZE
        .long    0x0
        .long    BHAN
        .long    N_QDSKER
        .long    0
        .long    0

        .long      LIT, 8, QERR
        .long      SEMIS
#
#


#  ******************
#  *   BLOCK-INIT   *
#  ******************
#
        .balign    4,0x00
N_BLINI:
        .long      10
        .ASCII      "BLOCK-INIT"
        .balign    4,0x00
BLINI:
        .long    DOCOL
        .long    BLINI+HEADSIZE
        .long    0x0
        .long    QDSKER
        .long    N_BLINI
        .long    0
        .long    0

        .long      BLFL, SFET, DTRAI
        .long      ROT
        .long      OFILE
        .long      ZEQU, NEGATE  # 0 if disk problems, 1 if not.
        .long      LWARN, FETCH, MIN # AND but WARNING is 0/1.
        .long      LWARN, STOR
        .long      BHAN, STOR
        .long      MTBUF
        .long      SEMIS
#

#  ******************
#  *   BLOCK-EXIT   *
#  ******************
#
        .balign    4,0x00
N_BLEXI:
        .long      10
        .ASCII      "BLOCK-EXIT"
        .balign    4,0x00
BLEXI:
        .long    DOCOL
        .long    BLEXI+HEADSIZE
        .long    0x0
        .long    BLINI
        .long    N_BLEXI
        .long    0
        .long    0

        .long      FLUSH
        .long      BHAN, FETCH
        .long      CFILE
        .long      ZERO, LWARN, STOR
        .long      LIT, -1, BHAN, STOR    #Regardless of close errors.
        .long      QDSKER
        .long      SEMIS
#

#  **************
#  *   BLSEEK   *
#  **************
#
        .balign    4,0x00
N_BLSEEK:
        .long      6
        .ASCII      "BLSEEK"
        .balign    4,0x00
BLSEEK:
        .long    DOCOL
        .long    BLSEEK+HEADSIZE
        .long    0x0
        .long    BLEXI
        .long    N_BLSEEK
        .long    0
        .long    0

        .long      BBUF
        .long      USTAR
        .long      BHAN, FETCH
        .long      PFILE           # Disk position.
        .long      QDSKER
        .long      SEMIS
#

#  ******************
#  *   BLOCK-READ   *
#  ******************
#
        .balign    4,0x00
N_BLREAD:
        .long      10
        .ASCII      "BLOCK-READ"
        .balign    4,0x00
BLREAD:
        .long    DOCOL
        .long    BLREAD+HEADSIZE
        .long    0x0
        .long    BLSEEK
        .long    N_BLREAD
        .long    0
        .long    0

        .long      BLSEEK # That's done
        .long      BBUF
        .long      BHAN, FETCH
        .long      RFILE           # Disk read 
        .long      SWAP, BBUF, UNEQ
        .long      LOR, QDSKER
        .long      SEMIS
#

#  *******************
#  *   BLOCK-WRITE   *
#  *******************
#
        .balign    4,0x00
N_BLWRITE:
        .long      11
        .ASCII      "BLOCK-WRITE"
        .balign    4,0x00
BLWRITE:
        .long    DOCOL
        .long    BLWRITE+HEADSIZE
        .long    0x0
        .long    BLREAD
        .long    N_BLWRITE
        .long    0
        .long    0

        .long      BLSEEK # That's done
        .long      BBUF
        .long      BHAN, FETCH
        .long      WFILE           # Disk write
        .long      QDSKER
        .long      SEMIS
#
#

#  *************
#  *   SHELL   *
#  *************
#
        .balign    4,0x00
N_SHELL:
        .long      5
        .ASCII      "SHELL"
        .balign    4,0x00
SHELL:
        .long    DOVAR
        .long    SHELL+HEADSIZE
        .long    0x0
        .long    BLWRITE
        .long    N_SHELL
        .long    0
        .long    0

         .long      7
        .ASCII      "/bin/sh"
        
        
        .balign    4,0x00
        .SPACE   252# Allow for some path

#
#


#  **************
#  *   SYSTEM   *
#  **************
#
        .balign    4,0x00
N_SYSTEM:
        .long      6
        .ASCII      "SYSTEM"
        .balign    4,0x00
SYSTEM:
        .long    DOCOL
        .long    SYSTEM+HEADSIZE
        .long    0x0
        .long    SHELL
        .long    N_SYSTEM
        .long    0
        .long    0

        .long      LIT, COMBUF, SSTOR, ZERO, LIT, COMBUF, CHAPP
        .long      ZERO, SHELL, CHAPP, LIT, -1, SHELL, PSTOR
        .long      X, X, X, LIT, fork, XOS
        .long      LDUP, QERRUR
        .long      LDUP, ZEQU
        .long      ZBRAN
        .long      SYSTEM1-.-CW
        .long      SHELL, CELLP, LIT, SYSARGS
#  Get environment vector
      .long      ARGS, FETCH, SFET, ONEP, LCELLS, PLUS
        .long      LIT, execve, XOS
        .long      LIT, 49, LERROR, BYE   # Returned, this is always wrong.
SYSTEM1:
        .long      LDUP
        .long      LIT, STATUS, ZERO, LIT, waitpid, XOS
        .long      LDUP
        .long      LIT, EINTR, EQUAL
        .long      BRAN
        .long      SYSTEM2-.-CW
        .long      DROP
        .long      ZBRAN
        .long      SYSTEM1-.-CW
SYSTEM2:
        .long      QERRUR
        .long      TDROP
        .long      SEMIS
#
        .balign    4,0x00
STATUS:  .long     0
SYSARG2: .ASCII     "-c"
         .byte      0
        .balign    4,0x00
COMBUF:  .SPACE   CW+256  #One cell for high level string manipulation.
        .balign    4,0x00
SYSARGS:.long   (SHELL+HEADSIZE)+CW, SYSARG2, COMBUF + CW, 0
#
 # 
#
#
#
#
# 
#
#
#
# 
# 
#


#  *********
#  *   '   *
#  *********
#
        .balign    4,0x00
N_ITICK:
        .long      1
        .ASCII      "'"
        .balign    4,0x00
ITICK:
        .long    DOCOL 
        .long    ITICK+HEADSIZE
        .long    0x0
        .long    SYSTEM
        .long    N_ITICK
        .long    0
        .long    0

        .long      LNAME, PRESENT
        .long      LDUP, ZEQU
        .long      LIT, 11, QERR
        .long      SEMIS
#

#  ***********
#  *   [']   *
#  ***********
#
        .balign    4,0x00
N_BTICK:
        .long      3
        .ASCII      "[']"
        .balign    4,0x00
BTICK:
        .long    DOCOL
        .long    (TICK+HEADSIZE)
        .long    B_IMMED
        .long    ITICK
        .long    N_BTICK
        .long    0
        .long    0

#
#

#  ******************
#  *   FORGET-VOC   *
#  ******************
#
        .balign    4,0x00
N_FORGV:
        .long      10
        .ASCII      "FORGET-VOC"
        .balign    4,0x00
FORGV:
        .long    DOCOL
        .long    FORGV+HEADSIZE
        .long    0x0
        .long    BTICK
        .long    N_FORGV
        .long    0
        .long    0

        .long      TDUP
        .long      SWAP
        .long      ULESS
        .long      ZBRAN
        .long      FORGV1-.-CW
#  Forget part of contents.
        .long      SWAP
        .long      TOR
        .long      TWID
FORGV0:
        .long      LDUP
        .long      LDUP
FORGV3:
        .long      TLFA,FETCH
        .long      LDUP
        .long      RR
        .long      ULESS
        .long      ZBRAN
        .long      FORGV3-.-CW
        .long      SWAP            #  Not to be forgotten entry found.
        .long      TLFA
        .long      STOR           #  Short other entries out.
        .long      TLFA,FETCH
        .long      LDUP
        .long      ZEQU
        .long      ZBRAN
        .long      FORGV0-.-CW        #  Repeat until end of wordlist.
        .long      DROP
        .long      FROMR
        .long      BRAN
        .long      FORGV2-.-CW
FORGV1:
#       { Vocabulary itself is also forgotten.}
        .long      TVFA
        .long      FETCH     # Unlink by linking next namespace.
        .long      VOCL
        .long      STOR
        .long      ONLY, FORTH
        .long      DEFIN
FORGV2: .long      SEMIS
#

#  **************
#  *   FORGET   *
#  **************
#
        .balign    4,0x00
N_FORG:
        .long      6
        .ASCII      "FORGET"
        .balign    4,0x00
FORG:
        .long    DOCOL
        .long    FORG+HEADSIZE
        .long    0x0
        .long    FORGV
        .long    N_FORG
        .long    0
        .long    0

        .long      TICK
        .long      LDUP
        .long      FENCE
        .long      FETCH
        .long      LESS
        .long      LIT, 21, QERR
        .long      LIT,FORGV
        .long      FORV
        .long      TNFA, FETCH, LDP, STOR
        .long      SEMIS
#

#  *************
#  *   (BACK   *
#  *************
#
        .balign    4,0x00
N_PBACK:
        .long      5
        .ASCII      "(BACK"
        .balign    4,0x00
PBACK:
        .long    DOCOL
        .long    PBACK+HEADSIZE
        .long    0x0
        .long    FORG
        .long    N_PBACK
        .long    0
        .long    0

        .long      HERE
        .long      SEMIS
#

#  *************
#  *   BACK)   *
#  *************
#
        .balign    4,0x00
N_BACKP:
        .long      5
        .ASCII      "BACK)"
        .balign    4,0x00
BACKP:
        .long    DOCOL
        .long    BACKP+HEADSIZE
        .long    0x0
        .long    PBACK
        .long    N_BACKP
        .long    0
        .long    0

        .long      HERE
        .long      CELLP
        .long      LSUB
        .long      COMMA
        .long      SEMIS
#

#  ****************
#  *   (FORWARD   *
#  ****************
#
        .balign    4,0x00
N_PFORWARD:
        .long      8
        .ASCII      "(FORWARD"
        .balign    4,0x00
PFORWARD:
        .long    DOCOL
        .long    PFORWARD+HEADSIZE
        .long    0x0
        .long    BACKP
        .long    N_PFORWARD
        .long    0
        .long    0

        .long      HERE
        .long      X
        .long      COMMA
        .long      SEMIS
#

#  ****************
#  *   FORWARD)   *
#  ****************
#
        .balign    4,0x00
N_FORWARDP:
        .long      8
        .ASCII      "FORWARD)"
        .balign    4,0x00
FORWARDP:
        .long    DOCOL
        .long    FORWARDP+HEADSIZE
        .long    0x0
        .long    PFORWARD
        .long    N_FORWARDP
        .long    0
        .long    0

        .long      HERE
        .long      OVER
        .long      CELLP
        .long      LSUB
        .long      SWAP
        .long      STOR
        .long      SEMIS
#

#  *************
#  *   BEGIN   *
#  *************
#
        .balign    4,0x00
N_BEGIN:
        .long      5
        .ASCII      "BEGIN"
        .balign    4,0x00
BEGIN:
        .long    DOCOL
        .long    BEGIN+HEADSIZE
        .long    B_IMMED
        .long    FORWARDP
        .long    N_BEGIN
        .long    0
        .long    0

        .long      PBACK
        .long      QCOMP, ONE
        .long      SEMIS
#

#  ************
#  *   THEN   *
#  ************
#
        .balign    4,0x00
N_THEN:
        .long      4
        .ASCII      "THEN"
        .balign    4,0x00
THEN:
        .long    DOCOL
        .long    THEN+HEADSIZE
        .long    B_IMMED
        .long    BEGIN
        .long    N_THEN
        .long    0
        .long    0

        .long      QCOMP, TWO, QPAIR
        .long      FORWARDP
        .long      SEMIS
#

#  **********
#  *   DO   *
#  **********
#
        .balign    4,0x00
N_DO:
        .long      2
        .ASCII      "DO"
        .balign    4,0x00
DO:
        .long    DOCOL
        .long    DO+HEADSIZE
        .long    B_IMMED
        .long    THEN
        .long    N_DO
        .long    0
        .long    0

         .long      LIT, XDO, COMMA, PFORWARD, PBACK
        .long      QCOMP, LIT,3    # Magic number
        .long      SEMIS
#

#  ***********
#  *   ?DO   *
#  ***********
#
        .balign    4,0x00
N_QDO:
        .long      3
        .ASCII      "?DO"
        .balign    4,0x00
QDO:
        .long    DOCOL
        .long    QDO+HEADSIZE
        .long    B_IMMED
        .long    DO
        .long    N_QDO
        .long    0
        .long    0

         .long      LIT, XQDO, COMMA, PFORWARD, PBACK
        .long      QCOMP, LIT,3    # Magic number
        .long      SEMIS
#

#  ************
#  *   LOOP   *
#  ************
#
        .balign    4,0x00
N_LLOOP:
        .long      4
        .ASCII      "LOOP"
        .balign    4,0x00
LLOOP:
        .long    DOCOL
        .long    LLOOP+HEADSIZE
        .long    B_IMMED
        .long    QDO
        .long    N_LLOOP
        .long    0
        .long    0

        .long      LIT, ONE, COMMA
        .long      PLOOP
        .long      SEMIS
#

#  *************
#  *   +LOOP   *
#  *************
#
        .balign    4,0x00
N_PLOOP:
        .long      5
        .ASCII      "+LOOP"
        .balign    4,0x00
PLOOP:
        .long    DOCOL
        .long    PLOOP+HEADSIZE
        .long    B_IMMED
        .long    LLOOP
        .long    N_PLOOP
        .long    0
        .long    0

        .long      QCOMP, LIT, 3, QPAIR
        .long      LIT, XPLOOP, COMMA
        .long      LIT, ZBRAN, COMMA, BACKP     # Until 
        .long      LIT, UNLOOP, COMMA
        .long      FORWARDP # For DO to push the leave address.
        .long      SEMIS
#

#  *************
#  *   UNTIL   *
#  *************
#
        .balign    4,0x00
N_UNTIL:
        .long      5
        .ASCII      "UNTIL"
        .balign    4,0x00
UNTIL:
        .long    DOCOL
        .long    UNTIL+HEADSIZE
        .long    B_IMMED
        .long    PLOOP
        .long    N_UNTIL
        .long    0
        .long    0

        .long      QCOMP, ONE, QPAIR
        .long      LIT, ZBRAN, COMMA, BACKP
        .long      SEMIS
#

#  *************
#  *   AGAIN   *
#  *************
#
        .balign    4,0x00
N_AGAIN:
        .long      5
        .ASCII      "AGAIN"
        .balign    4,0x00
AGAIN:
        .long    DOCOL
        .long    AGAIN+HEADSIZE
        .long    B_IMMED
        .long    UNTIL
        .long    N_AGAIN
        .long    0
        .long    0

        .long      QCOMP, ONE, QPAIR
        .long      LIT, BRAN, COMMA, BACKP
        .long      SEMIS
#

#  **************
#  *   REPEAT   *
#  **************
#
        .balign    4,0x00
N_REPEA:
        .long      6
        .ASCII      "REPEAT"
        .balign    4,0x00
REPEA:
        .long    DOCOL
        .long    REPEA+HEADSIZE
        .long    B_IMMED
        .long    AGAIN
        .long    N_REPEA
        .long    0
        .long    0

        .long      QCOMP, ONE, QPAIR   # Matches BEGIN ?
        .long      LIT, BRAN, COMMA, BACKP
        .long      LIT, 4, QPAIR # Matches WHILE ?
        .long      FORWARDP # WHILE target. 
        .long      SEMIS
#

#  **********
#  *   IF   *
#  **********
#
        .balign    4,0x00
N_LIF:
        .long      2
        .ASCII      "IF"
        .balign    4,0x00
LIF:
        .long    DOCOL
        .long    LIF+HEADSIZE
        .long    B_IMMED
        .long    REPEA
        .long    N_LIF
        .long    0
        .long    0

        .long      LIT, ZBRAN, COMMA, PFORWARD
        .long      QCOMP, TWO     # Magic number
        .long      SEMIS
#

#  ************
#  *   ELSE   *
#  ************
#
        .balign    4,0x00
N_LELSE:
        .long      4
        .ASCII      "ELSE"
        .balign    4,0x00
LELSE:
        .long    DOCOL
        .long    LELSE+HEADSIZE
        .long    B_IMMED
        .long    LIF
        .long    N_LELSE
        .long    0
        .long    0

        .long      QCOMP, TWO, QPAIR
        .long      LIT, BRAN, COMMA, PFORWARD
        .long      SWAP
        .long      FORWARDP
        .long      TWO     # Magic number
        .long      SEMIS
#

#  *************
#  *   WHILE   *
#  *************
#
        .balign    4,0x00
N_LWHILE:
        .long      5
        .ASCII      "WHILE"
        .balign    4,0x00
LWHILE:
        .long    DOCOL
        .long    LWHILE+HEADSIZE
        .long    B_IMMED
        .long    LELSE
        .long    N_LWHILE
        .long    0
        .long    0

        .long      QCOMP, LDUP, ONE, QPAIR
        .long      TOR    #  Save backward target. 
        .long      TOR
        .long      LIT, ZBRAN, COMMA, PFORWARD
        .long      LIT, 4 # Magic number
        .long      FROMR
        .long      FROMR
        .long      SEMIS
#

#  **************
#  *   SPACES   *
#  **************
#
        .balign    4,0x00
N_SPACES:
        .long      6
        .ASCII      "SPACES"
        .balign    4,0x00
SPACES:
        .long    DOCOL
        .long    SPACES+HEADSIZE
        .long    0x0
        .long    LWHILE
        .long    N_SPACES
        .long    0
        .long    0

        .long      ZERO
        .long      MAX
        .long      ZERO
        .long     XQDO
        .long      SPAX1-.-CW
SPAX2:  .long      SPACE
        .long     ONE, XPLOOP, ZBRAN
        .long      SPAX2-.-CW
        .long      UNLOOP     #LOOP
SPAX1:
        .long      SEMIS
#

#  **********
#  *   <#   *
#  **********
#
        .balign    4,0x00
N_BDIGS:
        .long      2
        .ASCII      "<#"
        .balign    4,0x00
BDIGS:
        .long    DOCOL
        .long    BDIGS+HEADSIZE
        .long    0x0
        .long    SPACES
        .long    N_BDIGS
        .long    0
        .long    0

        .long      PAD
        .long      HLD
        .long      STOR
        .long      SEMIS
#

#  **********
#  *   #>   *
#  **********
#
        .balign    4,0x00
N_EDIGS:
        .long      2
        .ASCII      "#>"
        .balign    4,0x00
EDIGS:
        .long    DOCOL
        .long    EDIGS+HEADSIZE
        .long    0x0
        .long    BDIGS
        .long    N_EDIGS
        .long    0
        .long    0

        .long      DROP
        .long      DROP
        .long      HLD
        .long      FETCH
        .long      PAD
        .long      OVER
        .long      LSUB
        .long      SEMIS
#

#  ************
#  *   SIGN   *
#  ************
#
        .balign    4,0x00
N_SIGN:
        .long      4
        .ASCII      "SIGN"
        .balign    4,0x00
SIGN:
        .long    DOCOL
        .long    SIGN+HEADSIZE
        .long    0x0
        .long    EDIGS
        .long    N_SIGN
        .long    0
        .long    0

        .long      ZLESS
        .long      ZBRAN
        .long      SIGN1-.-CW #IF
        .long      LIT, AMS
        .long      HOLD    #THEN
SIGN1:  .long      SEMIS
#

#  *********
#  *   #   *
#  *********
#
        .balign    4,0x00
N_DIG:
        .long      1
        .ASCII      "#"
        .balign    4,0x00
DIG:
        .long    DOCOL
        .long    DIG+HEADSIZE
        .long    0x0
        .long    SIGN
        .long    N_DIG
        .long    0
        .long    0

        .long      BASE
        .long      FETCH
        .long      MSMOD
        .long      ROT
        .long      LIT,9
        .long      OVER
        .long      LESS
        .long      ZBRAN
        .long      DIG1-.-CW  #IF
        .long      LIT,7
        .long      PLUS    #THEN
DIG1:   .long      LIT,0x30
        .long      PLUS
        .long      HOLD
        .long      SEMIS
#

#  **********
#  *   #S   *
#  **********
#
        .balign    4,0x00
N_DIGS:
        .long      2
        .ASCII      "#S"
        .balign    4,0x00
DIGS:
        .long    DOCOL
        .long    DIGS+HEADSIZE
        .long    0x0
        .long    DIG
        .long    N_DIGS
        .long    0
        .long    0

DIGS1:  .long      DIG     #BEGIN
        .long      OVER
        .long      OVER
        .long      LOR
        .long      ZEQU
        .long      ZBRAN
        .long      DIGS1-.-CW #UNTIL
        .long      SEMIS
#

#  *************
#  *   (D.R)   *
#  *************
#
        .balign    4,0x00
N_PDDOTR:
        .long      5
        .ASCII      "(D.R)"
        .balign    4,0x00
PDDOTR:
        .long    DOCOL
        .long    PDDOTR+HEADSIZE
        .long    0x0
        .long    DIGS
        .long    N_PDDOTR
        .long    0
        .long    0

        .long      TOR
        .long      SWAP
        .long      OVER
        .long      DABS
        .long      BDIGS
        .long      DIGS
        .long      ROT
        .long      SIGN
        .long      EDIGS
        .long      FROMR
        .long      OVER
        .long      LSUB, ZERO, MAX
        .long      ZERO
        .long     XQDO
        .long      PDDOT1-.-CW
PDDOT2:  .long      LBL, HOLD  #WARNING: HOLD outside of #>.
        .long     ONE, XPLOOP, ZBRAN
        .long      PDDOT2-.-CW
        .long      UNLOOP 
PDDOT1:
        .long      EDIGS  #Drop string instead of number.
        .long      SEMIS
#

#  ***********
#  *   D.R   *
#  ***********
#
        .balign    4,0x00
N_DDOTR:
        .long      3
        .ASCII      "D.R"
        .balign    4,0x00
DDOTR:
        .long    DOCOL
        .long    DDOTR+HEADSIZE
        .long    0x0
        .long    PDDOTR
        .long    N_DDOTR
        .long    0
        .long    0

        .long      PDDOTR
        .long      LTYPE
        .long      SEMIS
#

#  **********
#  *   .R   *
#  **********
#
        .balign    4,0x00
N_DOTR:
        .long      2
        .ASCII      ".R"
        .balign    4,0x00
DOTR:
        .long    DOCOL
        .long    DOTR+HEADSIZE
        .long    0x0
        .long    DDOTR
        .long    N_DOTR
        .long    0
        .long    0

        .long      TOR
        .long      STOD
        .long      FROMR
        .long      DDOTR
        .long      SEMIS
#

#  **********
#  *   D.   *
#  **********
#
        .balign    4,0x00
N_DDOT:
        .long      2
        .ASCII      "D."
        .balign    4,0x00
DDOT:
        .long    DOCOL
        .long    DDOT+HEADSIZE
        .long    0x0
        .long    DOTR
        .long    N_DDOT
        .long    0
        .long    0

        .long      ZERO
        .long      DDOTR
        .long      SPACE
        .long      SEMIS
#

#  *********
#  *   .   *
#  *********
#
        .balign    4,0x00
N_DOT:
        .long      1
        .ASCII      "."
        .balign    4,0x00
DOT:
        .long    DOCOL
        .long    DOT+HEADSIZE
        .long    0x0
        .long    DDOT
        .long    N_DOT
        .long    0
        .long    0

        .long      STOD
        .long      DDOT
        .long      SEMIS
#

#  *********
#  *   ?   *
#  *********
#
        .balign    4,0x00
N_QUES:
        .long      1
        .ASCII      "?"
        .balign    4,0x00
QUES:
        .long    DOCOL
        .long    QUES+HEADSIZE
        .long    0x0
        .long    DOT
        .long    N_QUES
        .long    0
        .long    0

        .long      FETCH
        .long      DOT
        .long      SEMIS
#

#  **********
#  *   U.   *
#  **********
#
        .balign    4,0x00
N_UDOT:
        .long      2
        .ASCII      "U."
        .balign    4,0x00
UDOT:
        .long    DOCOL
        .long    UDOT+HEADSIZE
        .long    0x0
        .long    QUES
        .long    N_UDOT
        .long    0
        .long    0

        .long      ZERO
        .long      DDOT
        .long      SEMIS
#

#  *****************
#  *   FOR-WORDS   *
#  *****************
#
        .balign    4,0x00
N_FORW:
        .long      9
        .ASCII      "FOR-WORDS"
        .balign    4,0x00
FORW:
        .long    DOCOL
        .long    FORW+HEADSIZE
        .long    0x0
        .long    UDOT
        .long    N_FORW
        .long    0
        .long    0

        .long      SWAP
        .long      TOR
        .long      TOR
FORW1:  .long      FROMR
        .long      RR
        .long      OVER
        .long      TLFA
        .long      FETCH
        .long      TOR
        .long      EXEC
        .long      RR
        .long      ZEQU
        .long      ZBRAN
        .long      FORW1-.-CW
        .long      RDROP
        .long      RDROP
        .long      SEMIS
#

#  ****************
#  *   FOR-VOCS   *
#  ****************
#
        .balign    4,0x00
N_FORV:
        .long      8
        .ASCII      "FOR-VOCS"
        .balign    4,0x00
FORV:
        .long    DOCOL
        .long    FORV+HEADSIZE
        .long    0x0
        .long    FORW
        .long    N_FORV
        .long    0
        .long    0

        .long      TOR
        .long      VOCL
        .long      FETCH
        .long      TOR
FORV1:  .long      FROMR
        .long      RR
        .long      OVER
        .long      TVFA
        .long      FETCH
        .long      TOR
        .long      EXEC
        .long      RR
        .long      ZEQU
        .long      ZBRAN
        .long      FORV1-.-CW
        .long      RDROP
        .long      RDROP
        .long      SEMIS
#

#  *************
#  *   WORDS   *
#  *************
#
        .balign    4,0x00
N_WORDS:
        .long      5
        .ASCII      "WORDS"
        .balign    4,0x00
WORDS:
        .long    DOCOL
        .long    WORDS+HEADSIZE
        .long    0x0
        .long    FORV
        .long    N_WORDS
        .long    0
        .long    0

        .long      CSLL
        .long      LOUT
        .long      STOR
        .long      ZERO            # End sentinel. 
        .long      LIT, NOOP
        .long      CONTEXT
        .long      FETCH
        .long      FORW
WORDS1:
        .long      LDUP
        .long      ZBRAN
        .long      WORDS2-.-CW
        .long      IDDOT
        .long      BRAN
        .long      WORDS1-.-CW
WORDS2:
        .long DROP
        .long      SEMIS
#
# 

#  *****************
#  *   EXIT-CODE   *
#  *****************
#
        .balign    4,0x00
N_XCODE:
        .long      9
        .ASCII      "EXIT-CODE"
        .balign    4,0x00
XCODE:
        .long    DOVAR
        .long    XCODE+HEADSIZE
        .long    0x0
        .long    WORDS
        .long    N_XCODE
        .long    0
        .long    0

        .long      0
#
# 
# 


#  ***********
#  *   BYE   *
#  ***********
#
        .balign    4,0x00
N_BYE:
        .long      3
        .ASCII      "BYE"
        .balign    4,0x00
BYE:
        .long    DOCOL
        .long    BYE+HEADSIZE
        .long    0x0
        .long    XCODE
        .long    N_BYE
        .long    0
        .long    0

# Exit to linux, with status as stored. 
        .long      XCODE, FETCH, X, X, LIT, exit, XOS
        .long      SEMIS   #Unnecessary, but helpful for decompilation.
# 
#

#  ************
#  *   LIST   *
#  ************
#
        .balign    4,0x00
N_LLIST:
        .long      4
        .ASCII      "LIST"
        .balign    4,0x00
LLIST:
        .long    DOCOL
        .long    LLIST+HEADSIZE
        .long    0x0
        .long    BYE
        .long    N_LLIST
        .long    0
        .long    0

        .long      BASE, FETCH, TOR
        .long      DECA
        .long      SCR,STOR

        .long      SKIP
         .long      6
SB5: .ASCII      "SCR # "
       .balign    4,0x00
        .long      LIT, SB5
        .long      LIT, 6
        .long      LTYPE
        .long      SCR, FETCH, DOT
        .long      SCR, FETCH, BLOCK, BBUF

        .long      LIT, 16, ZERO
        .long     XDO
        .long      LLIST9-.-CW
LLIST1: .long      LIT, ALF, SSLAS
        .long      CR, IDO, LIT, 2, DOTR
        .long      SPACE, LTYPE
        .long     ONE, XPLOOP, ZBRAN
        .long      LLIST1-.-CW
        .long      UNLOOP 
LLIST9:
        .long      TDROP
        .long      FROMR, BASE, STOR
        .long      SEMIS
#

#  *************
#  *   INDEX   *
#  *************
#
        .balign    4,0x00
N_INDEX:
        .long      5
        .ASCII      "INDEX"
        .balign    4,0x00
INDEX:
        .long    DOCOL
        .long    INDEX+HEADSIZE
        .long    0x0
        .long    LLIST
        .long    N_INDEX
        .long    0
        .long    0

        .long      LIT,AFF
        .long      EMIT,CR
        .long      ONEP,SWAP
        .long     XDO
        .long      INDE9-.-CW
INDE1:  .long      CR,IDO
        .long      LIT,3
        .long      DOTR,SPACE
        .long      ZERO,IDO
        .long      PLINE, LTYPE
        .long     ONE, XPLOOP, ZBRAN
        .long      INDE1-.-CW
        .long      UNLOOP 
INDE9:
        .long      SEMIS
#

#  **********
#  *   .S   *
#  **********
#
        .balign    4,0x00
N_DOTS:
        .long      2
        .ASCII      ".S"
        .balign    4,0x00
DOTS:
        .long    DOCOL
        .long    DOTS+HEADSIZE
        .long    0x0
        .long    INDEX
        .long    N_DOTS
        .long    0
        .long    0

        .long      CR
        .long      LIT, 'S', EMIT
        .long      LIT, ASO, EMIT
        .long      SPACE
        .long      SPFET, SZERO, FETCH
DOC2:   .long      OVER, OVER,  LSUB, ZLESS
        .long      ZBRAN
        .long      DOC1-.-CW
        .long      ZERO, CELLP, LSUB, LDUP, FETCH, DOT
        .long      BRAN
        .long      DOC2-.-CW
DOC1:    .long DROP, DROP
        .long      LIT, ASC, EMIT
        .long SEMIS
#

#  ********************
#  *   ENVIRONMENT?   *
#  ********************
#
        .balign    4,0x00
N_ENVQ:
        .long      12
        .ASCII      "ENVIRONMENT?"
        .balign    4,0x00
ENVQ:
        .long    DOCOL
        .long    ENVQ+HEADSIZE
        .long    0x0
        .long    DOTS
        .long    N_ENVQ
        .long    0
        .long    0

        .long      LIT, ENV, TWID, PFIND
        .long      TOR, TDROP, FROMR
        .long      LDUP
        .long      ZBRAN
        .long      ENVQ1-.-CW
        .long      EXEC
        .long      LIT, -1
ENVQ1:
        .long      SEMIS
#


# 
#
#
# This word is not even fig!

#  ***************
#  *   .SIGNON   *
#  ***************
#
        .balign    4,0x00
N_SIGNON:
        .long      7
        .ASCII      ".SIGNON"
        .balign    4,0x00
SIGNON:
        .long    DOCOL
        .long    SIGNON+HEADSIZE
        .long    0x0
        .long    ENVQ
        .long    N_SIGNON
        .long    0
        .long    0

# PRINT CPU TYPE (8088)
        .long      CR
        .long      BASE,FETCH
        .long      LIT,36, BASE,STOR
        .long      LCPU, DDOT
        .long      BASE,STOR
#
        .long      LENAME, LTYPE, SPACE
        .long      LVERSION, LTYPE, SPACE
        .long      CR
        .long      SEMIS
#
# 
#
#**** LAST DICTIONARY WORD ****

#  ************
#  *   TASK   *
#  ************
#
        .balign    4,0x00
N_TASK:
        .long      4
        .ASCII      "TASK"
        .balign    4,0x00
TASK:
        .long    DOCOL
        .long    TASK+HEADSIZE
        .long    0x0
        .long    SIGNON
        .long    N_TASK
        .long    0
        .long    0

        .long      SEMIS
#
 #  

 /* 

The remaining memory ( up to 'EM' ) is
used for:

        1. EXTENSION DICTIONARY
        2. DATA STACK
        3. TERMINAL INPUT BUFFER
        4. RETURN STACK
        5. USER VARIABLE AREA
        6. BUFFERS FOR SCREENS/FILES


 */ 


#       This is the proper way to do it.
#       No memory addresses should be arrived at through equates.
#       However now we must teach the linker to keep the
#       two sections together.

FORTHSIZE       =     . - _main
            .section .dict,"awx",@nobits

INITDP:                 #  It may be that it is not consecutive with TASK.
                        #  Because of bad linking, resulting in a unusable Forth.
        BUFFERSIZE      =  (KBBUF+2*CW)*NBUF

# For more task the TAS area is duplicated; the remainder is the same.
        .SPACE   EM - FORTHSIZE - TAS - BUFFERSIZE 
# INITR1:                         Return stack for next task
#          .SPACE   US              #  For next task, if present. 
# The following framework may be repeated for more tasks.
# It is floating, i.e. it may be shifted up and down, in order
# to have INITR0 at a STACKSIZE boundary. 
        .SPACE   STACKSIZE   # Data stack, only limited if more tasks present.
INITS0:                         # Grows down.
STRTIB: .SPACE   TIBSIZE     # Terminal input buffer.
        .SPACE   STACKSIZE       # Return stack, grows down.
INITR0:                         # Grows down
STRUSA: .SPACE   US              # User area
        .SPACE   STACKSIZE - US   # Spare user area 
        .SPACE   STACKSIZE             # Possibly the framework is shifted up by this.
BUF1:   .SPACE   BUFFERSIZE      # FIRST DISK BUFFER
ACTUAL_EM:  # Differs from EM if code is relocated.

# 
#

# 
        #
 /* 

  MISC. NOTES AND SCATTERED THOUGHTS

- Remember that all the FORTH words in this version are
  upper case letters.  Use <CAPS LOCK> when in FORTH.

# 

- Subscribe to FORTH Dimensions.  It is a valuable source
  of system and application ideas.  Talking with fellow
  FORTH programmers is sure to stir up some exciting ideas.
  Consider joining a FIG chapter.  See the back of FORTH
  Dimensions for more info.

 */ 

# Define the entry point, not valid for auto booting.
        
        # _ORIG


























#





